
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
.. currentmodule:: tau.core.resources

Resource extension for tau core mode.
The resource extension is a special extension that acts like a name dictionary
for actual model names. The official scheme names are 'res' and 'resource'. You
can used both of them indiferently.

The resources module of tau will read one or more resources (usualy files) 
which should contain a dictionary where the keys are resource alias and the values
are the actual model names.

The main class for the extension module is :class:`ResourcesFactory`.

By default, the :class:`ResourcesFactory` will use a resource 
file called 'tau_resources.py'. It will search for this file in the application
directory.

You can add additional resources with the method :meth:`ResourcesFactory.loadResource`.

What is a resource
------------------

A resource can be a python file or a simple python dictionary<str,str>. Below there 
is an example of how register a python dictionary as a resource.

Python resource files
~~~~~~~~~~~~~~~~~~~~~

If a resource is a python file, the factory will recognize resource keys as global
variables. The resource value will be the variable value which must be a string.
For example, the contents of a python resource file could be something like::

    my_device = 'my/tango/device'
    my_state = my_device + '/state'

Afterward, in your code, you could access the Device and Attribute objects by doing::

    >>> import tau
    >>> my_device_obj = tau.Device('res://my_device')
    >>> my_state_obj = tau.Attribute('res://my_state')

The advantage of this is that you can design an application that is independent 
on the real device names.

Note that nothing prevents you from placing any python code in the python resource file.
This means you can have a script that simplifies the contents of the file. Example::

    base = 'my/motor/'
    g = globals()
    
    for i in xrange(256):
        i_str = str(i)
        g['mym"+i_str] = base + i_str

Dictionary resources
~~~~~~~~~~~~~~~~~~~~

Instead of having a complete resource file you can also register a dictionary
as a resource::

    >>> d = { 'my_device' : 'my/tango/device', 'my_state' = 'my/tango/device/state' }
    >>> import tau
    >>> factory = tau.Factory('res')
    >>> factory.loadResource(d)
    >>> my_device_obj = tau.Device('res://my_device')
    >>> my_state_obj = tau.Attribute('res://my_state')
"""

from resfactory import *