
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
Tango extension for tau core mode. 
The Tango extension implements :mod:`tau.core` objects that connect to Tango
objects. The official scheme name is, obviously, 'tango'. As Tango is the default
tau scheme, when specifing a tango model name, the scheme prefix ('tango://') can
be omited.

You should never create objects of tango classes directly. Instead you
should use the :class:`tau.core.TauManager` and :class:`tau.core.TauFactory` APIs
to access all elements.

For example, to get a reference to the Tango attribute my/tango/device/state you
should do something like::

    >>> import tau
    >>> my_state = tau.Attribute('tango://my/tango/device/state')
    
In fact, because the tau default extension is Tango, you could omit the 'tango://'
prefix from the previous code::

    >>> import tau
    >>> my_state = tau.Attribute('my/tango/device/state')

The same is applied to a device::

    >>> import tau
    >>> my_device = tau.Device('my/tango/device')

...to a database::

    >>> import tau
    >>> db = tau.Database('homer:10000')

...and an attribute configuration::

    >>> import tau
    >>> me_state_config = tau.Attribute('my/tango/device/state?configuration')

The way to get access to the Tango factory is::

    >>> import tau
    >>> factory = tau.Factory()
"""

from enums import *
from tangodatabase import TangoDatabaseQuery, TangoDatabase
from tangodevice import TangoDeviceQuery, TangoDevice
from tangofactory import TangoFactory
from tangoattribute import TangoAttribute, TangoStateAttribute
from tangoconfiguration import TangoConfiguration