
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
tangodatabase.py: 
"""

import os
import PyTango
import tau.core

InvalidAlias = "nada"

class TangoDatabaseQuery(tau.core.TauDatabaseQuery):

    # helper class property that stores a reference to the corresponding factory
    _factory = None
    
    def __init__(self,complete_name, parent=None):
        self.db = db
        self.query_name = query
        if params:
            self.params = params.split('?param=')[1:]
            if not self.params:
                self.params = None
        else:
            self.params = None
        self.db_value = None
        host = db.get_db_host()
        port = db.get_db_port()
        complete_name = '%s:%s%s' % (host,port,query)
        if self.params:
            complete_name += params
        
        self.call__init__(tau.core.TauDatabaseQuery, complete_name, None)
        
        self.query = None
        try:
            self.query = getattr(self.db,self.query_name)
        except:
            self.debug("Query '%s' is not available in Database" % query)
            raise
            
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauModel implementation
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-    

    @classmethod
    def factory(cls):
        if cls._factory is None:
            cls._factory = tau.Factory("tango")
        return cls._factory

    
class TangoDatabase(tau.core.TauDatabase):

    def __init__(self,host=None,port=None,parent=None):
        
        pars = host, port
        if host is None or port is None:
            try:
                pars = os.environ.get('TANGO_HOST').split(':')
            except Exception, e:
                pars = ()
                print "Error getting env TANGO_HOST"
        self.dbObj = PyTango.Database(*pars)
        self._dbProxy = None
                
        complete_name = "%s:%s" % (self.dbObj.get_db_host(), self.dbObj.get_db_port())
        self.call__init__(tau.core.TauDatabase, complete_name, parent)

        try:
            self.get_class_for_device('DataBase')
        except:
            # Ok, old tango database.
            self.get_class_for_device = self.__get_class_for_device

    def __get_class_for_device(self, dev_name):
        """Backup method when connecting to tango 5 database device server"""
        # Ok, old tango database.
        db_dev = self.getDbProxy()
        serv_name = db_dev.command_inout("DbGetDeviceInfo",dev_name)[1][3]
        devs = self.get_device_class_list(serv_name)
        dev_name_lower = dev_name.lower()
        for i in xrange(len(devs)/2):
            idx = i*2
            if devs[idx].lower() == dev_name_lower:
                return devs[idx+1]
        return None

    def getDbProxy(self):
        if self._dbProxy is None:
            db_servers = self.get_server_list('Databaseds/*')
            if len(db_servers) >= 1:
                db_dev_name = self.get_device_name(db_servers[0], 'DataBase')
                db_name = self.getFullName()
                self._dbProxy = PyTango.DeviceProxy('%s/%s' %(db_name, db_dev_name[0]))
        return self._dbProxy

    def get_device_attribute_list(self, dev_name, wildcard):
        db_proxy = self.getDbProxy()
        if db_proxy:
            return db_proxy.command_inout("DbGetDeviceAttributeList", [dev_name, wildcard])
        return None
        
    # Export the PyTango.Database interface into this object.
    # This way we can call for example get_attribute_property on an object of this class
    def __getattr__(self, name):
        if not self.dbObj is None: 
            return getattr(self.dbObj,name)
        return None
    
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauModel implementation
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-    

    # helper class property that stores a reference to the corresponding factory
    _factory = None
    
    @classmethod
    def factory(cls):
        if cls._factory is None:
            cls._factory = tau.Factory(scheme='tango')
        return cls._factory
        
    def getValueObj(self,cache=True):
        return self.dbObj
    
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Mandatory methods to overwrite from TauDatabase
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-    
    def getElementAlias(self, full_name):
        try:
            alias = self.getValueObj().get_alias(full_name)
            if alias and alias.lower() == InvalidAlias:
                alias = None 
        except:
            alias = None
        return alias
        
    def getElementFullName(self, alias):
        try:
            return self.getValueObj().get_device_alias(alias)
        except:
            pass
        return None
