
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
a list of helper methods
"""

import sys

def __translate_version_str2int(version_str):
    """Translates a version string in format x[.y[.z[...]]] into a 000000 number"""
    import math
    parts = version_str.split('.')
    i, v, l = 0, 0, len(parts)
    if not l: return v
    while i<3:
        try:
            v += int(parts[i])*int(math.pow(10,(2-i)*2))
            l -= 1
            i += 1
        except ValueError,ve:
            return v
        if not l: return v
    return v
    
    try:
        v += 10000*int(parts[0])
        l -= 1
    except ValueError,ve:
        return v
    if not l: return v
    
    try:
        v += 100*int(parts[1])
        l -= 1
    except ValueError,ve:
        return v
    if not l: return v

    try:
        v += int(parts[0])
        l -= 1
    except ValueError,ve:
        return v
    if not l: return v

def __get_python_version():
    return '.'.join(map(str,sys.version_info[:3]))

def __get_python_version_number():
    pyver_str = __get_python_version()
    if pyver_str is None: return None
    return __translate_version_str2int(pyver_str)

def __get_pytango_version():
    try:
        import PyTango
    except:
        return None
    try:
        return PyTango.Release.version
    except:
        return '0.0.0'

def __get_pytango_version_number():
    tgver_str = __get_pytango_version()
    if tgver_str is None: return None
    return __translate_version_str2int(tgver_str)

def __get_pyqt_version():
    try:
        import PyQt4.Qt
        return PyQt4.Qt.QT_VERSION_STR
    except:
        pass

def __get_pyqt_version_number():
    pyqtver_str = __get_pyqt_version()
    if pyqtver_str is None: return None
    return __translate_version_str2int(pyqtver_str)

def __get_pyqwt_version():
    try:
        import PyQt4.Qwt5
        return PyQt4.Qwt5.QWT_VERSION_STR
    except:
        pass

def __get_pyqwt_version_number():
    pyqwtver_str = __get_pyqwt_version()
    if pyqwtver_str is None: return None
    return __translate_version_str2int(pyqwtver_str)

def __get_qub_version():
    try:
        import Qub4
        return "1.0.0"
    except:
        try:
            import Qub
            return "1.0.0"
        except:
            pass

def __get_qub_version_number():
    qubver_str = __get_qub_version()
    if qubver_str is None: return None
    return __translate_version_str2int(qubver_str)

def __get_qtcontrols_version():
    try:
        import qtcontrols
        return "1.0.0"
    except:
        pass

def __get_qtcontrols_version_number():
    qtcontrols_str = __get_qtcontrols_version()
    return __translate_version_str2int(qtcontrols_str)

def __w(msg):
    sys.stdout.write(msg)
    sys.stdout.flush()

def __wn(msg):
    __w(msg+'\n')

def check_dependencies():
    for msg in _check_dependencies(forlog=False):
        m = msg[1]
        if msg[0] != -1: m = '\t%s' % m
        print m

def log_dependencies():
    from tau.core.utils import Logger
    l = Logger("tau")
    for msg in _check_dependencies(forlog=True):
        if msg[0] != -1:
            l.info(msg[1])

def _check_dependencies(forlog=False):
    """Checks for the required and optional packages of tau"""
    
    if forlog:
        MSG = { 'OK' : '[OK]', 'ERR' : '[ERROR]', 'WARN' : '[WARNING]' }
    else:
        MSG = { 
            'OK'   : "[\033[0;32mOK\033[0m]",
            'ERR'  : "[\033[0;31mERROR\033[0m]",
            'WARN' : "[\033[0;33mWARNING\033[0m]" }
        
    core_requirements = {
    #    module       minimum  recommended 
        "Python"   : ("2.4.0", "2.5.0"),
        "PyTango"  : ("7.1.0", "7.1.0"),
    }
    
    widget_requirements = {
    #    module       minimum  recommended 
        "PyQt"     : ("4.2.0", "4.4.0"),
    }

    widget_optional_requirements = {
    #    module       minimum  recommended 
        "PyQwt"     : ("5.0.0", "5.0.2"),
        "Qub"       : ("1.0.0", "1.0.0"),
        "qtcontrols": ("1.0.0", "1.0.0"),
    }
    
    yield -1, "Checking required dependencies of tau.core..."
    r = core_requirements

    m = "Checking for Python >=%s..." % r["Python"][0]
    minPython, recPython = map(__translate_version_str2int, r["Python"])
    currPython, currPythonStr = __get_python_version_number(), __get_python_version()
    if currPython is None:
        yield 2, "{msg} {ERR} (Not found])".format(msg=m, **MSG)
    elif currPython < minPython:
        yield 1, "{msg} {WARN} (Found {fnd}. Recommended >={rec})".format(msg=m, fnd=currPythonStr, rec=r['Python'][1],**MSG)
    else:
        yield 0, "{msg} {OK} (Found {fnd})".format(msg=m, fnd=currPythonStr, **MSG)

    m = "Checking for PyTango >=%s..." % r["PyTango"][0]
    minPyTango, recPyTango = map(__translate_version_str2int, r["PyTango"])
    currPyTango, currPyTangoStr = __get_pytango_version_number(),__get_pytango_version()
    if currPyTango is None:
        yield 2, "{msg} {ERR} (Not found])".format(msg=m, **MSG)
    elif currPyTango < minPyTango:
        yield 1, "{msg} {WARN} (Found {fnd}. Recommended >={rec})".format(msg=m, fnd=currPyTangoStr, rec=r['PyTango'][1],**MSG)
    else:
        yield 0, "{msg} {OK} (Found {fnd})".format(msg=m, fnd=currPyTangoStr,**MSG)
    
    yield -1, "Checking required dependencies of tau.widget..."
    r = widget_requirements

    m = "Checking for PyQt >=%s..." % r["PyQt"][0]
    minPyQt, recPyQt = map(__translate_version_str2int, r["PyQt"])
    currPyQt, currPyQtStr = __get_pyqt_version_number(), __get_pyqt_version()
    if currPyQt is None:
        yield 2, "{msg} {ERR} (Not found])".format(msg=m, **MSG)
    elif currPyQt < minPyQt:
        yield 1, "{msg} {WARN} (Found {fnd}. Recommended >={rec})".format(msg=m, fnd=currPyQtStr, rec=r['PyQt'][1],**MSG)
    else:
        yield 0, "{msg} {OK} (Found {fnd})".format(msg=m, fnd=currPyQtStr,**MSG)

    yield -1, "Checking OPTIONAL dependencies of tau.widget..."
    r = widget_optional_requirements

    m = "Checking for PyQwt >=%s..." % r["PyQwt"][0]
    minPyQwt, recPyQwt = map(__translate_version_str2int, r["PyQwt"])
    currPyQwt, currPyQwtStr = __get_pyqwt_version_number(), __get_pyqwt_version()
    if currPyQwt is None:
        yield 1, "{msg} {WARN} (Not found])".format(msg=m, **MSG)
    elif currPyQwt < minPyQwt:
        yield 1, "{msg} {WARN} (Found {fnd}. Recommended >={rec})".format(msg=m, fnd=currPyQwtStr, rec=r['PyQwt'][1],**MSG)
    else:
        yield 0, "{msg} {OK} (Found {fnd})".format(msg=m, fnd=currPyQwtStr,**MSG)
    
    m = "Checking for Qub >=%s..." % r["Qub"][0]
    minQub, recQub = map(__translate_version_str2int, r["Qub"])
    currQub, currQubStr = __get_qub_version_number(), __get_qub_version()
    if currQub is None:
        yield 1, "{msg} {WARN} (Not found])".format(msg=m, **MSG)
    elif currQub < minQub:
        yield 1, "{msg} {WARN} (Found {fnd}. Recommended >={rec})".format(msg=m, fnd=currQubStr, rec=r['Qub'][1],**MSG)
    else:
        yield 0, "{msg} {OK} (Found {fnd})".format(msg=m, fnd=currQubStr,**MSG)
    
    m = "Checking for qtcontrols >=%s..." % r["qtcontrols"][0]
    minqtcontrols, recqtcontrols = map(__translate_version_str2int, r["qtcontrols"])
    currqtcontrols, currqtcontrolsStr = __get_qtcontrols_version_number(), __get_qtcontrols_version()
    if currqtcontrols is None:
        yield 1, "{msg} {WARN} (Not found])".format(msg=m, **MSG)
    elif currqtcontrols < minqtcontrols:
        yield 1, "{msg} {WARN} (Found {fnd}. Recommended >={rec})".format(msg=m, fnd=currqtcontrolsStr, rec=r['qtcontrols'][1],**MSG)
    else:
        yield 0, "{msg} {OK} (Found {fnd})".format(msg=m, fnd=currqtcontrolsStr,**MSG)
    

def Manager():
    """Returns the one and only TauManager
    
    It is a shortcut to::

        import tau.core
        manager = tau.core.TauManager()
    
    :return: the TauManager
    :rtype: :class:`tau.core.TauManager`
    
    .. seealso:: :class:`tau.core.TauManager`"""
    import taumanager
    return taumanager.TauManager()

def Factory(scheme=None):
    """Returns the one and only Factory for the given scheme
    
    It is a shortcut to::

        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory(scheme)

    :param scheme: a string representing the scheme. Default value is None meaning ``tango`` scheme
    :type scheme: str
    :return: a tau factory
    :rtype: :class:`tau.core.TauFactory`"""
    return Manager().getFactory(scheme=scheme)()

def Device(device_name):
    """Returns the tau device for the given device name

    It is a shortcut to::

        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory()
        device  = factory.getDevice(device_name)
    
    :param device_name: the device name
    :type device_name: str
    :return: a tau device
    :rtype: :class:`tau.core.TauDevice`"""
    return Factory().getDevice(device_name)

def Attribute(dev_or_attr_name, attr_name=None):
    """Returns the tau attribute for either the pair (device name, attribute name) 
    or full attribute name
    
    - Attribute(full_attribute_name)
    - Attribute(device_name, attribute_name)

    It is a shortcut to::

        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory()
        attribute  = factory.getAttribute(full_attribute_name)
    
    or::
    
        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory()
        device  = factory.getDevice(device_name)
        attribute = device.getAttribute(attribute_name)
    
    :param dev_or_attr_name: the device name or full attribute name
    :type dev_or_attr_name: str or TauDevice
    :param attr_name: attribute name
    :type attr_name: str
    :return: a tau attribute
    :rtype: :class:`tau.core.TauAttribute`"""
    import types
    if attr_name is None:
        return Factory().getAttribute(dev_or_attr_name)
    else:
        if type(dev_or_attr_name) in types.StringTypes:
            dev = Device(dev_or_attr_name)
        else:
            dev = dev_or_attr_name
        return dev.getAttribute(attr_name)

def Configuration(attr_or_conf_name, conf_name=None):
    """Returns the tau configuration for either the pair (attribute name, conf name) 
    or full conf name
    
    - Configuration(full_conf_name)
    - Configuration(attribute_name, conf_name)
    
    It is a shortcut to::

        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory()
        conf  = factory.getConfiguration(attr_or_conf_name)

    or::
    
        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory()
        attribute  = factory.getAttribute(attribute_name)
        conf = attribute.getConfig(conf_name)
        
    :param attr_or_conf_name: the full attribute name or full conf name
    :type attr_or_conf_name: str
    :param conf_name: conf name
    :type conf_name: str or None
    :return: a tau configuration
    :rtype: :class:`tau.core.TauConfiguration`"""
    if conf_name is None:
        return Factory().getConfiguration(attr_or_conf_name)
    else:
        return Attribute(attr_or_conf_name).getConfig(conf_name)

def Database(db_name=None):
    """Returns the tau database
    
    It is a shortcut to::

        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory()
        db  = factory.getDatabase(db_name)
        
    :param db_name: database name. If None (default) it will use the TANGO_HOST value
    :type db_name: str or None
    :return: a tau database
    :rtype: :class:`tau.core.TauDatabase`"""
    return Factory().getDatabase(db_name)

def Object(klass, name):
    """Returns an tau object of given class for the given name
    
    It is a shortcut to::

        import tau.core
        manager = tau.core.TauManager()
        factory = manager.getFactory()
        obj  = factory.getObject(klass, name)

    :param klass: a tau model subclass (TauDevice, for example)
    :type klass: class TauModel
    :param name: the full object name
    :type name: str
    :return: a tau object
    :rtype: :class:`tau.core.TauModel`"""
    return Factory().getObject(klass, name)

import utils

Critical = utils.Logger.Critical
Error    = utils.Logger.Error
Warning  = utils.Logger.Warning
Info     = utils.Logger.Info
Debug    = utils.Logger.Debug
Trace    = utils.Logger.Trace
    
setLogLevel = utils.Logger.setLogLevel
setLogFormat = utils.Logger.setLogFormat
resetLogLevel = utils.Logger.resetLogLevel
resetLogFormat = utils.Logger.resetLogFormat

enableLogOutput = utils.Logger.enableLogOutput
disableLogOutput = utils.Logger.disableLogOutput

def log(level, msg, *args, **kw):
    return utils.Logger.getRootLog().log(level, msg, *args, **kw)

def trace(msg, *args, **kw):
    return log(Trace, msg, *args, **kw)

def debug(msg, *args, **kw):
    return utils.Logger.getRootLog().debug(msg, *args, **kw)

def info(msg, *args, **kw):
    return utils.Logger.getRootLog().info(msg, *args, **kw)

def warning(msg, *args, **kw):
    return utils.Logger.getRootLog().warning(msg, *args, **kw)

def error(msg, *args, **kw):
    return utils.Logger.getRootLog().error(msg, *args, **kw)

def critical(msg, *args, **kw):
    return utils.Logger.getRootLog().critical(msg, *args, **kw)

