
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
taulistener.py: 
"""

import PyTango

import utils

class TauListener(utils.Logger):
    """ TauListener Interface"""
    
    def __init__(self, name, parent=None):
        self.call__init__(utils.Logger, name, parent)
    
    def eventReceived(self, src, type, evt_value):
        """ Method to implement the event notification"""
        pass

    def attributeList(self):
        """ Method to return the attributes of the widget"""
        return []


class TauExceptionListener(object):
    """Class for handling ConnectionFailed, DevFailed and TauException exceptions.

    The TauExceptionListeners are invoked when the """

    def connectionFailed(self, ex):
        self._printException(ex)

    def devFailed(self, exception):
        self._printException(self)

    def exceptionReceived(self, exception):
        if isinstance(exception, PyTango.ConnectionFailed):
            self.connectionFailed(exception)

        elif isinstance(exception, PyTango.DevFailed):
            self.devFailed(exception)

        else:
            self._printException(self, exception)

    def _printException(self, exception):
        print self.__class__.__name__, "received", exception.__class__.__name__, str(exception)

