
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
singleton.py: 
"""

class Singleton(object):
    """ This class allows Singleton objects
    The __new__ method is overriden to force Singleton behaviour.
    The Singleton is created for the lowest subClass.
    @warning although __new__ is overriden __init__ is still being called for each instance=Singleton()
    """
    
    ## Singleton object
    _the_instance = None
    
    def __new__(cls, *p, **k):
        #srubio: added type checking
        if cls != type(cls._the_instance):
            cls._the_instance = object.__new__(cls)
            
            #srubio: added init_single check
            if 'init_single' in cls.__dict__: 
                cls._the_instance.init_single(*p,**k)
            else: cls._the_instance.init(*p,**k)
        return cls._the_instance
     
    def init(self, *p,**k):
        pass

    
