
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
LCD.py: 
"""

from PyQt4 import QtCore, QtGui, Qt

import colors
import taubase

class TauLCDValue(QtGui.QLCDNumber, taubase.TauBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent = None, designMode = False):
        name = self.__class__.__name__

        self.call__init__wo_kw(QtGui.QLCDNumber, parent)
        self.call__init__(taubase.TauBaseWidget, name, designMode=designMode)
        
        self.defineStyle()
    
    def defineStyle(self):
        self.setFrameShape(QtGui.QFrame.Panel)
        self.setFrameShadow(QtGui.QFrame.Raised)
        self.setSmallDecimalPoint(True)
        self.setMode(QtGui.QLCDNumber.Dec)
        self.setSegmentStyle(QtGui.QLCDNumber.Filled)
        self.updateStyle()
    
    # The minimum size of the widget (a limit for the user)
    def minimumSizeHint(self):
        return QtCore.QSize(20, 20)
    
    # The default size of the widget
    def sizeHint(self):
        return QtCore.QSize(60, 22)
    
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget over writing
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    
    def getModelClass(self):
        import tau.core
        return tau.core.TauAttribute
        
    def isReadOnly(self):
        return True
        
    def getCharsToDisplayFromFormat(self,fmt):
        n = ''
        for c in fmt:
            if c == '.':
                break
            if c.isdigit():
                n += c
        return int(n)
    
    def updateStyle(self):
        n = 6
        model = self.getModelObj()
        if not model is None:
            fmt = model.getFormat()
            if fmt:
                n = self.getCharsToDisplayFromFormat(fmt)
        
        if n != self.numDigits():
            self.setNumDigits(n)
        
        self.setAutoFillBackground(self.getShowQuality())
        
        if self.getShowQuality():
            v = self.getModelValueObj()
            quality = None
            if v:
                quality = v.quality
            bg_brush, fg_brush = colors.QT_ATTRIBUTE_QUALITY_PALETTE.qbrush(quality)
            palette = self.palette()
            palette.setBrush(QtGui.QPalette.Window,bg_brush)
            palette.setBrush(QtGui.QPalette.WindowText,fg_brush)
        else:
            palette = self.palette()
            palette.setBrush(QtGui.QPalette.Window, QtCore.Qt.white)
            palette.setBrush(QtGui.QPalette.WindowText, QtCore.Qt.black)
        self.update()
                        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    model = QtCore.pyqtProperty("QString", taubase.TauBaseWidget.getModel, 
                                taubase.TauBaseWidget.setModel, 
                                taubase.TauBaseWidget.resetModel) 
                                
    useParentModel = QtCore.pyqtProperty("bool", 
                                         taubase.TauBaseWidget.getUseParentModel, 
                                         taubase.TauBaseWidget.setUseParentModel, 
                                         taubase.TauBaseWidget.resetUseParentModel)
    
    showQuality = QtCore.pyqtProperty("bool", 
                                      taubase.TauBaseWidget.getShowQuality, 
                                      taubase.TauBaseWidget.setShowQuality,
                                      taubase.TauBaseWidget.resetShowQuality)
