
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
SaveDataDir.py: 
"""


from PyQt4 import QtCore, QtGui, Qt
from tau.widget import TauBaseWidget
from tau.widget import SaveDataDir_ui
import traceback

class SaveDataDir(QtGui.QWidget, TauBaseWidget):
    
    __pyqtSignals__ = ("stored()","mistake()")
    
    def __init__(self,name = None,parent=None, designMode = False):
        try:
            self.__name = name.__name__
        except:
            self.__name = "SaveDataDir"
        self.call__init__wo_kw(QtGui.QWidget, parent)
        self.call__init__(TauBaseWidget, self.__name, parent, designMode)
        try:
            self.debug("[%s]__init__()"%self.__name)
            self.ui = SaveDataDir_ui.Ui_SaveDataDir()
            self.ui.setupUi(self)
            QtCore.QObject.connect(self.ui.OverWrite,QtCore.SIGNAL("stateChanged(int)"),self.setOverWrite)
            self.__data = None
            self.__overWrite = False
        except Exception, e:
            self.warning("[%s]__init__(): Exception!"%self.__name)
            self.traceback()
    #__init__
    
    def getHost(self):
        return self.ui.HostEdit.Text()
    #getHost
    
    def setHost(self,host):
        self.ui.HostEdit.setText(host)
    #setHost
    
    def getUser(self):
        return self.ui.UserEdit.Text()
    #getUser
    
    def setUser(self,user):
        self.ui.UserEdit.setText(user)
    #setUser
    
    def getPasswd(self):
        return self.ui.PasswdEdit.Text()
    #getPasswd
    
    def setPasswd(self,password):
        self.ui.PasswdEdit.setText(password)
    #setPasswd
    
    def getDirectory(self):
        return self.ui.DirectoryEdit.Text()
    #getDirectory
    
    def setDirectory(self,directory):
        self.ui.DirectoryEdit.setText(directory)
    #setDirectory
    
    def getName(self):
        return self.ui.NameEdit.Text()
    #getName
    
    def setName(self,name):
        self.ui.NameEdit.setText(name)
    #setName
    
    def getType(self):
        return self.ui.TypeEdit.currentText()
    #getType
    
    def setType(self,type):
        ind = self.ui.TypeEdit.findText(type)# ?? Qt.MatchExactly
        if not ind == -1:
            self.ui.TypeEdit.setCurrentIndex(ind)
    #setType
    
    def getDataStore(self):
        return self.__data
    #getDataStore
    
    def setDataStore(self,data):
        self.__data = data
    #setDataStore
    
    def getOverWrite(self):
        #return self.ui.OverWrite.isChecked() #no events
        return self.__overWrite
    #getOverWrite
    def setOverWrite(self,check):
        #self.ui.OverWrite.setCheckState(Qt.CheckState(check)) #no events
        self.__overWrite = bool(check)
    
    def doDataStore(self):
        try:
            #TODO login to a remote server
            # if not exist, create dir
            dir = self.getDirectory()
            if not os.path.isdir(dir):
                os.mkdir(dir,0750)
            # create the file and drop the data in. Mode depends on the checkstate
            if self.__overWrite:
                f = open(dir+"/"+self.getName()+"."+self.getType(),"w",0640)
            else:
                f = open(dir+"/"+self.getName()+"."+self.getType(),"a",0640)
            f.write(self.__data)
            f.close()
            self.emit(QtCore.SIGNAL("stored()"))
        except Exception, e:
            self.warning("[%s]doDataStore(): Exception! %s"%(self.__name,e))
            self.traceback()
            self.emit(QtCore.SIGNAL("mistake()"))
    #doDataStore
#SaveDataDir
