
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
This package contains a collection of Qt based widgets designed to interact with
tau models. The widgets are generic in the sence that they do not assume any
behavior associated with a specific HW device. They intend to represent only
abstract model data.
"""

import release as Release
import traceback

from tau.core.utils import Logger

_logger = Logger(__name__)

from colors import QT_DEVICE_STATE_PALETTE, QT_ATTRIBUTE_QUALITY_PALETTE
from taubase import TauBaseWidget, TauBaseComponent
from labels import *
from scalaredit import *
from combobox import TauValueComboBox, TauQueryComboBox, TauAttrListComboBox
from LCD import TauLCDValue
from leds import TauStateLed
from leds import TauBoolLed
from buttons import TauLauncherButton, TauCommandButton
from containers import TauBaseContainer, TauFrame, TauGroupBox
from containers import TauScrollArea, TauWidget
from mainwindow import TauMainWindow
from graphics import TauGraphicsScene
from graphics import TauGraphicsItem, TauGraphicsAttributeItem
from graphics import TauGraphicsStateItem, TauGroupStateItem
from graphics import TauEllipseStateItem, TauRectStateItem, TauPolygonStateItem
from graphics import TauTextStateItem, TauTextAttributeItem
from graphics import AbstractGraphicsFactory
from graphicsview import TauGraphicsView#, TauJDrawSynopticsView
from led import *
from limitswitch import TauLimitSwitch
from actionfactory import *
from widgetfactory import TauWidgetFactory
from buttonbox import TauButtonBox
from tauvalue import TauValue, TauValuesFrame
from forms import TauForm, TauAttrForm, TauCommandsForm
from valuestable import TauValuesTable
from logger import TauLoggerTable, TauLoggerWidget
from configuration import BaseConfigurableClass, TauBaseConfigurableClass

try:
    from synoptic import TauJDrawSynopticsView
except:
    _logger.info('failed to import synoptic')

try: 
    from class_table import TauClassTable
except:
    _logger.info('failed to import class_table')

try: 
    from props_table import TauPropTable
except:
    _logger.info('failed to import props_table')
    _logger.traceback()

try: 
    from devs_tree import TauDevTree
except:
    _logger.info('failed to import devs_tree: %s'%traceback.format_exc())

try:
    from gauges import *
except:
    _logger.info('failed to import gauges')

try: 
    from tauLogo import TauLogo
except:
    _logger.warning('failed to import tauLogo')

try: 
    from taugrid import TauGrid
except:
    _logger.info('failed to import taugrid')


