
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
actionfactory.py: 
"""


__all__ = ["ActionFactory"]

import imp, os
import xml.dom.minidom

from PyQt4 import QtCore, QtGui, Qt
import tau.core.utils as utils

import actions

class ActionFactory(utils.Singleton, utils.Logger):
    """A Singleton class designed to provide Action related objects."""

    def __init__(self):
        """ Initialization. Nothing to be done here for now."""
        pass

    def init(self, *args):
        """Singleton instance initialization."""
        self.call__init__(utils.Logger,'ActionFactory')
        self.actions = self.__getActions()
        self.menus = self.__getMenus()
    
    def __getClasses(self, super_class):
        ret = {}
        klass_type = type(super_class)
        for name in dir(actions):
            klass = getattr(actions, name)
            if klass == super_class:
                continue
            if (type(klass) == klass_type and issubclass(klass, super_class)):
                ret[klass.menuID] = klass
        return ret
    
    def __getActions(self):
        """Calculates the map of existing action classes"""
        return self.__getClasses(actions.TauAction)

    def __getMenus(self):
        """Calculates the map of existing menu classes""" 
        return self.__getClasses(actions.TauMenu)

    def getActions(self):
        return self.actions
    
    def getMenus(self):
        return self.menus

    def getNewAction(self, widget, id):
        klass = self.actions.get(id)
        if klass is None:
            return None
        return klass(widget)

    def getNewMenu(self, widget, data):
        doc = xml.dom.minidom.parseString(data)
        m_node = doc.childNodes[0]
        return self.buildMenu(widget, m_node)
    
    def buildAction(self, widget, a_node):
        if not a_node.hasAttribute('class'):
            return None
        
        id = a_node.getAttribute('class')
        action = self.getNewAction(widget, id)
        
        # if node has alternative label, display it instead
        if a_node.hasAttribute('label'):
            action.setText(a_node.getAttribute('label'))
            
        if a_node.hasAttribute('checkable'):
            action.setCheckable(bool(a_node.getAttribute('checkable')))
        
        if a_node.hasAttribute('icon'):
            icon = a_node.getAttribute('icon')
            #TODO
            #action.setIcon(icon)
        return action
    
    def buildMenu(self, widget, m_node):
        menu = None 
        if m_node.hasAttribute('class'):
            klass = self.menus.get(m_node.getAttribute('class'))
            if klass is None:
                return None
            menu = klass(widget)
        else:
            menu = actions.TauMenu(widget)
            menu.buildFromXML(m_node)
        return menu
            