
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
class_table.py: 
"""


from PyQt4 import QtCore, QtGui, Qt, Qwt5
from tau.widget import TauBaseWidget, taubase
import tau.core
import traceback

class TauClassTable(QtGui.QTableWidget, TauBaseWidget):
    ''' This widget will show a list of devices using table. '''
    __pyqtSignals__ = ("modelChanged(const QString &)","deviceSelected(QString)",) 
    
    def __init__(self, parent=None, designMode = False):
        try:
            name = "TauClassTable"
            self._useParentModel = True
            self._localModel = ''
            self.call__init__wo_kw(QtGui.QTableWidget, parent)
            self.call__init__(taubase.TauBaseWidget, name, designMode=designMode)
            self.setObjectName(name)
            #self.setModelCheck('controls01:10000') 
            self.defineStyle()
            self.__classes=""   
            #Signal
            QtCore.QObject.connect(self,QtCore.SIGNAL("cellClicked(int,int)"),self.deviceClicked)
        except Exception,e:
            self.traceback()

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # My methods 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def deviceClicked(self):
        '''QSIGNAL: this method is used to emit deviceSelected(QString) signal; it can be connected to other widgets. '''
        item = QtGui.QTableWidgetItem()
        row = self.currentRow()
        item = self.item(row,1)
        self.device_name = item.text()
        self.dev = str(self.device_name)
        #Signal 
        self.emit(QtCore.SIGNAL("deviceSelected(QString)"), QtCore.QString(self.dev))		

    #@QtCore.pyqtSignature("setDictionary(QString)")
    #def setDictionary(self,classes):
        #'''
        #This method build the dictionary with names of serwers and devices depending on the given classes in QTProperty or in another widget, 
        #because this method can be used to connect TauClassTable with another widget such as TauLineValueEdit.
        #'''
        #self.db = self.getModelObj()
        #classes = str(classes).split(',')
        #self.list_serv = {}
        #self.setRowCount(1)
        #index = 0
        #for my_class in classes:
            #self.list_serv = self.get_servers_for_class(my_class)
            #for my_server in self.list_serv:
                #self.list_dev = self.get_devices_for_server(my_server, my_class)
                #for my_device in self.list_dev:
                    #self.setValue(my_server,index,0)
                    #self.setValue(my_device,index,1)
                    #index = index + 1
                    #self.setRowCount(index+1)
        #self.setRowCount(index)
        #self.horizontalHeader().setResizeMode(QtGui.QHeaderView.Stretch)

    @QtCore.pyqtSignature("setTable(QString)")
    def setTable(self,classes):
        '''
        This method show a list of servers and devices depending on the given classes in QTProperty or in another widget, 
        because this method can be used to connect TauClassTable with another widget such as TauLineValueEdit.
        '''
        self.db = self.getModelObj()
        classes = type(classes) in (list,set,tuple) and classes or str(classes).split(',')
        self.list_serv = {}
        self.setRowCount(1)	
        index = 0
        for my_class in classes:
            self.list_serv = self.get_servers_for_class(my_class)
            for my_server in self.list_serv:
                self.list_dev = self.get_devices_for_server(my_server, my_class)
                for my_device in self.list_dev:
                    self.setValue(my_server,index,0)
                    self.setValue(my_device,index,1)
                    index = index + 1
                    self.setRowCount(index+1)
        self.setRowCount(index)
        self.horizontalHeader().setResizeMode(QtGui.QHeaderView.Stretch)

    def setValue(self,value,i,j):
        item = QtGui.QTableWidgetItem()
        item.setText(QtGui.QApplication.translate("PLCTabWidget", value, None, QtGui.QApplication.UnicodeUTF8))
        self.setItem(i,j,item)

    def defineStyle(self):
        self.setColumnCount(2)
        self.setRowCount(0)
        self.setGeometry(QtCore.QRect(0,0,290,500))
        headerItem = QtGui.QTableWidgetItem()
        headerItem.setText(QtGui.QApplication.translate("PLCTabWidget", "Server Name", None, QtGui.QApplication.UnicodeUTF8))
        self.setHorizontalHeaderItem(0,headerItem)
        headerItem1 = QtGui.QTableWidgetItem()
        headerItem1.setText(QtGui.QApplication.translate("PLCTabWidget", "Device Name", None, QtGui.QApplication.UnicodeUTF8))
        self.setHorizontalHeaderItem(1,headerItem1)	
        self.horizontalHeader().setResizeMode(QtGui.QHeaderView.Stretch)
        #self.updateStyle()

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Methods  for database commands
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def get_servers_for_class(self, class_name):
        executable_name = class_name
        instances = self.db.get_instance_name_list(class_name)
        return [class_name+'/'+instance for instance in instances]

    def get_devices_for_server(self, server_name, class_name=''):
        devslist = self.db.get_device_class_list(server_name)
        return [dev for dev in devslist if '/' in dev and not dev.startswith('dserver')]

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Methods  for QTProperty "classes"
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def setClasses(self,classes):
        self.__classes = str(classes)
        self.setTable(self.__classes)

    def getClasses(self):
        return self.__classes

    def resetClasses(self):
        self.__classes=""
        self.setTable(self.__classes)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget over writing methods 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def updateStyle(self):
        pass

    def sizeHint(self):
        return QtGui.QTableWidget.sizeHint(self)

    def minimumSizeHint(self):
        return QtGui.QTableWidget.minimumSizeHint(self)

    def getModelClass(self):
        return tau.core.TauDatabase

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    model = QtCore.pyqtProperty("QString", taubase.TauBaseWidget.getModel, 
                                taubase.TauBaseWidget.setModel, 
                                taubase.TauBaseWidget.resetModel)

    classes = QtCore.pyqtProperty("QString", getClasses, setClasses, resetClasses)
								
    useParentModel = QtCore.pyqtProperty("bool", 
                                         taubase.TauBaseWidget.getUseParentModel, 
                                         taubase.TauBaseWidget.setUseParentModel,
                                         taubase.TauBaseWidget.resetUseParentModel)
                                         
if __name__ == "__main__":
    import sys,os
    app = Qt.QApplication(sys.argv)
    
    args=sys.argv[1:]
    if not args: args = ['Starter']
    form = TauClassTable()
    form.setModel(os.getenv('TANGO_HOST'))
    form.setTable(args)
    form.show()
    sys.exit(app.exec_())                                         
 