
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
colors.py: 
"""

from PyQt4 import QtGui,QtCore

import tau.core.utils as utils

class QtColorPalette(utils.ColorPalette):

    def __init__(self, dat, int_decoder_dict):
        utils.ColorPalette.__init__(self, dat, int_decoder_dict)
        self._qcolor_cache_fg = dict()
        self._qcolor_cache_bg = dict()
        self._qbrush_cache_fg = dict()
        self._qbrush_cache_bg = dict()
        self._qvariant_cache_fg = dict()
        self._qvariant_cache_bg = dict()

    def qbrush(self, stoq):
        #print stoq
        """Returns the brush for the specified state or quality"""
        name = self._decoder(stoq)

        f = self._qbrush_cache_fg
        b = self._qbrush_cache_bg
        if not f.has_key(name):
            f[name] = QtGui.QBrush(self.qcolor(stoq)[1])

        if not b.has_key(name):
            b[name] = QtGui.QBrush(self.qcolor(stoq)[0])
            if name == 'None':
                b[name].setStyle(QtCore.Qt.BDiagPattern)

        return ( b[name], f[name] )

    def qcolor(self, stoq):
        """Returns the color for the specified state or quality"""
        name = self._decoder(stoq)

        f = self._qcolor_cache_fg
        b = self._qcolor_cache_bg
        if not f.has_key(name):
            f[name] = QtGui.QColor(self.number(name, True))

        if not b.has_key(name):
            b[name] = QtGui.QColor(self.number(name))

        return ( b[name], f[name] )

    def qvariant(self, stoq):
        """Returns the color for the specified state or quality"""
        name = self._decoder(stoq)

        f = self._qvariant_cache_fg
        b = self._qvariant_cache_bg
        if not f.has_key(name):
            (back,fore) = self.qcolor(name)
            f[name] = QtCore.QVariant(fore)
            b[name] = QtCore.QVariant(back)

        return ( b[name], f[name] )

import PyTango

QT_DEVICE_STATE_PALETTE = QtColorPalette(utils.DEVICE_STATE_DATA, PyTango.DevState)
QT_ATTRIBUTE_QUALITY_PALETTE = QtColorPalette(utils.ATTRIBUTE_QUALITY_DATA, PyTango.AttrQuality)
