
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
containersplugin.py: 
"""

from PyQt4 import Qt

import tauplugin
import tau.core.utils

try:
    from tau.widget import TauJDrawSynopticsView
    __SUPER_CLASS = tauplugin.TauWidgetPlugin
except:
    __SUPER_CLASS = object
    l = tau.core.utils.Logger("Designer")
    l.info("TauJDrawSynopticsView will not be available")

class TauJDrawSynopticsViewPlugin(__SUPER_CLASS):

    """TauJDrawSynopticsViewPlugin(tauplugin.TauWidgetPlugin)
    
    Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin via a PyQt-specific custom plugin class.
    """

    def getWidgetClass(self):
        return TauJDrawSynopticsView
        
    def createWidget(self, parent):
        widget = tauplugin.TauWidgetPlugin.createWidget(self, parent)
        bg_brush = Qt.QBrush(Qt.QPixmap(':/designer/tau.png'))
        scene = Qt.QGraphicsScene(parent)
        scene.setBackgroundBrush(bg_brush)
        widget.setScene(scene)
        return widget

    def getIconName(self):
        return 'graphicsview.png'

    def isContainer(self):
        return True

try:
    from tau.widget import TauGraphicsView
    __SUPER_CLASS = tauplugin.TauWidgetPlugin
except:
    __SUPER_CLASS = object
    l = tau.core.utils.Logger("Designer")
    l.info("TauGraphicsView will not be available")
    
class TauGraphicsViewPlugin(__SUPER_CLASS):

    """TauGraphicsViewPlugin(tauplugin.TauWidgetPlugin)
    
    Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin via a PyQt-specific custom plugin class.
    """

    def getWidgetClass(self):
        return TauGraphicsView
        
    def createWidget(self, parent):
        widget = tauplugin.TauWidgetPlugin.createWidget(self, parent)
        bg_brush = Qt.QBrush(Qt.QPixmap(':/designer/tau.png'))
        scene = Qt.QGraphicsScene(parent)
        scene.setBackgroundBrush(bg_brush)
        widget.setScene(scene)
        return widget

    def getIconName(self):
        return 'graphicsview.png'

    def isContainer(self):
        return True
    
try:
    from tau.widget import TauFrame
    __SUPER_CLASS = tauplugin.TauWidgetPlugin
except:
    __SUPER_CLASS = object
    l = tau.core.utils.Logger("Designer")
    l.info("TauFrame will not be available")
    
class TauFramePlugin(__SUPER_CLASS):

    """TauFramePlugin(tauplugin.TauWidgetPlugin)
    
    Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin via a PyQt-specific custom plugin class.
    """

    def getWidgetClass(self):
        return TauFrame

    def getIconName(self):
        return 'frame.png'

    def isContainer(self):
        return True
    
try:
    from tau.widget import TauGroupBox
    __SUPER_CLASS = tauplugin.TauWidgetPlugin
except:
    __SUPER_CLASS = object
    l = tau.core.utils.Logger("Designer")
    l.info("TauGroupBox will not be available")
    
class TauGroupBoxPlugin(__SUPER_CLASS):

    """TauGroupBoxPlugin(tauplugin.TauWidgetPlugin)
    
    Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin via a PyQt-specific custom plugin class.
    """

    def getWidgetClass(self):
        return TauGroupBox

    def getIconName(self):
        return 'groupbox.png'

    def isContainer(self):
        return True
    

