
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
RawDataChooser.py:  widget for importing RawData (from file or from a function)
"""


import sys
from PyQt4 import QtCore, QtGui
import numpy
from tau.core.utils import SafeEvaluator

from ui import ui_RawDataChooser 

class RawDataWidget(QtGui.QWidget, ui_RawDataChooser.Ui_RawDataChooser ):
    
    def __init__(self, parent=None):
        super(RawDataWidget,self).__init__(parent)
        self.setupUi(self)

        #connecttions
        self.connect(self.openFilesBT,QtCore.SIGNAL( "clicked()"), self.onOpenFilesButtonClicked)
        self.connect(self.addCurveBT,QtCore.SIGNAL( "clicked()"), self.onAddCurveButtonClicked)
                
        #set validators in LE's
        self.xFromLE.setValidator(QtGui.QDoubleValidator(self))
        self.xToLE.setValidator(QtGui.QDoubleValidator(self))
        self.xStepLE.setValidator(QtGui.QDoubleValidator(self))
        
        
    def onOpenFilesButtonClicked(self):
        """ Emit a ReadFromFiles signal with the selected xcol and skiprows as parameters"""
        xcol = self.xcolSB.value()
        if xcol == self.xcolSB.minimum(): xcol = None
        skiprows = self.headerSB.value()
        self.emit(QtCore.SIGNAL("ReadFromFiles"),xcol, skiprows)
        
    def onAddCurveButtonClicked(self):
        """ Emit a AddCurve signal with a rawdata dictionary as a parameter.
        The rawdata dictionary is prepared from the from the GUI's selection."""
        rawdata = {}
        if self.xRangeRB.isChecked():
            rawdata['x']=numpy.arange(float(self.xFromLE.text()), float(self.xToLE.text()), float(self.xStepLE.text()) )
        else:
            sev=SafeEvaluator() 
            try: 
                rawdata['x']=sev.eval(str(self.xValuesLE.text()))
            except:
                QtGui.QMessageBox.warning(self, 'Invalid x values' 'Cannot interpret the x values.\n Use Python expressions like "[1, 3 , 67]" or "arange(100)")')
                return
        rawdata['f(x)']=str(self.f_xLE.text())
        self.emit(QtCore.SIGNAL("AddCurve"),rawdata)
        
if __name__ == "__main__":
    import sys 
    app = QApplication(sys.argv)
    form = RawDataWidget()
    form.show()
    sys.exit(app.exec_())



