
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
qcontainers.py: 
"""

import copy
import sys

from PyQt4 import QtGui, QtCore

class QPanel(QtGui.QWidget):
    
    defaultTitleStyle = {
        'start_color' : 'rgb(60, 150, 255)',
        'stop_color'  : 'rgb(0, 85, 227)',
        'font_color'  : 'white',
    }
    
    titleBarStyle = """
.QGroupBox {{
    padding: 2px 2px;
    border-width: 1px;
    border-style: solid;
    border-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                  stop: 0 {start_color}, stop: 1 {stop_color});
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
    border-bottom-left-radius: 0px;
    border-bottom-right-radius: 0px;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                      stop: 0 {start_color}, stop: 1 {stop_color});
}}
.QGroupBox::title {{
    subcontrol-origin: content;
    subcontrol-position: center left;
    
    font-weight: normal;
    color: {font_color};
}}"""
    
    defaultContentStyle = {}
    
    contentBarStyle = """
.QFrame {{
    border-width: 1px;
    border-style: solid;
    border-color: rgb(0, 85,227);
    border-top-left-radius: 0px;
    border-top-right-radius: 0px;
    border-bottom-left-radius: 5px;
    border-bottom-right-radius: 5px;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 1.0, y2: 1.0,
                                      stop: 0 #E0E0E0, stop: 1 #FFFFFF);
}}"""
    
    def __init__(self, title, parent=None):
        QtGui.QWidget.__init__(self, parent)
        l = QtGui.QVBoxLayout()
        l.setSpacing(0)
        self.setLayout(l)
        titleBar = self._buildTitleBar(title)
        l.addWidget(titleBar)
        self.content = QtGui.QFrame()
        l.addWidget(self.content)
        self.updateStyle()
    
    def _buildTitleBar(self, title):
        self.titleBar = QtGui.QGroupBox('')
        l = QtGui.QHBoxLayout()
        self.titleBar.setLayout(l)
        style = QtGui.QApplication.instance().style()
        self.titleIcon = QtGui.QLabel('h')
        self.titleIcon.setPixmap(style.standardIcon(QtGui.QStyle.SP_ArrowRight).pixmap(16,16))
        self.titleLabel = QtGui.QLabel(title)
        self.upDownIcon = QtGui.QLabel('i')
        self.upDownIcon.setPixmap(style.standardIcon(QtGui.QStyle.SP_ArrowDown).pixmap(16,16))
        l.addWidget(self.titleIcon)
        l.addWidget(self.titleLabel)
        l.addWidget(self.upDownIcon)
        return self.titleBar
    
    def getTitleStyle(self, **kwargs):
        titleStyle = copy.copy(QPanel.defaultTitleStyle)
        titleStyle.update(kwargs)
        return QPanel.titleBarStyle.format(**titleStyle)

    def getContentStyle(self, **kwargs):
        contentStyle = copy.copy(QPanel.defaultContentStyle)
        contentStyle.update(kwargs)
        return QPanel.contentBarStyle.format(**contentStyle)
    
    def updateStyle(self):
        titleStyle = self.getTitleStyle()
        contentStyle = self.getContentStyle()
        self.titleBar.setStyleSheet(titleStyle)
        self.content.setStyleSheet(contentStyle)

def main():
    app = QtGui.QApplication([])
    w = QtGui.QWidget()
    panel = QPanel("Database", w)
    l = QtGui.QFormLayout()
    panel.content.setLayout(l)
    l.addRow("&Host", QtGui.QLineEdit())
    l.addRow("&Port", QtGui.QLineEdit())

    w.show()
    w.adjustSize()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()
    