
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
motor.py: 
"""


from PyQt4 import QtCore, QtGui
from tau.widget import TauBaseWidget
from ui_taumotorh import Ui_TauMotorH
from ui_taumotorh2 import Ui_TauMotorH2
from ui_taumotorv import Ui_TauMotorV
from ui_taumotorv2 import Ui_TauMotorV2


def showDialogConfigureMotor(parent):
    Dialog = QtGui.QDialog(parent)
    Dialog.resize((QtCore.QSize(QtCore.QRect(0,0,310,309).size()).expandedTo(Dialog.minimumSizeHint())))
    motorV2 = TauMotorV2(Dialog)
    motorV2.setModel(parent.model)
    motorV2.setGeometry(QtCore.QRect(10,10,291,291))
    Dialog.show()


class TauMotorH(QtGui.QWidget, TauBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent = None, designMode = False):
        self.call__init__wo_kw(QtGui.QWidget, parent)
        self.call__init__(TauBaseWidget, str(self.objectName()), designMode=designMode)
        self.ui = Ui_TauMotorH()
        self.ui.setupUi(self)
        QtCore.QObject.connect(self.ui.config, QtCore.SIGNAL("clicked()"), self.configureMotor)

    def sizeHint(self):
        return QtCore.QSize(330,50)

    def configureMotor(self):
        showDialogConfigureMotor(self.ui.TauGroupBox)
        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @QtCore.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TauGroupBox.getModel()

    @QtCore.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TauGroupBox.setModel(model)

    @QtCore.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TauGroupBox.resetModel()


    @QtCore.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TauGroupBox.getShowText()

    @QtCore.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TauGroupBox.setShowText(showText)

    @QtCore.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TauGroupBox.resetShowText()


    model = QtCore.pyqtProperty("QString", getModel,setModel,resetModel)
    showText = QtCore.pyqtProperty("bool", getShowText,setShowText,resetShowText)

   

class TauMotorH2(QtGui.QWidget, TauBaseWidget):
    
    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent = None, designMode = False):
        self.call__init__wo_kw(QtGui.QWidget, parent)
        self.call__init__(TauBaseWidget, str(self.objectName()), designMode=designMode)
        self.ui = Ui_TauMotorH2()
        self.ui.setupUi(self)
        QtCore.QObject.connect(self.ui.config, QtCore.SIGNAL("clicked()"), self.configureMotor)

    def sizeHint(self):
        return QtCore.QSize(215,85)

    def configureMotor(self):
        showDialogConfigureMotor(self.ui.TauGroupBox)
        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @QtCore.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TauGroupBox.getModel()

    @QtCore.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TauGroupBox.setModel(model)

    @QtCore.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TauGroupBox.resetModel()


    @QtCore.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TauGroupBox.getShowText()

    @QtCore.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TauGroupBox.setShowText(showText)

    @QtCore.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TauGroupBox.resetShowText()


    model = QtCore.pyqtProperty("QString", getModel,setModel,resetModel)
    showText = QtCore.pyqtProperty("bool", getShowText,setShowText,resetShowText)

class TauMotorV(QtGui.QWidget, TauBaseWidget):
    
    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent = None, designMode = False):
        self.call__init__wo_kw(QtGui.QWidget, parent)
        self.call__init__(TauBaseWidget, str(self.objectName()), designMode=designMode)
        self.ui = Ui_TauMotorV()
        self.ui.setupUi(self)
        QtCore.QObject.connect(self.ui.config, QtCore.SIGNAL("clicked()"), self.configureMotor)

    def sizeHint(self):
        return QtCore.QSize(120,145)

    def configureMotor(self):
        showDialogConfigureMotor(self.ui.TauGroupBox)
        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @QtCore.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TauGroupBox.getModel()

    @QtCore.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TauGroupBox.setModel(model)

    @QtCore.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TauGroupBox.resetModel()


    @QtCore.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TauGroupBox.getShowText()

    @QtCore.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TauGroupBox.setShowText(showText)

    @QtCore.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TauGroupBox.resetShowText()


    model = QtCore.pyqtProperty("QString", getModel,setModel,resetModel)
    showText = QtCore.pyqtProperty("bool", getShowText,setShowText,resetShowText)

class TauMotorV2(QtGui.QWidget, TauBaseWidget):
    
    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent = None, designMode = False):
        self.call__init__wo_kw(QtGui.QWidget, parent)
        self.call__init__(TauBaseWidget, str(self.objectName()), designMode=designMode)
        self.ui = Ui_TauMotorV2()
        self.ui.setupUi(self)

    def sizeHint(self):
        return QtCore.QSize(300,275)
        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @QtCore.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TauGroupBox.getModel()

    @QtCore.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TauGroupBox.setModel(model)

    @QtCore.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TauGroupBox.resetModel()


    @QtCore.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TauGroupBox.getShowText()

    @QtCore.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TauGroupBox.setShowText(showText)

    @QtCore.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TauGroupBox.resetShowText()


    model = QtCore.pyqtProperty("QString", getModel,setModel,resetModel)
    showText = QtCore.pyqtProperty("bool", getShowText,setShowText,resetShowText)
   

###class TauMotorH2(QtGui.QGroupBox, TauBaseWidget):
###    
###    def __init__(self, parent = None, designMode = False):
###        name = "TauMotorH2"
###        self._prefix = ''
###        self._suffix = ''
###        
###        self.call__init__wo_kw(QtGui.QGroupBox, parent)
###        self.call__init__(TauBaseWidget, name, designMode = designMode)
###        
###        self.setObjectName(name)
###        self.defineStyle()
###        
###
###        ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###        ## SO ALL THE STUFFABOVE IS NECESSARY
###        ##
###        ## ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
###        ##############################################################################
###        ##
###        #class TauMotorH2(TauGroupBox):
###        #def __init__(self, parent = None):
###        #self.call__init__wo_kw(TauGroupBox,parent)
###        #self.call__init__(TauGroupBox,str(self.objectName()))
###        self.setupUi()
###        self.retranslateUi()
###        self.connect(self.config,QtCore.SIGNAL('clicked()'),self.configureMotor)
###
###
###    def configureMotor(self):
###        Dialog = QtGui.QDialog(self)
###        Dialog.resize((QtCore.QSize(QtCore.QRect(0,0,310,309).size()).expandedTo(Dialog.minimumSizeHint())))
###        motorV2 = TauMotorV2(Dialog)
###        motorV2.setModel(self.model)
###        motorV2.setGeometry(QtCore.QRect(10,10,291,291))
###        Dialog.show()
###
###
###
###    def minimumSizeHint(self):
###        return QtCore.QSize(211,80)
###    
###    def sizeHint(self):
###        return QtCore.QSize(211,80)
###    
###
###    def setupUi(self):
###
###        self.gridlayout = QtGui.QGridLayout(self)
###        self.gridlayout.setObjectName("gridlayout")
###
###        self.vboxlayout = QtGui.QVBoxLayout()
###        self.vboxlayout.setObjectName("vboxlayout")
###
###        self.hboxlayout = QtGui.QHBoxLayout()
###        self.hboxlayout.setObjectName("hboxlayout")
###
###        self.tauValueLabel_2 = TauValueLabel(self)
###        self.tauValueLabel_2.setFrameShape(QtGui.QFrame.NoFrame)
###        self.tauValueLabel_2.setFrameShadow(QtGui.QFrame.Plain)
###        self.tauValueLabel_2.setShowQuality(False)
###        self.tauValueLabel_2.setUseParentModel(True)
###        self.tauValueLabel_2.setObjectName("tauValueLabel_2")
###        self.hboxlayout.addWidget(self.tauValueLabel_2)
###
###        self.TauStateLed_17 = TauStateLed(self)
###        self.TauStateLed_17.setUseParentModel(True)
###        self.TauStateLed_17.setObjectName("TauStateLed_17")
###        self.hboxlayout.addWidget(self.TauStateLed_17)
###
###        spacerItem = QtGui.QSpacerItem(40,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
###        self.hboxlayout.addItem(spacerItem)
###
###        self.TauLimitSwitch = TauLimitSwitch(self)
###        self.TauLimitSwitch.setUseParentModel(True)
###        self.TauLimitSwitch.setBoolIndex(2)
###        self.TauLimitSwitch.setObjectName("TauLimitSwitch")
###        self.hboxlayout.addWidget(self.TauLimitSwitch)
###
###        self.label_3 = QtGui.QLabel(self)
###        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
###        self.label_3.setObjectName("label_3")
###        self.hboxlayout.addWidget(self.label_3)
###
###        self.TauLimitSwitch_2 = TauLimitSwitch(self)
###        self.TauLimitSwitch_2.setUseParentModel(True)
###        self.TauLimitSwitch_2.setBoolIndex(1)
###        self.TauLimitSwitch_2.setObjectName("TauLimitSwitch_2")
###        self.hboxlayout.addWidget(self.TauLimitSwitch_2)
###        self.vboxlayout.addLayout(self.hboxlayout)
###
###        self.hboxlayout1 = QtGui.QHBoxLayout()
###        self.hboxlayout1.setObjectName("hboxlayout1")
###
###        self.TauValueLineEdit_21 = TauValueLineEdit(self)
###        self.TauValueLineEdit_21.setUseParentModel(True)
###        self.TauValueLineEdit_21.setObjectName("TauValueLineEdit_21")
###        self.hboxlayout1.addWidget(self.TauValueLineEdit_21)
###
###        self.tauValueLabel_21 = TauValueLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.tauValueLabel_21.sizePolicy().hasHeightForWidth())
###        self.tauValueLabel_21.setSizePolicy(sizePolicy)
###        self.tauValueLabel_21.setUseParentModel(True)
###        self.tauValueLabel_21.setObjectName("tauValueLabel_21")
###        self.hboxlayout1.addWidget(self.tauValueLabel_21)
###
###        self.tauConfigLabel_18 = TauConfigLabel(self)
###        self.tauConfigLabel_18.setMaximumSize(QtCore.QSize(27,22))
###        self.tauConfigLabel_18.setUseParentModel(True)
###        self.tauConfigLabel_18.setObjectName("tauConfigLabel_18")
###        self.hboxlayout1.addWidget(self.tauConfigLabel_18)
###
###        self.config = QtGui.QToolButton(self)
###        self.config.setObjectName("config")
###        self.hboxlayout1.addWidget(self.config)
###        self.vboxlayout.addLayout(self.hboxlayout1)
###        self.gridlayout.addLayout(self.vboxlayout,0,0,1,1)
###
###
###    def retranslateUi(self):
###        self.tauValueLabel_2.setModel(QtGui.QApplication.translate("Form", "/State", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauStateLed_17.setModel(QtGui.QApplication.translate("Form", "/State", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch.setModel(QtGui.QApplication.translate("Form", "/Limit_switches", None, QtGui.QApplication.UnicodeUTF8))
###        self.label_3.setText(QtGui.QApplication.translate("Form", "- lim +", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch_2.setModel(QtGui.QApplication.translate("Form", "/Limit_switches", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauValueLineEdit_21.setModel(QtGui.QApplication.translate("Form", "/Position", None, QtGui.QApplication.UnicodeUTF8))
###        self.tauValueLabel_21.setModel(QtGui.QApplication.translate("Form", "/Position", None, QtGui.QApplication.UnicodeUTF8))
###        self.tauConfigLabel_18.setModel(QtGui.QApplication.translate("Form", "/Position?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.config.setText(QtGui.QApplication.translate("Form", "cfg", None, QtGui.QApplication.UnicodeUTF8))
###
###
###    ##############################################################################
###    ## VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
###    ##
###    ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###    ## SO ALL THE STUFF BELOW IS NECESSARY
###
###
###        
###    def defineStyle(self):
###        palette = QtGui.QPalette()
###        self.setPalette(palette)
###        self.updateStyle()
###        
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # TauBaseWidget over writing
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###                    
###    def getDisplayValue(self):
###        return (self._prefix or '') + TauBaseWidget.getDisplayValue(self) + (self._suffix or '')
###    
###    def isReadOnly(self):
###        return True
###    
###    def updateStyle(self):
###        if self.getShowQuality():
###            self.setAutoFillBackground(True)
###            #TODO: get quality/state from model and update accordingly
###        else:
###            self.setAutoFillBackground(False)
###            #TODO: restore colors
###        self.update()
###        
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # QT properties 
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###   
###    def getPrefixText(self):
###        return self._prefix
###    
###    @QtCore.pyqtSignature("setPrefixText(QString)")
###    def setPrefixText(self,prefix):
###        self._prefix = prefix
###        self.fireValueChanged()
###
###    def getSuffixText(self):
###        return self._suffix
###    
###    @QtCore.pyqtSignature("setSuffixText(QString)")
###    def setSuffixText(self,suffix):
###        self._suffix = suffix
###        self.fireValueChanged()
###
###    model = QtCore.pyqtProperty("QString", TauBaseWidget.getModel, 
###                                TauBaseWidget.setModel, TauBaseWidget.resetModel)
###    useParentModel = QtCore.pyqtProperty("bool", TauBaseWidget.getUseParentModel, 
###                                         TauBaseWidget.setUseParentModel, 
###                                         TauBaseWidget.resetUseParentModel)
###    showQuality = QtCore.pyqtProperty("bool", TauBaseWidget.getShowQuality, 
###                                      TauBaseWidget.setShowQuality, 
###                                      TauBaseWidget.resetShowQuality)
###    showText = QtCore.pyqtProperty("bool", TauBaseWidget.getShowText, 
###                                   TauBaseWidget.setShowText, 
###                                   TauBaseWidget.resetShowText)
###    prefixText = QtCore.pyqtProperty("QString", getPrefixText, setPrefixText, 
###                                     doc="prefix text (optional)")
###    suffixText = QtCore.pyqtProperty("QString", getSuffixText, setSuffixText, 
###                                    doc="suffix text (optional)")
###
###        
###
###
###class TauMotorV(QtGui.QGroupBox, TauBaseWidget):
###    
###    def __init__(self, parent = None):
###        name = "TauMotorV"
###        self._prefix = ''
###        self._suffix = ''
###        
###        self.call__init__wo_kw(QtGui.QGroupBox, parent)
###        self.call__init__(TauBaseWidget, name)
###        
###        self.setObjectName(name)
###        self.defineStyle()
###        
###
###        ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###        ## SO ALL THE STUFFABOVE IS NECESSARY
###        ##
###        ## ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
###        ##############################################################################
###        ##
###        #class TauMotorV(TauGroupBox):
###        #def __init__(self, parent = None):
###        #self.call__init__wo_kw(TauGroupBox,parent)
###        #self.call__init__(TauGroupBox,str(self.objectName()))
###        self.setupUi()
###        self.retranslateUi()
###        self.connect(self.config,QtCore.SIGNAL('clicked()'),self.configureMotor)
###
###
###    def configureMotor(self):
###        Dialog = QtGui.QDialog(self)
###        Dialog.resize((QtCore.QSize(QtCore.QRect(0,0,310,309).size()).expandedTo(Dialog.minimumSizeHint())))
###        motorV2 = TauMotorV2(Dialog)
###        motorV2.setModel(self.model)
###        motorV2.setGeometry(QtCore.QRect(10,10,291,291))
###        Dialog.show()
###
###
###
###
###    def minimumSizeHint(self):
###        return QtCore.QSize(150,128)
###    
###    def sizeHint(self):
###        return QtCore.QSize(150,128)
###    
###    def setupUi(self):
###
###        self.gridlayout = QtGui.QGridLayout(self)
###        self.gridlayout.setObjectName("gridlayout")
###
###        self.vboxlayout = QtGui.QVBoxLayout()
###        self.vboxlayout.setObjectName("vboxlayout")
###
###        self.hboxlayout = QtGui.QHBoxLayout()
###        self.hboxlayout.setObjectName("hboxlayout")
###
###        self.tauValueLabel_2 = TauValueLabel(self)
###        self.tauValueLabel_2.setFrameShape(QtGui.QFrame.NoFrame)
###        self.tauValueLabel_2.setFrameShadow(QtGui.QFrame.Plain)
###        self.tauValueLabel_2.setShowQuality(False)
###        self.tauValueLabel_2.setUseParentModel(True)
###        self.tauValueLabel_2.setObjectName("tauValueLabel_2")
###        self.hboxlayout.addWidget(self.tauValueLabel_2)
###
###        self.TauStateLed_17 = TauStateLed(self)
###        self.TauStateLed_17.setUseParentModel(True)
###        self.TauStateLed_17.setObjectName("TauStateLed_17")
###        self.hboxlayout.addWidget(self.TauStateLed_17)
###        self.vboxlayout.addLayout(self.hboxlayout)
###
###        self.hboxlayout1 = QtGui.QHBoxLayout()
###        self.hboxlayout1.setObjectName("hboxlayout1")
###
###        self.TauValueLineEdit_21 = TauValueLineEdit(self)
###        self.TauValueLineEdit_21.setUseParentModel(True)
###        self.TauValueLineEdit_21.setObjectName("TauValueLineEdit_21")
###        self.hboxlayout1.addWidget(self.TauValueLineEdit_21)
###
###        self.config = QtGui.QToolButton(self)
###        self.config.setObjectName("config")
###        self.hboxlayout1.addWidget(self.config)
###        self.vboxlayout.addLayout(self.hboxlayout1)
###
###        self.hboxlayout2 = QtGui.QHBoxLayout()
###        self.hboxlayout2.setObjectName("hboxlayout2")
###
###        self.tauValueLabel_21 = TauValueLabel(self)
###        self.tauValueLabel_21.setUseParentModel(True)
###        self.tauValueLabel_21.setObjectName("tauValueLabel_21")
###        self.hboxlayout2.addWidget(self.tauValueLabel_21)
###
###        self.tauConfigLabel_18 = TauConfigLabel(self)
###        self.tauConfigLabel_18.setMaximumSize(QtCore.QSize(27,22))
###        self.tauConfigLabel_18.setUseParentModel(True)
###        self.tauConfigLabel_18.setObjectName("tauConfigLabel_18")
###        self.hboxlayout2.addWidget(self.tauConfigLabel_18)
###        self.vboxlayout.addLayout(self.hboxlayout2)
###
###        self.hboxlayout3 = QtGui.QHBoxLayout()
###        self.hboxlayout3.setObjectName("hboxlayout3")
###
###        self.TauLimitSwitch = TauLimitSwitch(self)
###        self.TauLimitSwitch.setUseParentModel(True)
###        self.TauLimitSwitch.setBoolIndex(2)
###        self.TauLimitSwitch.setObjectName("TauLimitSwitch")
###        self.hboxlayout3.addWidget(self.TauLimitSwitch)
###
###        self.label_3 = QtGui.QLabel(self)
###        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
###        self.label_3.setObjectName("label_3")
###        self.hboxlayout3.addWidget(self.label_3)
###
###        self.TauLimitSwitch_2 = TauLimitSwitch(self)
###        self.TauLimitSwitch_2.setUseParentModel(True)
###        self.TauLimitSwitch_2.setBoolIndex(1)
###        self.TauLimitSwitch_2.setObjectName("TauLimitSwitch_2")
###        self.hboxlayout3.addWidget(self.TauLimitSwitch_2)
###        self.vboxlayout.addLayout(self.hboxlayout3)
###        self.gridlayout.addLayout(self.vboxlayout,0,0,1,1)
###
###
###    def retranslateUi(self):
###        self.tauValueLabel_2.setModel(QtGui.QApplication.translate("Form", "/State", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauStateLed_17.setModel(QtGui.QApplication.translate("Form", "/State", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauValueLineEdit_21.setModel(QtGui.QApplication.translate("Form", "/Position", None, QtGui.QApplication.UnicodeUTF8))
###        self.config.setText(QtGui.QApplication.translate("Form", "cfg", None, QtGui.QApplication.UnicodeUTF8))
###        self.tauValueLabel_21.setModel(QtGui.QApplication.translate("Form", "/Position", None, QtGui.QApplication.UnicodeUTF8))
###        self.tauConfigLabel_18.setModel(QtGui.QApplication.translate("Form", "/Position?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch.setModel(QtGui.QApplication.translate("Form", "/Limit_switches", None, QtGui.QApplication.UnicodeUTF8))
###        self.label_3.setText(QtGui.QApplication.translate("Form", "- lim +", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch_2.setModel(QtGui.QApplication.translate("Form", "/Limit_switches", None, QtGui.QApplication.UnicodeUTF8))
###
###    ##############################################################################
###    ## VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
###    ##
###    ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###    ## SO ALL THE STUFF BELOW IS NECESSARY
###
###
###        
###    def defineStyle(self):
###        palette = QtGui.QPalette()
###        self.setPalette(palette)
###        self.updateStyle()
###        
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # TauBaseWidget over writing
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###                    
###    def getDisplayValue(self):
###        return (self._prefix or '') + TauBaseWidget.getDisplayValue(self) + (self._suffix or '')
###    
###    def isReadOnly(self):
###        return True
###    
###    def updateStyle(self):
###        if self.getShowQuality():
###            self.setAutoFillBackground(True)
###            #TODO: get quality/state from model and update accordingly
###        else:
###            self.setAutoFillBackground(False)
###            #TODO: restore colors
###        self.update()
###        
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # QT properties 
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###   
###    def getPrefixText(self):
###        return self._prefix
###    
###    @QtCore.pyqtSignature("setPrefixText(QString)")
###    def setPrefixText(self,prefix):
###        self._prefix = prefix
###        self.fireValueChanged()
###
###    def getSuffixText(self):
###        return self._suffix
###    
###    @QtCore.pyqtSignature("setSuffixText(QString)")
###    def setSuffixText(self,suffix):
###        self._suffix = suffix
###        self.fireValueChanged()
###
###    model = QtCore.pyqtProperty("QString", TauBaseWidget.getModel, 
###                                TauBaseWidget.setModel, TauBaseWidget.resetModel)
###    useParentModel = QtCore.pyqtProperty("bool", TauBaseWidget.getUseParentModel, 
###                                         TauBaseWidget.setUseParentModel, 
###                                         TauBaseWidget.resetUseParentModel)
###    showQuality = QtCore.pyqtProperty("bool", TauBaseWidget.getShowQuality, 
###                                      TauBaseWidget.setShowQuality, 
###                                      TauBaseWidget.resetShowQuality)
###    showText = QtCore.pyqtProperty("bool", TauBaseWidget.getShowText, 
###                                   TauBaseWidget.setShowText, 
###                                   TauBaseWidget.resetShowText)
###    prefixText = QtCore.pyqtProperty("QString", getPrefixText, setPrefixText, 
###                                     doc="prefix text (optional)")
###    suffixText = QtCore.pyqtProperty("QString", getSuffixText, setSuffixText, 
###                                    doc="suffix text (optional)")
###
###        
###
###class TauMotorV2(QtGui.QGroupBox, TauBaseWidget):
###    
###    def __init__(self, parent = None):
###        name = "TauMotorV2"
###        self._prefix = ''
###        self._suffix = ''
###        
###        self.call__init__wo_kw(QtGui.QGroupBox, parent)
###        self.call__init__(TauBaseWidget, name)
###        
###        self.setObjectName(name)
###        self.defineStyle()
###        
###
###        ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###        ## SO ALL THE STUFFABOVE IS NECESSARY
###        ##
###        ## ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
###        ##############################################################################
###        ##
###        #class TauMotorV2(TauGroupBox):
###        #def __init__(self, parent = None):
###        #self.call__init__wo_kw(TauGroupBox,parent)
###        #self.call__init__(TauGroupBox,str(self.objectName()))
###        self.setupUi()
###        self.retranslateUi()
###
###
###    def minimumSizeHint(self):
###        return QtCore.QSize(260,270)
###    
###    def sizeHint(self):
###        return QtCore.QSize(260,270)
###    
###
###    def setupUi(self):
###
###        self.gridlayout = QtGui.QGridLayout(self)
###        self.gridlayout.setObjectName("gridlayout")
###
###        self.m1StateLed_2 = TauStateLed(self)
###        self.m1StateLed_2.setLedSize(24)
###        self.m1StateLed_2.setUseParentModel(True)
###        self.m1StateLed_2.setObjectName("m1StateLed_2")
###        self.gridlayout.addWidget(self.m1StateLed_2,0,0,1,1)
###
###        self.tauValueLabel = TauValueLabel(self)
###        self.tauValueLabel.setFrameShape(QtGui.QFrame.NoFrame)
###        self.tauValueLabel.setFrameShadow(QtGui.QFrame.Plain)
###        self.tauValueLabel.setShowQuality(False)
###        self.tauValueLabel.setUseParentModel(True)
###        self.tauValueLabel.setObjectName("tauValueLabel")
###        self.gridlayout.addWidget(self.tauValueLabel,0,1,1,1)
###
###        self.TauLimitSwitch = TauLimitSwitch(self)
###        self.TauLimitSwitch.setUseParentModel(True)
###        self.TauLimitSwitch.setBoolIndex(2)
###        self.TauLimitSwitch.setObjectName("TauLimitSwitch")
###        self.gridlayout.addWidget(self.TauLimitSwitch,0,3,1,1)
###
###        self.TauLimitSwitch_2 = TauLimitSwitch(self)
###        self.TauLimitSwitch_2.setUseParentModel(True)
###        self.TauLimitSwitch_2.setBoolIndex(1)
###        self.TauLimitSwitch_2.setObjectName("TauLimitSwitch_2")
###        self.gridlayout.addWidget(self.TauLimitSwitch_2,0,4,1,1)
###
###        self.m1PositionLabel_2 = TauConfigLabel(self)
###        self.m1PositionLabel_2.setUseParentModel(True)
###        self.m1PositionLabel_2.setObjectName("m1PositionLabel_2")
###        self.gridlayout.addWidget(self.m1PositionLabel_2,1,0,1,2)
###
###        self.m1PositionEdit_2 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_2.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_2.setUseParentModel(True)
###        self.m1PositionEdit_2.setObjectName("m1PositionEdit_2")
###        self.gridlayout.addWidget(self.m1PositionEdit_2,1,2,1,1)
###
###        self.m1Position_2 = TauValueLabel(self)
###        self.m1Position_2.setUseParentModel(True)
###        self.m1Position_2.setObjectName("m1Position_2")
###        self.gridlayout.addWidget(self.m1Position_2,1,3,1,2)
###
###        self.m1PositionUnitLabel_2 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_2.setUseParentModel(True)
###        self.m1PositionUnitLabel_2.setObjectName("m1PositionUnitLabel_2")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_2,1,5,1,1)
###
###        self.m1VelocityLabel_2 = TauConfigLabel(self)
###        self.m1VelocityLabel_2.setUseParentModel(True)
###        self.m1VelocityLabel_2.setObjectName("m1VelocityLabel_2")
###        self.gridlayout.addWidget(self.m1VelocityLabel_2,2,0,1,2)
###
###        self.m1VelocityEdit_2 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1VelocityEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1VelocityEdit_2.setSizePolicy(sizePolicy)
###        self.m1VelocityEdit_2.setUseParentModel(True)
###        self.m1VelocityEdit_2.setObjectName("m1VelocityEdit_2")
###        self.gridlayout.addWidget(self.m1VelocityEdit_2,2,2,1,1)
###
###        self.m1Velocity_2 = TauValueLabel(self)
###        self.m1Velocity_2.setUseParentModel(True)
###        self.m1Velocity_2.setObjectName("m1Velocity_2")
###        self.gridlayout.addWidget(self.m1Velocity_2,2,3,1,2)
###
###        self.m1VelocityUnitLabel_2 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1VelocityUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1VelocityUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1VelocityUnitLabel_2.setUseParentModel(True)
###        self.m1VelocityUnitLabel_2.setObjectName("m1VelocityUnitLabel_2")
###        self.gridlayout.addWidget(self.m1VelocityUnitLabel_2,2,5,1,1)
###
###        self.m1PositionLabel_4 = TauConfigLabel(self)
###        self.m1PositionLabel_4.setUseParentModel(True)
###        self.m1PositionLabel_4.setObjectName("m1PositionLabel_4")
###        self.gridlayout.addWidget(self.m1PositionLabel_4,3,0,1,2)
###
###        self.m1PositionEdit_4 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_4.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_4.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_4.setUseParentModel(True)
###        self.m1PositionEdit_4.setObjectName("m1PositionEdit_4")
###        self.gridlayout.addWidget(self.m1PositionEdit_4,3,2,1,1)
###
###        self.m1Position_4 = TauValueLabel(self)
###        self.m1Position_4.setUseParentModel(True)
###        self.m1Position_4.setObjectName("m1Position_4")
###        self.gridlayout.addWidget(self.m1Position_4,3,3,1,2)
###
###        self.m1PositionUnitLabel_4 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_4.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_4.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_4.setUseParentModel(True)
###        self.m1PositionUnitLabel_4.setObjectName("m1PositionUnitLabel_4")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_4,3,5,1,1)
###
###        self.m1AccelerationLabel_2 = TauConfigLabel(self)
###        self.m1AccelerationLabel_2.setUseParentModel(True)
###        self.m1AccelerationLabel_2.setObjectName("m1AccelerationLabel_2")
###        self.gridlayout.addWidget(self.m1AccelerationLabel_2,4,0,1,2)
###
###        self.m1AccelerationEdit_2 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1AccelerationEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1AccelerationEdit_2.setSizePolicy(sizePolicy)
###        self.m1AccelerationEdit_2.setUseParentModel(True)
###        self.m1AccelerationEdit_2.setObjectName("m1AccelerationEdit_2")
###        self.gridlayout.addWidget(self.m1AccelerationEdit_2,4,2,1,1)
###
###        self.m1Acceleration_2 = TauValueLabel(self)
###        self.m1Acceleration_2.setUseParentModel(True)
###        self.m1Acceleration_2.setObjectName("m1Acceleration_2")
###        self.gridlayout.addWidget(self.m1Acceleration_2,4,3,1,2)
###
###        self.m1AccelerationUnitLabel_2 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1AccelerationUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1AccelerationUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1AccelerationUnitLabel_2.setUseParentModel(True)
###        self.m1AccelerationUnitLabel_2.setObjectName("m1AccelerationUnitLabel_2")
###        self.gridlayout.addWidget(self.m1AccelerationUnitLabel_2,4,5,1,1)
###
###        self.m1PositionLabel_5 = TauConfigLabel(self)
###        self.m1PositionLabel_5.setUseParentModel(True)
###        self.m1PositionLabel_5.setObjectName("m1PositionLabel_5")
###        self.gridlayout.addWidget(self.m1PositionLabel_5,5,0,1,2)
###
###        self.m1PositionEdit_5 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_5.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_5.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_5.setUseParentModel(True)
###        self.m1PositionEdit_5.setObjectName("m1PositionEdit_5")
###        self.gridlayout.addWidget(self.m1PositionEdit_5,5,2,1,1)
###
###        self.m1Position_5 = TauValueLabel(self)
###        self.m1Position_5.setUseParentModel(True)
###        self.m1Position_5.setObjectName("m1Position_5")
###        self.gridlayout.addWidget(self.m1Position_5,5,3,1,2)
###
###        self.m1PositionUnitLabel_5 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_5.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_5.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_5.setUseParentModel(True)
###        self.m1PositionUnitLabel_5.setObjectName("m1PositionUnitLabel_5")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_5,5,5,1,1)
###
###        self.m1DecelerationLabel_2 = TauConfigLabel(self)
###        self.m1DecelerationLabel_2.setUseParentModel(True)
###        self.m1DecelerationLabel_2.setObjectName("m1DecelerationLabel_2")
###        self.gridlayout.addWidget(self.m1DecelerationLabel_2,6,0,1,2)
###
###        self.m1DecelerationEdit_2 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1DecelerationEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1DecelerationEdit_2.setSizePolicy(sizePolicy)
###        self.m1DecelerationEdit_2.setUseParentModel(True)
###        self.m1DecelerationEdit_2.setObjectName("m1DecelerationEdit_2")
###        self.gridlayout.addWidget(self.m1DecelerationEdit_2,6,2,1,1)
###
###        self.m1Deceleration_2 = TauValueLabel(self)
###        self.m1Deceleration_2.setUseParentModel(True)
###        self.m1Deceleration_2.setObjectName("m1Deceleration_2")
###        self.gridlayout.addWidget(self.m1Deceleration_2,6,3,1,2)
###
###        self.m1DecelerationUnitLabel_2 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1DecelerationUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1DecelerationUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1DecelerationUnitLabel_2.setUseParentModel(True)
###        self.m1DecelerationUnitLabel_2.setObjectName("m1DecelerationUnitLabel_2")
###        self.gridlayout.addWidget(self.m1DecelerationUnitLabel_2,6,5,1,1)
###
###        self.m1PositionLabel_6 = TauConfigLabel(self)
###        self.m1PositionLabel_6.setUseParentModel(True)
###        self.m1PositionLabel_6.setObjectName("m1PositionLabel_6")
###        self.gridlayout.addWidget(self.m1PositionLabel_6,7,0,1,2)
###
###        self.m1PositionEdit_6 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_6.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_6.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_6.setUseParentModel(True)
###        self.m1PositionEdit_6.setObjectName("m1PositionEdit_6")
###        self.gridlayout.addWidget(self.m1PositionEdit_6,7,2,1,1)
###
###        self.m1Position_6 = TauValueLabel(self)
###        self.m1Position_6.setUseParentModel(True)
###        self.m1Position_6.setObjectName("m1Position_6")
###        self.gridlayout.addWidget(self.m1Position_6,7,3,1,2)
###
###        self.m1PositionUnitLabel_6 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_6.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_6.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_6.setUseParentModel(True)
###        self.m1PositionUnitLabel_6.setObjectName("m1PositionUnitLabel_6")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_6,7,5,1,1)
###
###        self.m1StepPerUnitLabel_2 = TauConfigLabel(self)
###        self.m1StepPerUnitLabel_2.setUseParentModel(True)
###        self.m1StepPerUnitLabel_2.setObjectName("m1StepPerUnitLabel_2")
###        self.gridlayout.addWidget(self.m1StepPerUnitLabel_2,8,0,1,2)
###
###        self.m1StepPerUnitEdit_2 = TauValueLineEdit(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1StepPerUnitEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1StepPerUnitEdit_2.setSizePolicy(sizePolicy)
###        self.m1StepPerUnitEdit_2.setUseParentModel(True)
###        self.m1StepPerUnitEdit_2.setObjectName("m1StepPerUnitEdit_2")
###        self.gridlayout.addWidget(self.m1StepPerUnitEdit_2,8,2,1,1)
###
###        self.m1StepPerUnit_2 = TauValueLabel(self)
###        self.m1StepPerUnit_2.setUseParentModel(True)
###        self.m1StepPerUnit_2.setObjectName("m1StepPerUnit_2")
###        self.gridlayout.addWidget(self.m1StepPerUnit_2,8,3,1,2)
###
###        self.m1StepPerUnitUnitLabel_2 = TauConfigLabel(self)
###
###        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1StepPerUnitUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1StepPerUnitUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1StepPerUnitUnitLabel_2.setUseParentModel(True)
###        self.m1StepPerUnitUnitLabel_2.setObjectName("m1StepPerUnitUnitLabel_2")
###        self.gridlayout.addWidget(self.m1StepPerUnitUnitLabel_2,8,5,1,1)
###
###
###    def retranslateUi(self):
###        self.m1StateLed_2.setModel(QtGui.QApplication.translate("Dialog", "/State", None, QtGui.QApplication.UnicodeUTF8))
###        self.tauValueLabel.setModel(QtGui.QApplication.translate("Dialog", "/State", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch.setLedColor(QtGui.QApplication.translate("Dialog", "ORANGE", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch.setModel(QtGui.QApplication.translate("Dialog", "/Limit_switches", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch_2.setLedColor(QtGui.QApplication.translate("Dialog", "ORANGE", None, QtGui.QApplication.UnicodeUTF8))
###        self.TauLimitSwitch_2.setModel(QtGui.QApplication.translate("Dialog", "/Limit_switches", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_2.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Position?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_2.setModel(QtGui.QApplication.translate("Dialog", "/Position", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1Position_2.setModel(QtGui.QApplication.translate("Dialog", "/Position", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Position?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1VelocityLabel_2.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1VelocityLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Velocity?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1VelocityEdit_2.setModel(QtGui.QApplication.translate("Dialog", "/Velocity", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1Velocity_2.setModel(QtGui.QApplication.translate("Dialog", "/Velocity", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1VelocityUnitLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Velocity?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_4.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_4.setModel(QtGui.QApplication.translate("Dialog", "/Acceleration?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_4.setModel(QtGui.QApplication.translate("Dialog", "/Acceleration", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1Position_4.setModel(QtGui.QApplication.translate("Dialog", "/Acceleration", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_4.setModel(QtGui.QApplication.translate("Dialog", "/Acceleration?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1AccelerationLabel_2.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1AccelerationLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Deceleration?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1AccelerationEdit_2.setModel(QtGui.QApplication.translate("Dialog", "/Deceleration", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1Acceleration_2.setModel(QtGui.QApplication.translate("Dialog", "/Deceleration", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1AccelerationUnitLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Deceleration?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_5.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_5.setModel(QtGui.QApplication.translate("Dialog", "/Offset?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_5.setModel(QtGui.QApplication.translate("Dialog", "/Offset", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1Position_5.setModel(QtGui.QApplication.translate("Dialog", "/Offset", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_5.setModel(QtGui.QApplication.translate("Dialog", "/Offset?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1DecelerationLabel_2.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1DecelerationLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Base_rate?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1DecelerationEdit_2.setModel(QtGui.QApplication.translate("Dialog", "/Base_rate", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1Deceleration_2.setModel(QtGui.QApplication.translate("Dialog", "/Base_rate", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1DecelerationUnitLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Base_rate?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_6.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_6.setModel(QtGui.QApplication.translate("Dialog", "/Step_per_unit?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_6.setModel(QtGui.QApplication.translate("Dialog", "/Step_per_unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1Position_6.setModel(QtGui.QApplication.translate("Dialog", "/Step_per_unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_6.setModel(QtGui.QApplication.translate("Dialog", "/Step_per_unit?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitLabel_2.setSuffixText(QtGui.QApplication.translate("Dialog", ":", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Backlash?configuration=label", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitEdit_2.setModel(QtGui.QApplication.translate("Dialog", "/Backlash", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1StepPerUnit_2.setModel(QtGui.QApplication.translate("Dialog", "/Backlash", None, QtGui.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitUnitLabel_2.setModel(QtGui.QApplication.translate("Dialog", "/Backlash?configuration=unit", None, QtGui.QApplication.UnicodeUTF8))
###
###    ##############################################################################
###    ## VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
###    ##
###    ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###    ## SO ALL THE STUFF BELOW IS NECESSARY
###
###
###        
###    def defineStyle(self):
###        palette = QtGui.QPalette()
###        self.setPalette(palette)
###        self.updateStyle()
###        
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # TauBaseWidget over writing
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###                    
###    def getDisplayValue(self):
###        return (self._prefix or '') + TauBaseWidget.getDisplayValue(self) + (self._suffix or '')
###    
###    def isReadOnly(self):
###        return True
###    
###    def updateStyle(self):
###        if self.getShowQuality():
###            self.setAutoFillBackground(True)
###            #TODO: get quality/state from model and update accordingly
###        else:
###            self.setAutoFillBackground(False)
###            #TODO: restore colors
###        self.update()
###        
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # QT properties 
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###   
###    def getPrefixText(self):
###        return self._prefix
###    
###    @QtCore.pyqtSignature("setPrefixText(QString)")
###    def setPrefixText(self,prefix):
###        self._prefix = prefix
###        self.fireValueChanged()
###
###    def getSuffixText(self):
###        return self._suffix
###    
###    @QtCore.pyqtSignature("setSuffixText(QString)")
###    def setSuffixText(self,suffix):
###        self._suffix = suffix
###        self.fireValueChanged()
###
###    model = QtCore.pyqtProperty("QString", TauBaseWidget.getModel, 
###                                TauBaseWidget.setModel, TauBaseWidget.resetModel)
###    useParentModel = QtCore.pyqtProperty("bool", TauBaseWidget.getUseParentModel, 
###                                         TauBaseWidget.setUseParentModel, 
###                                         TauBaseWidget.resetUseParentModel)
###    showQuality = QtCore.pyqtProperty("bool", TauBaseWidget.getShowQuality, 
###                                      TauBaseWidget.setShowQuality, 
###                                      TauBaseWidget.resetShowQuality)
###    showText = QtCore.pyqtProperty("bool", TauBaseWidget.getShowText, 
###                                   TauBaseWidget.setShowText, 
###                                   TauBaseWidget.resetShowText)
###    prefixText = QtCore.pyqtProperty("QString", getPrefixText, setPrefixText, 
###                                     doc="prefix text (optional)")
###    suffixText = QtCore.pyqtProperty("QString", getSuffixText, setSuffixText, 
###                                    doc="suffix text (optional)")
###
###        
###
