
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
gauges.py: 
"""

import qtcontrols

__all__ = ['TauLinearGauge', 'TauCircularGauge']

import math

from PyQt4 import QtCore, QtGui, Qt
import taubase

class TauLinearGauge(qtcontrols.ELinearGauge, taubase.TauBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent = None, designMode = False):
        name = self.__class__.__name__
        self.call__init__wo_kw(qtcontrols.ELinearGauge, parent)
        self.call__init__(taubase.TauBaseWidget, name, designMode=designMode)

    def sizeHint(self):
        return QtCore.QSize(100, 200)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget overwritting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    
    def getModelClass(self):
        import tau.core
        return tau.core.TauAttribute

    def handleEvent(self, evt_src, evt_type, evt_value):
        v, v_ref = None, None
        
        model = self.getModelObj()
        v = self.getModelValueObj()
        
        if model is None or v is None:
            self.setEnabled(False)
        else:
            self.setEnabled(True)

        if not v is None:
            v_ref = v.w_value or self.minValue()
            v = v.value
        else:
            v = v_ref = self.minValue()

        self.setValue(v)
        self.setReference(v_ref)

        if model is None:
            # no information available
            # keep the old values
            pass
        else:
            try:
                minW, maxW = int(model.getMinWarning()), int(model.getMaxWarning())
                self.setLowWarning(minW)
                self.setHighWarning(maxW)
            except:
                pass
            try:
                minV, maxV = int(model.getMinValue()), int(model.getMaxValue())
                self.setMinValue(minV)
                self.setMaxValue(maxV)
            except:
                pass
            try:
                minA, maxA = int(model.getMinAlarm()), int(model.getMaxAlarm())
                self.setLowError(minA)
                self.setHighError(maxA)
            except:
                pass
        
        self.setToolTip(self.getFormatedToolTip())

    def isReadOnly(self):
        return True
    
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-  

    model = QtCore.pyqtProperty("QString", taubase.TauBaseWidget.getModel, 
                                taubase.TauBaseWidget.setModel, 
                                taubase.TauBaseWidget.resetModel)
                                
    useParentModel = QtCore.pyqtProperty("bool", 
                                         taubase.TauBaseWidget.getUseParentModel, 
                                         taubase.TauBaseWidget.setUseParentModel,
                                         taubase.TauBaseWidget.resetUseParentModel)

                                                                                  
class TauCircularGauge(qtcontrols.ECircularGauge, taubase.TauBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent = None, designMode = False):
        name = self.__class__.__name__
        self.call__init__(taubase.TauBaseWidget, name, designMode=designMode)
        self.call__init__wo_kw(qtcontrols.ECircularGauge, parent)
        
        self._logScale = False

    def sizeHint(self):
        return QtCore.QSize(100, 100)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget overwritting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    
    def getModelClass(self):
        import tau.core
        return tau.core.TauAttribute

    def handleEvent(self, evt_src, evt_type, evt_value):
        v, v_ref = None, None
        
        model = self.getModelObj()
        v = self.getModelValueObj()
        
        if model is None or v is None:
            self.setEnabled(False)
        else:
            self.setEnabled(True)

        if not v is None:
            v_ref = v.w_value or self.minValue()
            v = v.value
        else:
            v = v_ref = self.minValue()

        if self._logScale:
            v = math.log(v, 10)
                
        self.setValue(v)
        self.setReference(v_ref)

        if model is None:
            # no information available
            # keep the old values
            pass
        else:
            fmt = model.getFormat() or self.valueFormat()
            self.setValueFormat(fmt)
            try:
                minW, maxW = int(model.getMinWarning()), int(model.getMaxWarning())
                if self._logScale: minW, maxW = math.log(minW, 10), math.log(maxW, 10)
                self.setLowWarning(minW)
                self.setHighWarning(maxW)
            except:
                pass
            try:
                minV, maxV = int(model.getMinValue()), int(model.getMaxValue())
                if self._logScale: minV, maxV = math.log(minV, 10), math.log(maxV, 10)
                self.setMinValue(minV)
                self.setMaxValue(maxV)
            except:
                pass
            try:
                minA, maxA = int(model.getMinAlarm()), int(model.getMaxAlarm())
                if self._logScale: minA, maxA = math.log(minA, 10), math.log(maxA, 10)
                self.setLowError(minA)
                self.setHighError(maxA)
            except:
                pass
        
        self.setToolTip(self.getFormatedToolTip())

    def isReadOnly(self):
        return True
    
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-  
    
    def getLogScale(self):
        return self._logScale

    @QtCore.pyqtSignature("setLogScale(bool)")
    def setLogScale(self, val):
        self._logScale = bool(val)
        self.updateStyle()

    def resetLogScale(self):
        self._logScale = False
        self.updateStyle()

    logScale  = QtCore.pyqtProperty("bool", getLogScale, setLogScale, resetLogScale)
    
    model = QtCore.pyqtProperty("QString", taubase.TauBaseWidget.getModel, 
                                taubase.TauBaseWidget.setModel, 
                                taubase.TauBaseWidget.resetModel)
                                
    useParentModel = QtCore.pyqtProperty("bool", 
                                         taubase.TauBaseWidget.getUseParentModel, 
                                         taubase.TauBaseWidget.setUseParentModel,
                                         taubase.TauBaseWidget.resetUseParentModel)

def main():
    import sys
    import tau
    attr_name = sys.argv[1]
    attr = tau.Attribute(attr_name)
    
    a = QtGui.QApplication([])
    panel = QtGui.QWidget()
    l = QtGui.QGridLayout()
    w1 = TauCircularGauge()
    w2 = TauLinearGauge()
    w1.setModel(attr_name)
    w2.setModel(attr_name)
    
    l.addWidget(w1,0,0)
    l.addWidget(w2,1,0)
    panel.setLayout(l)
    panel.setVisible(True)
    a.exec_()

if __name__ == "__main__":
    main()
