
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
graphics.py: 
"""

import Queue

from PyQt4 import QtGui, QtCore

#import tau.widget
#import src
#from tauPane import *

import tau.core
import tau.core.utils as utils
import taubase
import colors
import time
import sys
import signal
import re
from threading import Thread 
import os
import subprocess
import traceback

#from tauDevicePanel import tauDevicePanel
    
class TauGraphicsUpdateThread(QtCore.QThread):
    
    def __init__(self, parent=None):
        """Parent most not be None and must be a TauGraphicsScene!"""
        if not isinstance(parent, TauGraphicsScene):
            raise RuntimeError("Illegal parent for TauGraphicsUpdateThread")
        QtCore.QThread.__init__(self, parent)
        self.log = utils.Logger('TauGraphicsUpdateThread')

    def run(self):
        #print "run... - TauGraphicsUpdateThread"
        p = self.parent()
        while True:
            item = p.getQueue().get(True)
            for v in p.views():
                # The first one is the prefered one because it improves performance
                # since updates don't come very often in comparison to with the refresh
                # rate of the monitor (~70Hz)
                if v.viewportUpdateMode() == QtGui.QGraphicsView.NoViewportUpdate:
                    # We call the update to the viewport instead of the view 
                    #itself because apparently there is a bug in QT 4.3 that
                    #prevents a proper update when the view is inside a QTab 
                    v.viewport().update()
                else:
                    v.invalidateScene(item.boundingRect())

class mythread(QtCore.QThread):
    def __init__(self,command):
        #print "my thread init"
        QtCore.QThread.__init__(self)
        self.command = command
        self.alive = True
    def run(self):
        #print "run....." 
        while self.alive:
            #print "running.."
            #print "command = ",command
            time.sleep(1)
    def stop(self):
        #print "stop......"
        self.alive = False   

class newDialog(QtGui.QDialog):
    """ This class create the dialog """
    def __init__(self, parent = None):
        #print "newDialog init ....."
        QtGui.QDialog.__init__(self, parent)

    def initComponents(self,newWidget,dev_name,title):
        #print "init Components ...."
        self.setWindowTitle(QtGui.QApplication.translate("",title, None, QtGui.QApplication.UnicodeUTF8))
        self.resize(QtCore.QSize(QtCore.QRect(0,0,300,300).size()).expandedTo(self.minimumSizeHint()))
        palette = QtGui.QPalette()

        brush = QtGui.QBrush(QtGui.QColor(143,165,203))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active,QtGui.QPalette.Button,brush)

        brush = QtGui.QBrush(QtGui.QColor(255,255,255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive,QtGui.QPalette.Base,brush)

        self.setPalette(palette)

        widgetLayout = QtGui.QVBoxLayout(self)
        widgetLayout.setContentsMargins(10,10,10,10)

        if not dev_name is None:
            lineText = QtGui.QLabel("   Device Name:  ")
            editText = QtGui.QTextEdit()
            editText.setText(dev_name)
            editText.setMaximumHeight(24)
            self.hboxlayout = QtGui.QHBoxLayout()
            self.hboxlayout.setObjectName("hboxlayout")
            self.hboxlayout.addWidget(lineText)
            self.hboxlayout.addWidget(editText)
            widgetLayout.addLayout(self.hboxlayout)
            #gridLayout.addLayout(self.hboxlayout,0,0,1,1)

        widgetLayout.addWidget(newWidget)

    def closeEvent(self,QCloseEvent):
        #print "Closing new Dialog ................."
        pass

    
class TauGraphicsScene(QtGui.QGraphicsScene):
    __pyqtSignals__ = ("refreshTree2","graphicItemSelected(const QString &)")
    
    def __init__(self, parent = None, strt = True):
        name = self.__class__.__name__
        #self.call__init__(utils.Logger, name, parent) #Inheriting from Logger caused exceptions in CONNECT
        QtGui.QGraphicsScene.__init__(self, parent)
        self.updateQueue = None
        self.updateThread = None
        self.threads = []
        self.pids = []
        
        ## Redundant graphic loopback needed for blue selection mark in synoptics
        QtCore.QObject.connect(self, QtCore.SIGNAL("graphicItemSelected(const QString &)"), self, QtCore.SLOT("selectGraphicItem(const QString &)"))                    
        
        try:
            self.logger = utils.Logger(name)
            self.logger.setLogLevel(self.logger.Info)
            self.debug = lambda l: self.logger.debug(l)
            self.info = lambda l: self.logger.info(l)
            self.warning = lambda l: self.logger.warning(l)
            self.error = lambda l: self.logger.error(l)
        except:
            print 'Unable to initialize TauGraphicsSceneLogger: %s'%traceback.format_exc()       
        if strt:
            self.start()
            
    def mousePressEvent(self,mouseEvent):
        self.info('In TauGraphicsScene.mousePressEvent(%s))'%str(mouseEvent.button()))
        if (mouseEvent.button() == QtCore.Qt.LeftButton):
            x = mouseEvent.scenePos().x()
            y = mouseEvent.scenePos().y()
            obj = self.itemAt(x,y)
            obj_name = hasattr(obj,'_name') and obj._name or ''
            self.selectGraphicItem(obj_name) # A null obj_name should deselect all
            self.emit(QtCore.SIGNAL("graphicItemSelected(const QString &)"),obj_name) # A null obj_name should deselect all
            if hasattr(obj,'_extensions') and obj._extensions:
                self.getExtList(obj)
                if obj._extensions.has_key('shellCommand'):
                    self.getShellCommand(obj)
                if obj._extensions.has_key('className'):
                    self.getClassName(obj)
                else: pass 
                #   self.getTauDevicePanel(obj)
            else: pass
            #   self.getTauDevicePanel(obj)
        if (mouseEvent.button() == QtCore.Qt.RightButton):
            x = mouseEvent.scenePos().x()
            y = mouseEvent.scenePos().y()
            obj = self.itemAt(x,y)
            self.selectGraphicItem(hasattr(obj,'_name') and obj._name or '')
            ''' This function is called when right clicking on TauDevTree area. A pop up menu will be
            shown with the available options. '''
            menu = QtGui.QMenu(None)#self.parent)        
            if isinstance(obj,TauGraphicsItem):
                if obj.contextMenu():
                    menu.addSeparator()
                    last_was_separator = True
                    for t in obj.contextMenu(): #It must be a list of tuples (ActionName,ActionMethod)
                        try:
                            k,action = t
                            if k:
                                configDialogAction = menu.addAction(k)
                                if action: self.connect(configDialogAction, QtCore.SIGNAL("triggered()"), action)
                                else: configDialogAction.setEnabled(False)
                                last_was_separator = False
                            elif not last_was_separator: 
                                menu.addSeparator()
                                last_was_separator = True
                        except Exception,e: 
                            self.warning('Unable to add Menu Action: %s:%s'%(t,e))
                elif hasattr(obj,'_name') and str(obj._name).count('/')>=2:
                    def test_device(device): self.launchProcess('tg_devtest %s &'%device)
                    def show_atkpanel(device): self.launchProcess('atkpanelALBA %s &'%device)                     
                    action = menu.addAction('Show %s Panel'%obj._name)
                    self.connect(action, QtCore.SIGNAL("triggered()"), lambda dev=obj._name: show_atkpanel(dev))
                    action = menu.addAction('Test %s'%obj._name)
                    self.connect(action, QtCore.SIGNAL("triggered()"), lambda dev=obj._name: test_device(dev))                           
                menu.addSeparator()
                try: 
                    menu.exec_(QtCore.QPoint(mouseEvent.screenPos().x(),mouseEvent.screenPos().y()))
                except Exception,e:
                    self.error( traceback.format_exc())
            del menu
            
    def launchProcess(self,process):
        if not hasattr(self,'ChildrenProcesses'): self.ChildrenProcesses = {}
        if process in self.ChildrenProcesses: 
            self.warning( 'Process %s is already running!'%process)
            return
        self.ChildrenProcesses[process] = subprocess.Popen(process,shell=True)
        return
        
    def killProcess(self,regexp):
        if '*' in regexp and not '.*' in regexp:
            regexp = regexp.replace('*','.*')
        for name,process in self.ChildrenProcesses.iteritems():
            try:
                if re.match(regexp,name): process.terminate()
            except Exception,e: self.error( 'Unable to stop %s process: %s' % (name,str(e)))
        return
        
    @QtCore.pyqtSignature("selectGraphicItem(const QString &)")
    def selectGraphicItem(self,item_name):
        self.info('In TauGraphicsScene.selectGraphicItem(%s))'%item_name)
        if not hasattr(self,'SelectionCircle'):
            self.SelectionCircle = QtGui.QGraphicsEllipseItem()
            self.addItem(self.SelectionCircle)
        self.SelectionCircle.hide()
        if item_name:
            item_name = str(item_name).lower().strip().split()[0]
            for item in self.items():
                if hasattr(item,'_name') and str(item._name).lower() == item_name:
                    try:
                        if hasattr(item,'_extensions') and item._extensions and 'noSelect' in item._extensions:
                            return False
                        x,y = item.x(),item.y()
                        rect = item.boundingRect()
                        h,w = rect.height(),rect.width()
                        self.SelectionCircle.setRect(x-(.5*w),y-(.5*h),w*2,h*2)
                        color = QtGui.QColor(QtCore.Qt.blue)
                        color.setAlphaF(.10)
                        self.SelectionCircle.setBrush(color)
                        pen = QtGui.QPen(QtCore.Qt.CustomDashLine)
                        pen.setWidth(4)
                        pen.setColor(QtGui.QColor(QtCore.Qt.blue))
                        self.SelectionCircle.setPen(pen)
                        #self.SelectionCircle.setVisible(True)
                        #self.update(self.SelectionCircle.rect())
                        self.SelectionCircle.show()
                        self.info('> Moved the SelectionCircle to item %s at %f,%f' % (item._name,x,y))
                        return True
                    except Exception,e:
                        self.warning('selectGraphicsItem(%s) failed! %s' % (item_name,str(e)))
                        print traceback.format_exc()
                        return False
        self.warning('No matching object found for "%s"'%item_name)

    def getExtList(self,obj):
        self.standAlone = False
        self.noPrompt = False
        ExtList = ''
        if obj._extensions.has_key('ExtensionsList'):
            ExtList = obj._extensions['ExtensionsList']    
        ExtList = ExtList.split(',')
        self.noPrompt = obj._extensions.has_key('noPrompt') or 'noPrompt' in ExtList
        self.standAlone = obj._extensions.has_key('standAlone') or 'standAlone' in ExtList
        #for ext in ExtList:
            #if ext == "noPrompt":
                #self.noPrompt = True
            #if ext == "standAlone":
                #self.standAlone = True

    def getShellCommand(self,obj):
        shellCom = obj._extensions['shellCommand']
        if self.noPrompt ==True:
            subprocess.call(shellCom,shell=True)
        else:
            yes = QtGui.QMessageBox.Ok
            no = QtGui.QMessageBox.Cancel
            result = QtGui.QMessageBox.question(self.parent(),"Shell command","Would you like to call shell command '" +shellCom+ "' ?",yes, no)
            if result == yes:
                    subprocess.call(shellCom,shell=True) 

    def getClassName(self,obj):
        clName = obj._extensions['className']
        if clName == 'atkpanel.MainPanel' or clName =="atkpanel":
            #subprocess.Popen(['atkpanel', obj._name])
            self.getAtkPanel(obj)
        elif clName == 'noPanel': 
            #do nothing
            #print "        className = ",clName
            pass
        else:
            if obj._extensions.has_key('classParams'):
                clParam = obj._extensions['classParams']
                self.getClass(clName,clParam,obj._name)
            else:
                self.getClass(clName,obj._name,obj._name)

    def getAtkPanel(self,obj):
        tg_host = os.environ.get('TANGO_HOST')
        #print os.environ.get('JAVALIB')#return None but it should work?
        javalib = '/homelocal/sicilia/lib/java'
        classpath = '%s/atkpanel.jar:%s/ATKCore.jar:%s/ATKWidget.jar:%s/TangORB.jar'%(javalib,javalib,javalib,javalib)
        command = ['/usr/bin/java','-classpath', classpath, '-DTANGO_HOST=%s' % tg_host,'atkpanel.MainPanel',obj._name]
        p = subprocess.Popen(command)
        if self.standAlone == False:
            self.pids.append(p.pid)

    def getClass(self,clName,clParam,objName):
        if clName in globals():
            myclass = globals()[clName]
            #print "The class ",clName, " found in globals!"
        elif clName in locals():
            myclass = locals()[clName]
            #print "The class ",clName, " found in locals!"
        else:
            try:
                myclass = getattr(QtGui,clName)
                #print "The class ",clName, " found in QtGui!"
            except:
                try:
                    myclass = getattr(tau.widget,clName)
                    #print "The class ",clName, " found in tau.widget!"
                except:
                    self.warning( "The class ",clName, "can not be found!\n" + '-'*80)
                    return

        nameclass = myclass()
        try:nameclass.setClasses(clParam)
        except:pass
        try:nameclass.setModel(clParam)
        except:pass
        try: nameclass.setTable(clParam)
        except:pass

        if isinstance(nameclass,QtCore.QObject):
            if self.standAlone == False:
                obj = newDialog(self.parent())
            else:
                obj = newDialog()
    
            obj.initComponents(nameclass,objName,clName)
            obj.setModal(False)
            obj.setVisible(True)
            obj.exec_()
        return

    def getTauDevicePanel(self,obj):
        try:
            from tauDevicePanel import tauDevicePanel
        except:
            from tauPanel.tauDevicePanel import tauDevicePanel
        nameclass = tauDevicePanel()
        name = "TauDevicePanel"
        nameclass.setModel(obj._name)
        nameclass.setSpectraAtkMode(True)
        if self.standAlone == False:
            obj = newDialog(self.parent())
        else:
            obj = newDialog()

        dev_name = None
        obj.initComponents(nameclass,dev_name,name)
        obj.setModal(False)
        obj.setVisible(True)
        obj.exec_()

    def start(self):
        if self.updateThread:
            return
        self.updateQueue = Queue.Queue()
        self.updateThread = TauGraphicsUpdateThread(self)
        self.updateThread.start()#QtCore.QThread.HighPriority)
    
    def getQueue(self):
        return self.updateQueue
    
    def updateSceneItem(self, item):
        self.updateQueue.put(item)

    def updateScene(self):
        self.update()


#class TauGraphicsItem(QtCore.QObject,taubase.TauBaseComponent):
class TauGraphicsItem(taubase.TauBaseComponent):#,QtCore.QObject):
    """Base class for all Tau Graphics Items"""
    #__pyqtSignals__ = ("refreshIcon",)
    
    def __init__(self, name = None, parent = None):
        name = name or self.__class__.__name__
        self.call__init__(taubase.TauBaseComponent, name, parent)
        self._contextMenu = []
        
    def setContextMenu(self,menu):
        '''Context Menu must be a list of tuples (ActionName,ActionMethod), empty tuples insert separators between options.'''
        self._contextMenu = menu
        
    def contextMenu(self):
        return self._contextMenu
        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Mandatory methods to be implemented in any subclass of TauComponent
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getParentTauComponent(self):
        """ Returns a parent Tau component or None if no parent TauBaseComponent 
            is found."""
        p = self.parentItem()
        while p and not isinstance(p, TauGraphicsItem):
            p = self.parentItem()
        return p

    #def fireEvent(self, type):
    def fireEvent(self, evt_src = None, evt_type = None, evt_value = None):
        """fires a value changed event to all listeners"""
        self.updateStyle()

    def updateStyle(self):
        """ Method called when the component detects an event that triggers a change
            in the style."""
        if self.scene():
            self.scene().updateSceneItem(self)

    def isReadOnly(self):
        return True

    def __str__(self):
        return self.log_name + "(" + self.modelName + ")"

    def getModelClass(self):
        return tau.core.TauAttribute

class TauGraphicsStateItem2(TauGraphicsItem):

    def __init__(self, name = None, parent = None):
        name = name or self.__class__.__name__
        self.call__init__(TauGraphicsItem, name, parent)
        self._name = None
        self._currFgBrush = None
        self._currBgBrush = None
        self._currText = None
        self._map = None
        self._default = None
        self._visible = None
        self._extensions = None
        self.noTooltip = False
        self.ignoreRepaint = False
        self._currHtmlText = None
        self._unitVisible = True

    def updateStyle(self):
        v = self.getModelValueObj()

        self._currText = self.getDisplayValue()

        if  not self._map is None:
            #print  "    map is not null"
            if list[self._currText] == self._map[1]:
                self.setVisible(self._map[2])
                self._visible = self._map[2]
            else:
                self.setVisible(self._default)
                self._visible = self._default
        #else:
            #print "    map is null"
        TauGraphicsItem.updateStyle(self)

    def setModel(self, model):
        #print ('In TauGraphicsStateItem2(%s).setModel(%s)' % (type(self),model))
        model = str(model)    
        self._name = model
        TauGraphicsItem.setModel(self, model)        

    def getDisplayValue(self):
        #print 'In TauGraphicsStateItem2().getDisplayValue()'
        value = None
        try:
            attrvalue = self.getModelValueObj()
            if attrvalue:
                try: value = float(attrvalue.value) and "%.2e" %attrvalue.value
                except: value = "%s"%attrvalue.value
            else: value = self.getNoneValue()
            #value = str(attrvalue.value)
            #print 'In TauGraphicsStateItem2(%s).getDisplayValue()=%s' % (self._name,value)
        except Exception,e:
            self.error('Unexpected exception in getDisplayValue:' + str(e))
            self.traceback()
        return value

    def setExtensions(self):
        if self._extensions:
                ExtList = ''
                if self._extensions.has_key('ExtensionsList'):
                    ExtList = self._extensions['ExtensionsList']
                    ExtList = ExtList.split(',')
                    for ext in ExtList:
                        if ext == "noTooltip":
                            self.noTooltip = True
                        if ext == "ignoreRepaint":
                            self.ignoreRepaint = True
                if self.noTooltip == False:
                    self.setToolTip(self._name)
        else:
            self.setToolTip(self._name)

    def setUnitVisible(self,yesno):
        pass

class TauGraphicsAttributeItem(TauGraphicsItem):

    def __init__(self, name = None, parent = None):
        name = name or self.__class__.__name__
        self.call__init__(TauGraphicsItem, name, parent)
        self._currFgBrush = None
        self._currBgBrush = None
        self._currText = None
        self._currHtmlText = None
        self._unitVisible = True
        self._name = None
        self._map = None
        self._default = None
        self._visible = None
        self._extensions = None
        self.noTooltip = False
        self.ignoreRepaint = False
        self._currValue = None

    def getUnit(self):
        unit = ''
        modelObj = self.getModelObj()
        if not modelObj is None:
            unit = modelObj.getUnit() or '-'
        return unit

    def updateStyle(self):
        #print "updateStyle ->TauGraphicsAttributeItem"
        v = self.getModelValueObj()
        self._currText = self.getDisplayValue()
        
        if self.getShowQuality():
            quality = None
            if v:
                quality = v.quality
            self._currHtmlText = colors.QT_ATTRIBUTE_QUALITY_PALETTE.htmlStyle('TD',quality)
            self._currHtmlText += "<table cellpadding='1'><tr><td class='%s'>%s</td>" % (quality,self._currText)
            if self._unitVisible:
                self._currHtmlText += "<td>%s</td>" % self.getUnit()
            self._currHtmlText += "</tr></table>"
            self._currHtmlText = self._currHtmlText.decode('unicode-escape')
        else:
            self._currText += ' ' + unit
            self._currText = self._currText.decode('unicode-escape')
            self._currHtmlText = self.currText
        
        TauGraphicsItem.updateStyle(self)

    def setUnitVisible(self,yesno):
        self._unitVisible = yesno

    def setModel(self, model):
        #print ('In TauGraphicsAttributeItem.setModel(%s)'%model)
        model = str(model)
        self._name = model
        #model = "LT01/VC/PNV-03/status"
        TauGraphicsItem.setModel(self, model)

    def getDisplayValue(self):
        attrvalue = self.getModelValueObj()
        if not attrvalue:
            return self.getNoneValue()
        return str(attrvalue.value)

class TauGraphicsStateItem(TauGraphicsItem):

    def __init__(self, name = None, parent = None):
        name = name or self.__class__.__name__
        self.call__init__(TauGraphicsItem, name, parent)
        self._name = None
        self._currFgBrush = None
        self._currBgBrush = None
        self._currText = None
        self._map = None
        self._default = None
        self._visible = None
        self._extensions = None
        self.noTooltip = False
        self.ignoreRepaint = False

    def updateStyle(self):
        v = self.getModelValueObj()
        if v:
            v = v.value
        self._currText = self.getDisplayValue()
        bg_brush, fg_brush = colors.QT_DEVICE_STATE_PALETTE.qbrush(v)

        if self.getShowQuality():
            self._currBgBrush = bg_brush
            self._currFgBrush = fg_brush
        else:
            self._currBrush = QtGui.QBrush(QtCore.Qt.NoBrush)

        list = {'ON':0,'OFF':1,'CLOSE':2,'OPEN':3,'INSERT':4,'EXTRACT':5,'MOVING':6,'STANDBY':7,'FAULT':8,'INIT':9,'RUNNING':10,'ALARM':11,'DISABLE':12,'UNKNOWN':13}

        if  not self._map is None:
            if list[self._currText] == self._map[1]:
                self.setVisible(self._map[2])
                self._visible = self._map[2]
            else:
                self.setVisible(self._default)
                self._visible = self._default

        TauGraphicsItem.updateStyle(self)

    def setModel(self, model):
        model = str(model)
        self._name = model

        if not model.lower().endswith('/state'):
            model += '/state'

        TauGraphicsItem.setModel(self, model)

    def getDisplayValue(self):
        attrvalue = self.getModelValueObj()
        if not attrvalue:
            return self.getNoneValue()
        return str(attrvalue.value)

    def setExtensions(self):
        if self._extensions:
                ExtList = ''
                if self._extensions.has_key('ExtensionsList'):
                    ExtList = self._extensions['ExtensionsList']
                    ExtList = ExtList.split(',')
                    for ext in ExtList:
                        if ext == "noTooltip":
                            self.noTooltip = True
                        if ext == "ignoreRepaint":
                            self.ignoreRepaint = True
                if self.noTooltip == False:
                    self.setToolTip(self._name)
        else:
            self.setToolTip(self._name)

    def setUnitVisible(self,yesno):
        pass

class TauEllipseStateItem(QtGui.QGraphicsEllipseItem, TauGraphicsStateItem):

    def __init__(self, name = None, parent = None, scene = None):
        name = name or self.__class__.__name__
        QtGui.QGraphicsEllipseItem.__init__(self, parent, scene)
        self.call__init__(TauGraphicsStateItem, name, parent)

    def paint(self,painter,option,widget = None):
        self.setExtensions()
        if self._currBgBrush:
            self.setBrush(self._currBgBrush)
        QtGui.QGraphicsEllipseItem.paint(self,painter,option,widget)


class TauRectStateItem(QtGui.QGraphicsRectItem, TauGraphicsStateItem):

    def __init__(self, name = None, parent = None, scene = None):
        name = name or self.__class__.__name__
        QtGui.QGraphicsRectItem.__init__(self, parent, scene)
        self.call__init__(TauGraphicsStateItem, name, parent)

    def paint(self,painter,option,widget):
        self.setExtensions()
        if self._currBgBrush:
            self.setBrush(self._currBgBrush)
        QtGui.QGraphicsRectItem.paint(self,painter,option,widget)

class TauGroupStateItem(QtGui.QGraphicsItemGroup, TauGraphicsStateItem):#, QtCore.QObject):

    def __init__(self, name = None, parent = None, scene = None):
        name = name or self.__class__.__name__
        QtGui.QGraphicsItemGroup.__init__(self, parent, scene)
        self.call__init__(TauGraphicsStateItem, name, parent)

    def paint(self,painter,option,widget):
        QtGui.QGraphicsItemGroup.paint(self,painter,option,widget)

        
class TauPolygonStateItem(QtGui.QGraphicsPolygonItem,TauGraphicsStateItem):
    
    def __init__(self, name = None, parent = None, scene = None):
        name = name or self.__class__.__name__
        #QtGui.QGraphicsRectItem.__init__(self, parent, scene)
        QtGui.QGraphicsPolygonItem.__init__(self, parent, scene)
        self.call__init__(TauGraphicsStateItem, name, parent)
        
    def paint(self,painter,option,widget):
        self.setExtensions()
        if self._currBgBrush:
            self.setBrush(self._currBgBrush)
        QtGui.QGraphicsPolygonItem.paint(self,painter,option,widget)


class TauTextStateItem(QtGui.QGraphicsTextItem, TauGraphicsStateItem):
    
    def __init__(self, name = None, parent = None, scene = None):
        name = name or self.__class__.__name__
        QtGui.QGraphicsTextItem.__init__(self, parent, scene)
        self.call__init__(TauGraphicsStateItem, name, parent)
        
    def paint(self,painter,option,widget):
        self.setPlainText(self._currText)
        QtGui.QGraphicsTextItem.paint(self,painter,option,widget)


class TauTextAttributeItem(QtGui.QGraphicsTextItem, TauGraphicsStateItem2, QtGui.QGraphicsRectItem):
    """A QGraphicsItem that represents the text"""
    
    def __init__(self, name = None, parent = None, scene = None):
        name = name or self.__class__.__name__
        QtGui.QGraphicsTextItem.__init__(self, parent, scene)
        self.call__init__(TauGraphicsStateItem2, name, parent)
        
    def paint(self,painter,option,widget):
        if self._currText:
            text2 = "<body bgcolor='lime' style ='font-size:14px;margin-top:60px;'>"+self._currText+"</font></body>"
            self.setHtml(text2)
        QtGui.QGraphicsTextItem.paint(self,painter,option,widget)


import tau.core

TYPE_TO_GRAPHICS = { 
    None : { "Rectangle"      : QtGui.QGraphicsRectItem,
             "RoundRectangle" : QtGui.QGraphicsRectItem,
             "Ellipse"        : QtGui.QGraphicsEllipseItem,
             "Polyline"       : QtGui.QGraphicsPolygonItem,
             "Label"          : QtGui.QGraphicsTextItem,
             "Line"           : QtGui.QGraphicsLineItem,
             "Group"          : QtGui.QGraphicsItemGroup, 
             "SwingObject"    : QtGui.QGraphicsRectItem, 
             "Image"          : QtGui.QGraphicsPixmapItem, },
             
    tau.core.TauDevice : { "Rectangle"      : TauRectStateItem,
                           "RoundRectangle" : TauRectStateItem,
                           "Ellipse"        : TauEllipseStateItem,
                           "Polyline"       : TauPolygonStateItem, 
                           "Label"          : TauTextStateItem,
                           "Line"           : QtGui.QGraphicsLineItem,
                           "Group"          : TauGroupStateItem, 
                           "SwingObject"    : TauTextStateItem,
                           "Image"          : QtGui.QGraphicsPixmapItem, },

    tau.core.TauAttribute : { "Rectangle"      : QtGui.QGraphicsRectItem,
                              "RoundRectangle" : QtGui.QGraphicsRectItem,
                              "Ellipse"        : QtGui.QGraphicsEllipseItem,
                              "Polyline"       : QtGui.QGraphicsPolygonItem, 
                              "Label"          : QtGui.QGraphicsTextItem,
                              "Line"           : QtGui.QGraphicsLineItem,
                              "Group"          : QtGui.QGraphicsItemGroup,
                              "SwingObject"    : TauTextAttributeItem,
                              "Image"          : QtGui.QGraphicsPixmapItem, },    
}


class AbstractGraphicsFactory:
    
    def __init__(self):
        pass
    
    def getSceneObj(self):   
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getSceneObj()")
    
    def getObj(self,name,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getObj()")

    def getRectangleObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getRectangleObj()")

    def getRoundRectangleObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getRoundRectangleObj()")

    def getLineObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getLineObj()")
    
    def getEllipseObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getEllipseObj()")

    def getPolylineObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getPolylineObj()")
    
    def getLabelObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getLabelObj()")
    
    def getGroupObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getGroupObj()")
    
    def getSwingObjectObj(self,params):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getSwingObjectObj()")
    
    def getImageObj(self,parms):
        raise RuntimeError("Invalid call to AbstractGraphicsFactory::getImageObj()")
    
    def getGraphicsClassItem(self,cls,type):
        try:
            if   issubclass(cls, tau.core.TauDevice):    cls = tau.core.TauDevice
            elif issubclass(cls, tau.core.TauAttribute): cls = tau.core.TauAttribute
        except:
            pass
        return TYPE_TO_GRAPHICS.get(cls,TYPE_TO_GRAPHICS.get(None)).get(type)

    def getGraphicsItem(self,type,params):
        name = params.get(self.getNameParam())
        cls = None
        cls = tau.core.TauManager().findObjectClass(name)
        #if cls: print 'In graphics.getGraphicsItem(%s): %s has class %s' % (str(type),name,cls)
        grph_cls = self.getGraphicsClassItem(cls, type)
        item = grph_cls()
        ## It's here were Attributes are subscribed
        self.set_common_params(item,params)
        #if cls: print 'In graphics.getGraphicsItem(%s): %s has class %s and graph class %s' % (str(type),name,cls,item)
        return item

    def getNameParam(self):
        """Returns the name of the parameter which contains the name identifier.
           Default implementation returns 'name'.
           Overwrite has necessary."""
        return 'name'
    
    def set_common_params(self,item,params):
        """Sets the common parameters. Default implementation does nothing.
           Overwrite has necessary."""
        pass
    
#if __name__ == "__main__":
#    import sys
#    app = QtGui.QApplication(sys.argv)
#    try:
#        from tauDevicePanel import *
#    except:
#        from tauPanel.tauDevicePanel import *
#    cos = tauDevicePanel()
#    print "ala ma kota"
#    cos.show()
#    sys.exit(app.exec_())
