
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
leds.py: 
"""

#!/usr/bin/env python

from PyQt4 import QtCore, QtGui, Qt
import led
from tau.widget import TauBaseWidget
import tau.core
import PyTango


class TauBoolLed(led.Led, TauBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent = None, designMode = False):
        self._ledColor = 'GREEN'
        self._ledColorOff = 'GREENOFF'
        self._ledSize = 24
        self._ledColorPixmap = None
        self._ledColorOffPixmap = None

        self.call__init__wo_kw(led.Led, parent)
        self.call__init__(TauBaseWidget, str(self.objectName()), designMode=designMode)

        self.setLedColor(self._ledColor)
        self.setLedColorOff(self._ledColorOff)
        
        self.setEventFilters([tau.core.utils.EventFilters.IGNORE_CONFIG]) #This widget does not need to attend to config events

    def sizeHint(self):
        return QtCore.QSize(24, 24)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget overwritting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getModelClass(self):
        return tau.core.TauAttribute

    def isReadOnly(self):
        return True

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getLedColor(self):
        return self._ledColor

    @QtCore.pyqtSignature("setLedColor(QString)")
    def setLedColor(self, lc):
        self._ledColor = str(lc)

        status = led.LedStatus.ON
        color = self._ledColor
        if color.endswith('OFF'):
            status = led.LedStatus.OFF
            color = color[:-3]
        color = led.LedColor[color]
        file_name = self.toCompleteLedName(self.ledsize, status, color)
        self._ledColorPixmap = QtGui.QPixmap(file_name)
        
    def resetLedColor(self,lc):
        self.setLedColor('GREEN')

#-----------

    def getLedColorOff(self):
        return self._ledColorOff

    @QtCore.pyqtSignature("setLedColorOff(QString)")
    def setLedColorOff(self, lc):
        self._ledColorOff = str(lc)

        status = led.LedStatus.ON
        color = self._ledColorOff
        if color.endswith('OFF'):
            status = led.LedStatus.OFF
            color = color[:-3]

        color = led.LedColor[color]
        file_name = self.toCompleteLedName(self.ledsize, status, color)
        self._ledColorOffPixmap = QtGui.QPixmap(file_name)
        
    def resetLedColorOff(self,lc):
        self.setLedColorOff('GREENOFF')

#-----------

    def getLedSize(self):
        return self._ledSize

    @QtCore.pyqtSignature("setLedSize(int)")
    def setLedSize(self,size):
        self._ledSize = size
        try:
            getattr(led.LedSize,str(self._ledSize))
            self.changeSize(self._ledSize)
        except:
            pass

    def resetLedSize(self,size):
        self.setLedSize(24)

    model = QtCore.pyqtProperty("QString", TauBaseWidget.getModel,
                                TauBaseWidget.setModel,
                                TauBaseWidget.resetModel)

    useParentModel = QtCore.pyqtProperty("bool",
                                         TauBaseWidget.getUseParentModel,
                                         TauBaseWidget.setUseParentModel,
                                         TauBaseWidget.resetUseParentModel)

    ledColor = QtCore.pyqtProperty("QString", getLedColor, setLedColor,
                                   resetLedColor,
                                   doc='valid values are (case insensitive) "BLUE", "GREEN", "RED", "YELLOW", "ORANGE", "BLUEOFF", "GREENOFF", "REDOFF", "YELLOWOFF", "ORANGEOFF"')

    ledColorOff = QtCore.pyqtProperty("QString", getLedColorOff, setLedColorOff,
                                      resetLedColorOff,
                                      doc='valid values are (case insensitive) "BLUE", "GREEN", "RED", "YELLOW", "ORANGE", "BLUEOFF", "GREENOFF", "REDOFF", "YELLOWOFF", "ORANGEOFF"')

    ledSize = QtCore.pyqtProperty("int", getLedSize, setLedSize, resetLedSize,
                                  doc='valid values as 24 and 48')

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT signal handlers
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-


    def handleEvent(self, src, type_, evt_value):
        if evt_value is None:
            return
        if type_ == tau.core.TauEventType.Error:
            self._setProblemsBackground(True)
            self.updateStyle()
            return
        self._setProblemsBackground(False)

        if evt_value.value: #we know that evt_value will always be a DeviceAttribute because we filter out Config events
            self.setPixmap(self._ledColorPixmap)
        else:
            self.setPixmap(self._ledColorOffPixmap)
        self.updateStyle()


class TauStateLed(led.Led, TauBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    DEVICE_STATE_DATA = {
        PyTango.DevState.ON      : (led.LedStatus.ON,   led.LedColor.GREEN),
        PyTango.DevState.OFF     : (led.LedStatus.OFF,  led.LedColor.GREEN),
        PyTango.DevState.CLOSE   : (led.LedStatus.ON,   led.LedColor.GREEN),
        PyTango.DevState.OPEN    : (led.LedStatus.ON,   led.LedColor.GREEN),
        PyTango.DevState.INSERT  : (led.LedStatus.ON,   led.LedColor.GREEN),
        PyTango.DevState.EXTRACT : (led.LedStatus.ON,   led.LedColor.GREEN),
        PyTango.DevState.MOVING  : (led.LedStatus.ON,   led.LedColor.BLUE),
        PyTango.DevState.STANDBY : (led.LedStatus.ON,   led.LedColor.YELLOW),
        PyTango.DevState.FAULT   : (led.LedStatus.ON,   led.LedColor.RED),
        PyTango.DevState.INIT    : (led.LedStatus.ON,   led.LedColor.YELLOW),
        PyTango.DevState.RUNNING : (led.LedStatus.ON,   led.LedColor.BLUE),
        PyTango.DevState.ALARM   : (led.LedStatus.ON,   led.LedColor.ORANGE),
        PyTango.DevState.DISABLE : (led.LedStatus.OFF,  led.LedColor.ORANGE),
        PyTango.DevState.UNKNOWN : (led.LedStatus.OFF,  led.LedColor.BLUE),
        None                     : (led.LedStatus.OFF,  led.LedColor.RED)}

    def __init__(self, parent = None, designMode = False):
        name = self.__class__.__name__
        self.call__init__wo_kw(led.Led, parent)
        self.call__init__(TauBaseWidget, name, designMode=designMode)

    # The minimum size of the widget (a limit for the user)
    def minimumSizeHint(self):
        return QtCore.QSize(11, 11)

    # The default size of the widget
    def sizeHint(self):
        return QtCore.QSize(22, 22)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget overwritting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getModelClass(self):
        import tau.core
        return tau.core.TauAttribute

    def isReadOnly(self):
        return True

    def getFormatedToolTip(self, cache=True):
        """ The tooltip should refer to the device and not the state attribute.
            That is why this method is being rewritten
        """
        if self.modelObj is None:
            return self.getNoneValue()
        parent = self.modelObj.getParentObj()
        if parent is None:
            return self.getNoneValue()
        return self.toolTipObjToStr( parent.getDisplayDescrObj() )

    def handleEvent(self, src, type_, evt_value):

        if type_ == tau.core.TauEventType.Error:
            self._setProblemsBackground(True)
            self.updateStyle()
            return
        self._setProblemsBackground(False)
    
        v = getattr(evt_value, 'value',                                  
                        getattr(self.getModelValueObj(), 'value', None)) #tries to get the value from the event itself, and asks if not

        onOff,state = TauStateLed.DEVICE_STATE_DATA[v]
        if onOff == led.LedStatus.OFF:
            self.off()
        else:
            self.on()
        self.changeColor(state)
        
        #update tooltip
        self.setToolTip(self.getFormatedToolTip())
        
        #TODO: update whatsThis
        
        #update appearance
        self.updateStyle()

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getLedSize(self):
        return self.ledsize

    @QtCore.pyqtSignature("setLedSize(QString)")
    def setLedSize(self,size):
        self.changeSize(size)

    def resetLedSize(self,size):
        self.changeSize('24')

    model = QtCore.pyqtProperty("QString", TauBaseWidget.getModel,
                                TauBaseWidget.setModel,
                                TauBaseWidget.resetModel)

    useParentModel = QtCore.pyqtProperty("bool",
                                         TauBaseWidget.getUseParentModel,
                                         TauBaseWidget.setUseParentModel,
                                         TauBaseWidget.resetUseParentModel)

    ledSize = QtCore.pyqtProperty("int", getLedSize, setLedSize, resetLedSize,
                                  doc='valid values as 24 and 48')


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    
    model = 'fbecheri/rfplant/1/led1'
    if len(sys.argv)>1: model=sys.argv[1]
    
    wklass = TauBoolLed
    if model.lower().endswith('state'):
        wklass = TauStateLed
    
    w = wklass()
#    w.setLedColorOff('RED')
    #w.setLedColor('BLUE')
    w.setModel(model)
    
    w.setVisible(True)
    
    sys.exit(app.exec_())

