
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
limitswitch.py: 
"""

#!/usr/bin/env python

from PyQt4 import QtCore, QtGui, Qt
import PyTango
import tau.core
from led import *

from tau.widget import TauBaseWidget

class TauLimitSwitch(Led, TauBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent = None, designMode = False):
        self._ledColor = 'RED'
        self._ledSize = 24
        self._boolIndex = 0
        self.call__init__wo_kw(Led, parent)
        self.call__init__(TauBaseWidget, str(self.objectName()), designMode=designMode)

    def sizeHint(self):
        return QtCore.QSize(22, 22)
    
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget overwritting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    
    def getModelClass(self):
        return tau.core.TauAttribute

    def isReadOnly(self):
        return True

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getLedColor(self):
        return self._ledColor

    @QtCore.pyqtSignature("setLedColor(QString)")
    def setLedColor(self, lc):
        self._ledColor = str(lc)
        try:
            c = getattr(LedColor,self._ledColor)
            self.changeColor(c)
        except:
            pass
    
    def resetLedColor(self,lc):
        self.setLedColor('RED')

    def getLedSize(self):
        return self._ledSize
    
    @QtCore.pyqtSignature("setLedSize(int)")
    def setLedSize(self,size):
        self._ledSize = size
        try:
            getattr(LedSize,str(self._ledSize))
            self.changeSize(self._ledSize)
        except:
            pass

    def resetLedSize(self,size):
        self.setLedSize(24)


    def getBoolIndex(self):
        return self._boolIndex

    @QtCore.pyqtSignature("setBoolIndex(int)")
    def setBoolIndex(self,i):
        self._boolIndex = i
        try:
            getattr(BoolIndex,str(self._boolIndex))
            self.changeSize(self._boolIndex)
        except:
            pass

    def resetBoolIndex(self):
        self.setBoolIndex(0)

    model = QtCore.pyqtProperty("QString", TauBaseWidget.getModel, 
                                TauBaseWidget.setModel, TauBaseWidget.resetModel)
    useParentModel = QtCore.pyqtProperty("bool", TauBaseWidget.getUseParentModel, 
                                         TauBaseWidget.setUseParentModel,
                                         TauBaseWidget.resetUseParentModel)
    ledColor = QtCore.pyqtProperty("QString", getLedColor, setLedColor,
                                   resetLedColor, doc='valid values are (case insensitive) "BLUE", "GREEN", "RED", "YELLOW", "ORANGE"')
    ledSize = QtCore.pyqtProperty("int", getLedSize, setLedSize, resetLedSize,
                                  doc='valid values as 24 and 48')
    boolIndex = QtCore.pyqtProperty("int", getBoolIndex, setBoolIndex, resetBoolIndex,
                                  doc='valid values are 0,1,2')

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT signal handlers
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    def handleEvent(self, evt_src, evt_type, evt_value):
        #v = self.getModelValueObj()
        if evt_type not in [tau.core.TauEventType.Change, tau.core.TauEventType.Periodic]:
            return
        v = evt_value
        if v:
            v = v.value[self._boolIndex]
            if v:
                self.on()
            else:
                self.off()
            self.updateStyle()
        else:
            pass
