import tau
from tau import Device
from tau.widget import TauAttrListComboBox, TauBaseWidget 
import PyTango
from PyQt4 import Qt
from tau.widget.resources import qrc_extra_icons_leds_images24

class ConfigurationDialog(Qt.QDialog):
    
    def __init__(self, parent, initMacroServer=None, initDoor=None):
        Qt.QDialog.__init__(self,parent)
        self.initMacroServer = initMacroServer
        self.initDoor = initDoor
        self.initComponents()
        
    def initComponents(self):
        self.setModal(True)
        macroServerLabel = Qt.QLabel("MacroServer:", self)
        self.macroServerComboBox = self.createComboBox(self)
        doorLabel = Qt.QLabel("Door:", self)
        self.doorComboBox = TauAttrListComboBox(self) 
        self.doorComboBox.setModel(self.macroServerComboBox.currentText() + "/doorList")
        if not self.initDoor is None:
            index = self.doorComboBox.findText(self.initMacroServer) 
            if index != -1:
                self.doorComboBox.setCurrentIndex(index)
        self.buttonBox = Qt.QDialogButtonBox(Qt.QDialogButtonBox.Ok|
                                                Qt.QDialogButtonBox.Cancel)
        gridLayout = Qt.QGridLayout()
        gridLayout.addWidget(macroServerLabel,0,0)
        gridLayout.addWidget(self.macroServerComboBox,0,1)
        gridLayout.addWidget(doorLabel,1,0)
        gridLayout.addWidget(self.doorComboBox,1,1)
        
        self.setLayout(Qt.QVBoxLayout())
        self.layout().addLayout(gridLayout)
        self.layout().addWidget(self.buttonBox)
        self.adjustSize()
        
        self.connect(self.buttonBox, Qt.SIGNAL("accepted()"), self, Qt.SLOT("accept()"))
        self.connect(self.buttonBox, Qt.SIGNAL("rejected()"), self, Qt.SLOT("reject()"))
        self.connect(self.macroServerComboBox, Qt.SIGNAL("currentIndexChanged(const QString&)"),self.onMacroServerComboBoxChanged)
        
    def createComboBox(self, parent):
        """Create combobox with list of all macroservers on tango host,
           Next to text in combobox is a icon which depicts state of macroserver"""
        comboBox = Qt.QComboBox(parent)
        f = tau.core.TauManager().getFactory()()
        db = f.getDatabase()
        macroServerList = db.getValueObj().get_device_name('*','MacroServer')
        for macroServer in macroServerList:
            state = Device(macroServer).getState()
            icon = None
            if state == PyTango.DevState.ON:
                icon = Qt.QIcon(":/leds/images24/ledgreen.png")
            elif state == PyTango.DevState.FAULT:
                icon = Qt.QIcon(":/leds/images24/ledred.png")
            elif state == None:
                icon = Qt.QIcon(":/leds/images24/ledredoff.png")
            comboBox.addItem(icon, macroServer)
            
            if not self.initMacroServer is None:
                index = comboBox.findText(self.initMacroServer) 
                if index != -1:
                    comboBox.setCurrentIndex(index)
        return comboBox
    
    def onMacroServerComboBoxChanged(self, macroServerName):
        self.doorComboBox.setModel(macroServerName + "/doorList")
        
        
def addActions(target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)
                
def createAction(parent, 
                     text, 
                     slot=None, 
                     shortcut=None, 
                     icon=None, 
                     tip=None,
                     checkable=False,
                     signal="triggered()"):
        """Create an action for usage of main application (menu, toolbar, event actions)"""
        action = Qt.QAction(text, parent)
        if icon is not None:
            action.setIcon(Qt.QIcon(icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            parent.connect(action, Qt.SIGNAL(signal), slot)
        if checkable:
            action.setCheckable(True)
        return action
    
class MacroCommon:
    
    def __init__(self):
        self._doorName = ""
    
    def doorName(self):
        return self._doorName
    
    def setDoorName(self, doorName):
        self._doorName = doorName
#        self.emit(Qt.SIGNAL("doorChanged"),doorName)
        
    def setModel(self, model):
        """Sets new model for application, and change window title witn new macroserver name."""
        TauBaseWidget.setModel(self, model)
        self.setWindowTitle(Qt.QApplication.applicationName() + ": " + model)

    def changeConfiguration(self):
        """This method is used to change macroserver as a model of application.
           It shows dialog with list of all macroservers on tango host, if the user
           Cancel dialog it doesn't do anything."""
        dialog = ConfigurationDialog(self, self.modelName, self.doorName())
        if dialog.exec_():
            self.setModel(str(dialog.macroServerComboBox.currentText()))
            self.emit(Qt.SIGNAL("doorChanged"), str(dialog.doorComboBox.currentText()))
#            self.setDoorName(str(dialog.doorComboBox.currentText()))
        else:
            return