
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
dooroutput.py: 
"""

from PyQt4 import QtGui,QtCore
import tau.core
        
class DoorOutput(QtGui.QTextEdit):
    
    def __init__(self, parent = None):
        QtGui.QTextEdit.__init__(self, parent)
    
        self._doorName = None

        self._doorOutputListener = None
        self._doorWarningListener = None
        self._doorInfoListener = None
        self._doorErrorListener = None
        
        self.setReadOnly(True)
        self.setFont(QtGui.QFont("Courier",9))
        
    def doorName(self):
        return self._doorName
    
    def setDoorName(self, doorName):
        self._doorName = doorName
        
    def doorOutputListener(self):
        return self._doorOutputListener
    
    def setDoorOutputListener(self, doorOutputListener):
        self._doorOutputListener = doorOutputListener
        
    def doorWarningListener(self):
        return self._doorWarningListener
    
    def setDoorWarningListener(self, doorWarningListener):
        self._doorWarningListener = doorWarningListener
        
    def doorInfoListener(self):
        return self._doorInfoListener
    
    def setDoorInfoListener(self, doorInfoListener):
        self._doorInfoListener = doorInfoListener
        
    def doorErrorListener(self):
        return self._doorErrorListener
    
    def setDoorErrorListener(self, doorErrorListener):
        self._doorErrorListener = doorErrorListener

#    def sizeHint(self):
#        return self.minimumSizeHint()
#
#    def minimumSizeHint(self):
#        return QtCore.QSize(600,150)
        
    def onDoorChanged(self, doorName):
        self.setDoorName(doorName)
        self.resetListeners()
    
    def resetListeners(self):
        
        if not self.doorOutputListener() is None:
            QtCore.QObject.disconnect(self.doorOutputListener(),
                                      QtCore.SIGNAL("doorOutputChanged"),
                                      self.onDoorOutputChanged)
        if not self.doorInfoListener() is None:
            QtCore.QObject.disconnect(self.doorInfoListener(),
                                      QtCore.SIGNAL("doorInfoChanged"),
                                      self.onDoorInfoChanged)
        if not self.doorWarningListener() is None:
            QtCore.QObject.disconnect(self.doorWarningListener(),
                                      QtCore.SIGNAL("doorWarningChanged"),
                                      self.onDoorWarningChanged)
        if not self.doorErrorListener() is None:
            QtCore.QObject.disconnect(self.doorErrorListener(),
                                      QtCore.SIGNAL("doorErrorChanged"),
                                      self.onDoorErrorChanged)
            
        self.clear()
            
        self.setDoorOutputListener(DoorAttrListener("Output"))
        self.setDoorInfoListener(DoorAttrListener("Info"))
        self.setDoorWarningListener(DoorAttrListener("Warning"))
        self.setDoorErrorListener(DoorAttrListener("Error"))
        
        QtCore.QObject.connect(self.doorOutputListener(),
                               QtCore.SIGNAL("doorOutputChanged"),
                               self.onDoorOutputChanged)
        QtCore.QObject.connect(self.doorInfoListener(),
                               QtCore.SIGNAL("doorInfoChanged"),
                               self.onDoorInfoChanged)
        QtCore.QObject.connect(self.doorWarningListener(),
                               QtCore.SIGNAL("doorWarningChanged"),
                               self.onDoorWarningChanged)
        QtCore.QObject.connect(self.doorErrorListener(),
                               QtCore.SIGNAL("doorErrorChanged"),
                               self.onDoorErrorChanged)
        
        self.doorOutputListener().setDoorName(self.doorName())
        self.doorInfoListener().setDoorName(self.doorName())
        self.doorWarningListener().setDoorName(self.doorName())
        self.doorErrorListener().setDoorName(self.doorName())
    
    def onDoorOutputChanged(self, eventValue):
        """call on output attribute changed"""
        if eventValue is None: return
        self.setTextColor(QtCore.Qt.black)
        for outputLine in eventValue:
            self.append("OUTPUT  " + outputLine)
        self.moveCursor(QtGui.QTextCursor.End)
    
    def onDoorInfoChanged(self, eventValue):        
        """call on info attribute changed"""
        if eventValue is None: return
        self.setTextColor(QtCore.Qt.black)
        for infoLine in eventValue:
            self.append("INFO  " + infoLine)
        self.moveCursor(QtGui.QTextCursor.End)
        
    def onDoorErrorChanged(self, eventValue):
        """call on error attribute changed"""
        if eventValue is None: return
        self.setTextColor(QtCore.Qt.red)
        for errorLine in eventValue:
            self.append("ERROR  " + errorLine)
        self.moveCursor(QtGui.QTextCursor.End)
                
    def onDoorWarningChanged(self, eventValue):
        """call on warning attribute changed"""        
        if eventValue is None: return
        self.setTextColor(QtCore.Qt.black)
        for warningLine in eventValue:
            self.append("WARNING  " + warningLine)
        self.moveCursor(QtGui.QTextCursor.End)
        
    def contextMenuEvent(self,event):
        menu = self.createStandardContextMenu() 
        clearAction = QtGui.QAction("Clear", menu)
        menu.addAction(clearAction)
        if not len(self.toPlainText()):
            clearAction.setEnabled(False) 
        
        QtCore.QObject.connect(clearAction, QtCore.SIGNAL("triggered()"), self.clear)               
        menu.exec_(event.globalPos())
 
        
class DoorDebug(QtGui.QTextEdit):
    
    def __init__(self, parent = None):
        QtGui.QTextEdit.__init__(self, parent)
        
        self._doorName = None

        self._doorDebugListener = None
        
        self.setReadOnly(True)
        self.setFont(QtGui.QFont("Courier",9))
    
    def doorName(self):
        return self._doorName
    
    def setDoorName(self, doorName):
        self._doorName = doorName
        
    def doorDebugListener(self):
        return self._doorDebugListener
    
    def setDoorDebugListener(self, doorDebugListener):
        self._doorDebugListener = doorDebugListener

#    def sizeHint(self):
#        return self.minimumSizeHint()
#
#    def minimumSizeHint(self):
#        return QtCore.QSize(600,150)
    
    def onDoorChanged(self, doorName):
        self.setDoorName(doorName)
        self.resetListeners()
    
    def resetListeners(self):
        
        if not self.doorDebugListener() is None:
            QtCore.QObject.disconnect(self.doorDebugListener(),
                                      QtCore.SIGNAL("doorDebugChanged"),
                                      self.onDoorDebugChanged)
            
        self.clear()
            
        self.setDoorDebugListener(DoorAttrListener("Debug"))
        
        QtCore.QObject.connect(self.doorDebugListener(),
                               QtCore.SIGNAL("doorDebugChanged"),
                               self.onDoorDebugChanged)
        
        self._doorDebugListener.setDoorName(self._doorName)
        
    def onDoorDebugChanged(self, eventValue):
        """call on debug attribute changed"""
        if eventValue is None: return
        for debugLine in eventValue: 
            self.append("DEBUG  " + debugLine)
        self.moveCursor(QtGui.QTextCursor.End)
        
    def contextMenuEvent(self,event):
        menu = self.createStandardContextMenu() 
        clearAction = QtGui.QAction("Clear", menu)
        menu.addAction(clearAction)
        if not len(self.toPlainText()):
            clearAction.setEnabled(False) 
        
        QtCore.QObject.connect(clearAction, QtCore.SIGNAL("triggered()"), self.clear)               
        menu.exec_(event.globalPos())


class DoorResult(QtGui.QTextEdit):
    
    def __init__(self, parent = None):
        QtGui.QTextEdit.__init__(self, parent)
        
        self._doorName = None

        self._doorResultListener = None
        
        self.setReadOnly(True)
        self.setFont(QtGui.QFont("Courier",9))
        
    def doorName(self):
        return self._doorName
    
    def setDoorName(self, doorName):
        self._doorName = doorName
        
    def doorResultListener(self):
        return self._doorResultListener
    
    def setDoorResultListener(self, doorResultListener):
        self._doorResultListener = doorResultListener

#    def sizeHint(self):
#        return self.minimumSizeHint()
#
#    def minimumSizeHint(self):
#        return QtCore.QSize(600,150)
    
    def onDoorChanged(self, doorName):
        self.setDoorName(doorName)
        self.resetListeners()
    
    def resetListeners(self):
        if not self.doorResultListener() is None:
            QtCore.QObject.disconnect(self.doorResultListener(),
                                      QtCore.SIGNAL("doorResultChanged"),
                                      self.onDoorResultChanged)
            
        self.clear()
            
        self.setDoorResultListener(DoorAttrListener("Result"))
        
        QtCore.QObject.connect(self.doorResultListener(),
                               QtCore.SIGNAL("doorResultChanged"),
                               self.onDoorResultChanged)
        
        self.doorResultListener().setDoorName(self.doorName())
        
    def onDoorResultChanged(self, eventValue):
        """call on result attribute changed"""
        if eventValue is None: return        
        for resultLine in eventValue:
            self.append("RESULT  " + resultLine)
        self.moveCursor(QtGui.QTextCursor.End)
        
    def contextMenuEvent(self,event):
        menu = self.createStandardContextMenu() 
        clearAction = QtGui.QAction("Clear", menu)
        menu.addAction(clearAction)
        if not len(self.toPlainText()):
            clearAction.setEnabled(False) 
        
        QtCore.QObject.connect(clearAction, QtCore.SIGNAL("triggered()"), self.clear)               
        menu.exec_(event.globalPos())
        
#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
# Door attributes listeners
#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

class DoorAttrListener(QtCore.QObject):
        
    def __init__(self, attrName):
        QtCore.QObject.__init__(self)
        self.attrName = attrName
        self.attrObj = None
        
    def setDoorName(self, doorName):
        if not self.attrObj is None:
            self.attrObj.removeListener(self)
        self.attrObj = tau.Attribute(doorName, self.attrName)
        self.attrObj.addListener(self)

    def eventReceived(self, src, type, value):
        if (type == tau.core.TauEventType.Error or
            type == tau.core.TauEventType.Config):
            return
        self.emit(QtCore.SIGNAL('door%sChanged' % self.attrName), value.value)
