
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
macrodescriptionviewer.py: 
"""

from PyQt4 import QtGui,QtCore
from PyQt4.Qt import *

from tau.widget import TauBaseWidget
import tau.core

class TauMacroDescriptionViewer(QtGui.QTextEdit, TauBaseWidget):
    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent = None, designMode = False):
        name = "TauMacroDescriptionView"
        self.call__init__wo_kw(QtGui.QTextEdit, parent)
        self.call__init__(TauBaseWidget, name)
        self.setReadOnly(True)
        self.setFont(QtGui.QFont("Courier",9))
        self._macroName = None
        
    def defineStyle(self):
        """ Defines the initial style for the widget """
        self.updateStyle()

#    def sizeHint(self):
#        return self.minimumSizeHint()
#
#    def minimumSizeHint(self):
#        return QtCore.QSize(250,200)

    def getModelClass(self):
        return tau.core.TauDevice

    def updateStyle(self):
        self.update()
        
    def onMacroChanged(self, macroName):
        """Can be connected to an event emitted after macro name was changed.
           As an argument receives macroName and ask BaseMacroServer object 
           about already prepared and stored in MacroInfoObj object macro description"""
        macroServer = self.getModelObj()
        
        if macroServer is None or macroName == "":
            self.setText("")
            return
        
        self.setText(macroServer.getMacroInfoObj(macroName).doc)

    def getFormatedToolTip(self,cache=True):
        """This method was overridden to get rid of the default tooltip of TauWidget"""
        return ""
        
        
    model = QtCore.pyqtProperty("QString", 
                                TauBaseWidget.getModel, 
                                TauBaseWidget.setModel, 
                                TauBaseWidget.resetModel)
    
    useParentModel = QtCore.pyqtProperty("bool", 
                                         TauBaseWidget.getUseParentModel, 
                                         TauBaseWidget.setUseParentModel,
                                         TauBaseWidget.resetUseParentModel)
    
def test():
    import sys    
    app = QtGui.QApplication(sys.argv)
    tauMacroDescriptionView = TauMacroDescriptionViewer(designMode=1)
    if len(sys.argv) != 2:
        tauMacroDescriptionView.setModel("macroserver/bl98/1")
    else:
        tauMacroDescriptionView.setModel(sys.argv[1])
    tauMacroDescriptionView.onMacroChanged("mv")
    tauMacroDescriptionView.show()
    sys.exit(app.exec_())
    
       
if __name__ == "__main__":
    test()


