from PyQt4 import Qsci, QtGui, Qt, QtCore
from tau.widget import TauMainWindow
from common import createAction

class MacroEditor(TauMainWindow):
    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent = None, designMode = False):
        TauMainWindow.__init__(self, parent, designMode)
        self.initComponents()
        
    def initComponents(self):
        self.textEdit = Qsci.QsciScintilla()
        self.textEdit.setAutoCompletionThreshold(1)
        self.textEdit.setAutoCompletionSource(Qsci.QsciScintilla.AcsAll)
        self.textEdit.setAutoIndent(True)
        self.textEdit.setCallTipsStyle(Qsci.QsciScintilla.CallTipsContext)
        self.textEdit.setCallTipsVisible(0)
        self.pythonLexer = Qsci.QsciLexerPython(self.textEdit)
        self.api = Qsci.QsciAPIs(self.pythonLexer)
        self.api.add(QtCore.QString("dupa(a,b)this is function dupa"))
        self.api.prepare()
        self.pythonLexer.setAPIs(self.api)
        self.textEdit.setLexer(self.pythonLexer)
        self.setCentralWidget(self.textEdit)
        self.createActions()
#        self.createMenus()
        self.createToolBars();
#        self.createStatusBar();
        self.setCurrentFile("")
        
    def closeEvent(self, event):
        if self.maybeSave():
#            self.writeSettings()
            event.accept()
        else: 
            event.ignore()
            
    def newFile(self):
        if self.maybeSave():
            self.textEdit.clear()
            self.setCurrentFile("")
                
    def open(self):
        if self.maybeSave():
            fileName = QtGui.QFileDialog.getOpenFileName(self)
        if not fileName is None and file != "":
            self.loadFile(fileName)
            
    def save(self):
        if self.curFile == "":
            return self.saveAs()
        else:
            return self.saveFile(curFile)
        
    def saveAs(self):
        self.fileName = QtGui.QFileDialog.getSaveFileName(self)
        if self.fileName == "":
            return False
        return saveFile(self.fileName)

    def about(self):
        QtGui.QMessageBox.about(self, "About MacroEditor", "The MacroEditor by Zbigniew Reszela")

    def documentWasModified(self):
        self.setWindowModified(self.textEdit.isModified())

        
    def createActions(self):
        self.newAct = createAction(self,
                                   "New", 
                                   self.newFile, 
                                   "Ctrl+N",
                                   ":/actions/document-new.svg", 
                                   "Create a new file")
        self.openAct = createAction(self, 
                                    "Open", 
                                    self.open, 
                                    "Ctrl+O",
                                    ":/actions/document-open.svg", 
                                    "Open an existing file")
        self.saveAct = createAction(self, 
                                    "Save", 
                                    self.save, 
                                    "Ctrl+S", 
                                    ":/actions/document-save.svg",
                                    tip="Save the document to disk")
        self.saveAsAct = createAction(self, 
                                      "Save As...", 
                                      self.saveAs,
                                      icon=":/actions/document-save-as.svg", 
                                      tip="Save the document under a new name")
        self.exitAct = createAction(self, 
                                    "Exit", 
                                    self.close, 
                                    "Ctrl+Q", 
                                    tip="Exit the application")
        self.cutAct = createAction(self, 
                                   "Cut", 
                                   self.cut, 
                                   "Ctrl+X",
                                   ":/actions/edit-cut.svg", 
                                   tip="Cut the current selection's contents to the clipboard")
        self.copyAct = createAction(self, 
                                    "Copy", 
                                    self.copy, 
                                    "Ctrl+C", 
                                    ":/actions/edit-copy.svg",
                                    tip="Copy the current selection's contents to the clipboard")
        self.pasteAct = createAction(self, 
                                     "Paste", 
                                     self.paste, 
                                     "Ctrl+V", 
                                     ":/actions/edit-paste.svg",
                                     tip="Paste the clipboard's contents into the current selection")
        self.aboutAct = createAction(self, 
                                     "About", 
                                     self.about, 
                                     tip="Show the application's About box")
        self.cutAct.setEnabled(False)
        self.copyAct.setEnabled(False)
        self.connect(self.textEdit, QtCore.SIGNAL("copyAvailable(bool)"), self.cutAct.setEnabled)
        self.connect(self.textEdit, QtCore.SIGNAL("copyAvailable(bool)"), self.copyAct.setEnabled)
        
    def createMenus(self):
        fileMenu = self.menuBar().addMenu("File")
        fileMenu.addAction(self.newAct)
        fileMenu.addAction(self.openAct)
        fileMenu.addAction(self.saveAct)
        fileMenu.addAction(self.saveAsAct)
        
        editMenu = self.menuBar().addMenu("Edit")
        editMenu.addAction(self.cutAct)
        editMenu.addAction(self.copyAct)
        editMenu.addAction(self.pasteAct)

        self.menuBar().addSeparator()

        helpMenu = self.menuBar().addMenu("&Help")
        helpMenu.addAction(self.aboutAct)
        
    def createToolBars(self):
        fileToolBar = self.addToolBar("File")
        fileToolBar.setIconSize(Qt.QSize(36,36))
        fileToolBar.addAction(self.newAct)
        fileToolBar.addAction(self.openAct)
        fileToolBar.addAction(self.saveAct)
        fileToolBar.addAction(self.saveAsAct)
    
        editToolBar = self.addToolBar("Edit")
        editToolBar.setIconSize(Qt.QSize(36,36))
        editToolBar.addAction(self.cutAct)
        editToolBar.addAction(self.copyAct)
        editToolBar.addAction(self.pasteAct)

    def createStatusBar(self):
        self.statusBar().showMessage("Ready")
            
    def save(self):
        pass
            
    def maybeSave(self):
        if self.textEdit.isModified():
            ret = QtGui.QMessageBox.warning(self, "MacroEditor",
                     "The document has been modified\nDo you want to save your changes?",
                     QtGui.QMessageBox.Yes | QtGui.QMessageBox.Default,
                     QtGui.QMessageBox.No,
                     QtGui.QMessageBox.Cancel | QtGui.QMessageBox.Escape)
            if ret == QtGui.QMessageBox.Yes:
                return self.save()
            elif ret == QtGui.QMessageBox.Cancel:
                return False
        return True
        
    def loadFile(self, fileName):
        try:
            fileHandle = open(fileName, 'r')
        except IOError, e:
            QtGui.QMessageBox.warning(self, "MacroEditor", "Cannot read file %s:\n%s."%(fileName, e))
            return False
        fileContents = fileHandle.read()    
        QtGui.QApplication.setOverrideCursor(QtCore .Qt.WaitCursor)
        self.textEdit.setText(fileContents)
        QtGui.QApplication.restoreOverrideCursor()
    
        self.setCurrentFile(fileName)
        self.statusBar().showMessage("File loaded", 2000)

    def saveFile(self, fileName):
        try:
            file = open(fileName, 'w')
        except IOError, e:
            QtGui.QMessageBox.warning(self, "MacroEditor", "Cannot write file %s:\n%s."%(fileName, e))               
            return False
        QtGui.QApplication.setOverrideCursor(Qt.WaitCursor)
        file.write(self.textEdit.text())
        QtGui.QApplication.restoreOverrideCursor()
        self.setCurrentFile(fileName)
        self.statusBar().showMessage(("File saved"), 2000)
        return True
    
    def setCurrentFile(self, fileName):
        self.curFile = fileName
        self.textEdit.setModified(False)
        self.setWindowModified(False)

        if self.curFile == "":
            shownName = "untitled.txt"
        else:
            shownName = self.strippedName(self.curFile)
        self.setWindowTitle("%s[*] - %s"%(shownName,"MacroExecutor"))

    def strippedName(self, fullFileName):
        return QtCore.QFileInfo(fullFileName).fileName()
    
    def newFile(self):
        pass
    
    def cut(self):
        pass
    
    def copy(self):
        pass
    
    def paste(self):
        pass
    

def test():
    import sys    
    app = QtGui.QApplication(sys.argv)
    macroEditor = MacroEditor()
    macroEditor.show()
    sys.exit(app.exec_())
    
       
if __name__ == "__main__":
    test()