#from tau.lib.tau.widget import macroexecutor
#from tau.lib.tau.widget.macroexecutor.macroexecutor import addActions,\
#    createAction

#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
sequenceeditor.py: 
"""

from PyQt4 import Qt
import os, traceback, sys
from tau.widget import TauMainWindow, TauWidget, TauStateLed
from tau.widget.macroexecutor.common import addActions, createAction, MacroCommon
from tau.widget.macroexecutor.dooroutput import DoorAttrListener
from tau import Device
import PyTango
import xml.dom.minidom

#from tau.core.tango.macroserver.parameter import ParamRepeat
from tau.widget.macroexecutor.macroparameterseditor import MacroParametersEditor
from tau.widget.macroexecutor.macroparameterseditor.model import MacroNode
from tau.widget.macroexecutor.sequenceeditor.model import MacroSequenceTreeModel
from tau.widget.macroexecutor.sequenceeditor.sequenceeditor import MacroComboBox
from tau.widget.macroexecutor.macroparameterseditor.delegate import ParamEditorDelegate

from tau.widget.macroexecutor import globals

import tau.widget.resources.qrc_tango_icons_status

class FavoritesMacrosEditor(TauWidget):
    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent=None, designMode = False):
        name = "FavoritesMacrosEditor"
        TauWidget.__init__(self, parent, designMode)
        self.initComponents()
        
    def initComponents(self):
        self.mainLayout = Qt.QHBoxLayout()
        self.setLayout(self.mainLayout)
        
        self.tree = FavouriteMacrosTree(self)
        self.tree.setFocusPolicy(Qt.Qt.NoFocus)
        self.mainLayout.addWidget(self.tree)
                
        self.buttonGroupLayout = Qt.QVBoxLayout()
        self.buttonGroupLayout.setContentsMargins(0,0,0,0)
        
        delIcon = Qt.QIcon(":/actions/list-remove.svg")
        self.delButton = Qt.QPushButton(delIcon,"",self)
        self.delButton.setFocusPolicy(Qt.Qt.NoFocus)
        self.buttonGroupLayout.addWidget(self.delButton)
        upIcon = Qt.QIcon(":/actions/go-up.svg")
        self.upButton = Qt.QPushButton(upIcon,"",self)
        self.upButton.setFocusPolicy(Qt.Qt.NoFocus)
        self.buttonGroupLayout.addWidget(self.upButton)
        downIcon = Qt.QIcon(":/actions/go-down.svg")
        self.downButton = Qt.QPushButton(downIcon,"",self)
        self.downButton.setFocusPolicy(Qt.Qt.NoFocus)
        self.buttonGroupLayout.addWidget(self.downButton)
        spacerItem = Qt.QSpacerItem(0,40,Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Expanding)
        self.buttonGroupLayout.addItem(spacerItem)
        self.disableButtons()
        
        self.mainLayout.addLayout(self.buttonGroupLayout)
        
        Qt.QObject.connect(self.delButton, Qt.SIGNAL("clicked()"), self.onDelete)
        Qt.QObject.connect(self.upButton, Qt.SIGNAL("clicked()"), self.onUp)
        Qt.QObject.connect(self.downButton, Qt.SIGNAL("clicked()"), self.onDown)
        self.connect(self.tree, Qt.SIGNAL("favoriteMacroChanged"), self.manageButtons)
        
    def getValues(self):
        return self.tree.getValues()
            
    def disableButtons(self):
        self.delButton.setEnabled(False)
        self.upButton.setEnabled(False)
        self.downButton.setEnabled(False)
        
    def manageButtons(self, currentMacroNode):
        self.disableButtons()
        if len(self.tree.selectedIndexes()) == 0:
            return
        self.setEnabledDel(True)
        if len(self.tree.selectedIndexes()) != 2:
            return
        self.setEnabledUp(currentMacroNode.isAllowedMoveUp())
        self.setEnabledDown(currentMacroNode.isAllowedMoveDown())    
        
    def addMacro(self, macroNode):
        self.tree.addMacro(macroNode)
#    
    def onDelete(self):
        self.tree.onDeleteMacro()
        self.disableButtons()
        
    def onUp(self):
        self.tree.onUpMacro()
        self.disableButtons()
        
    def onDown(self):
        self.tree.onDownMacro()
        self.disableButtons()
        
    def setEnabledAdd(self, trueFalse):
        self.addButton.setEnabled(trueFalse)
                    
    def setEnabledDel(self,trueFalse):
        self.delButton.setEnabled(trueFalse)
            
    def setEnabledUp(self, trueFalse):
        self.upButton.setEnabled(trueFalse)
        
    def setEnabledDown(self, trueFalse):
        self.downButton.setEnabled(trueFalse)
    
    def isMacroSelected(self):
        return len(self.tree.selectedIndexes()) == 2
        
    def toXml(self):
        return self.tree.toXml()

    def fromXml(self, doc):
        self.tree.fromXml(doc)
        self.fillAdditionalInfos()
        
    def fillAdditionalInfos(self):
        for macro in self.tree.root().allMacros():
            if macro.hasParams():
                self.fillMacroAdditionalInfos(macro)
#                
    def fillMacroAdditionalInfos(self, macro):        
        macroInfoObj = self.getModelObj().getMacroInfoObj(macro.name())
        macro.setParams(macroInfoObj.getParamList())
        if macroInfoObj._hasParamComplex():
            self.fillParamRepeatAdditionalInfos(macro)
                    
    def fillParamRepeatAdditionalInfos(self,macro):
        paramsPairList = zip(macro.params(), macro.paramsBranch().children())
        for params in paramsPairList:
            if isinstance(params[0].get('type'),list):
                params[1].setParams(params[0].get('type'))


class FavouriteMacrosTree(Qt.QTreeView):
    
    def __init__(self, parent=None):
        
        Qt.QTreeView.__init__(self, parent)
        
        self.setSelectionBehavior(Qt.QTreeView.SelectRows)
        self.setSelectionMode(Qt.QTreeView.ExtendedSelection)
        self.setRootIsDecorated(False)
        self.setItemsExpandable(False)
#        self.setDragEnabled(True)
#        self.setAcceptDrops(True)
        self.setTabKeyNavigation(True)
        self.setEditTriggers(Qt.QAbstractItemView.EditKeyPressed)
        self.setDropIndicatorShown(True)

        treeModel = MacroSequenceTreeModel(self)
        headers = ["Macro","Parameters"]
        treeModel.headers = headers        
        self.setModel(treeModel)
        
        delegate = ParamEditorDelegate(self)
        self.setItemDelegate(delegate)
            
    def selectionChanged(self, selected, deselected):
        selectedIndexes = self.selectedIndexes()
        if len(selectedIndexes) == 2:
            currentMacroNode = self.model().nodeFromIndex(selectedIndexes[0])
            self.emit(Qt.SIGNAL("favoriteMacroChanged"), currentMacroNode) 
        else:
            self.emit(Qt.SIGNAL("favoriteMacroChanged"), None)
        Qt.QAbstractItemView.selectionChanged(self, selected, deselected)
        
    def sizeHint(self):
        return self.minimumSizeHint()

    def minimumSizeHint(self):
        return Qt.QSize(350,300)
        
    def expanded(self):
        for column in range(self.model().columnCount(Qt.QModelIndex())):
            self.resizeColumnToContents(column)
    
    def clearTree(self):
        self.model().clearTree()
            
    def getValues(self):
        return self.model().getParams()
    
    def toXml(self):
        return self.model().toXml()
            
    def fromXml(self, doc):
        self.model().fromXml(doc)
        self.expandAll()
    
    def root(self):
        return self.model().root()
    
    def setRoot(self, root):
        self.model().setRoot(root)
        self.model().reset()
    
    def resetRoot(self):
        self.model().resetRoot()
        
    def addMacro(self, macroNode):
        self.model().addMacro(self.root(), macroNode)
        self.expandAll()
            
    def onDeleteMacro(self):
        selectedIndexes = self.selectedIndexes()
        for i, idx in enumerate(selectedIndexes):
            if not i % 2: continue
            node = self.model().nodeFromIndex(idx)
            if not isinstance(node,MacroNode): continue
            self.model().deleteMacro(idx)
        self.emit(Qt.SIGNAL("macroChanged"),None)
    
    def onUpMacro(self):
        selectedIndexes = self.selectedIndexes()
        idx = selectedIndexes[0]
        node = self.model().nodeFromIndex(idx)
        if not isinstance(node,MacroNode):
            return
        self.model().upMacro(idx)
        self.expandAll()
        
    def onDownMacro(self):
        selectedIndexes = self.selectedIndexes()
        idx = selectedIndexes[0]
        node = self.model().nodeFromIndex(idx)
        if not isinstance(node,MacroNode):
            return
        self.model().downMacro(idx)
        self.expandAll()
    
    def dropEvent(self, event):
        Qt.QTreeView.dropEvent(self, event)
        self.expandAll()
        

class TauMacroExecutor(MacroCommon, TauMainWindow):
    
    def __init__(self, parent = None, designMode = False):
        TauMainWindow.__init__(self, parent, designMode)
        
        self._doorName = ""
        self._doorStateListener = DoorAttrListener("State")
        
        self.initComponents()
        
    def initComponents(self):
        self.macroParametersEditor = MacroParametersEditor(self)        
        self.favoritesMacrosEditor = FavoritesMacrosEditor(self)
        self.favoritesMacrosEditor.setUseParentModel(True)
        self.favoritesMacrosEditor.setFocusPolicy(Qt.Qt.NoFocus)
        
        splitter = Qt.QSplitter(self)
        splitter.setOrientation(Qt.Qt.Vertical)
        splitter.addWidget(self.macroParametersEditor)
        splitter.addWidget(self.favoritesMacrosEditor) 
        self.setCentralWidget(splitter)
        
        self.createActions()
        self.createToolBar()
        
        self.connect(self, Qt.SIGNAL("macroChanged"), self.macroParametersEditor.setMacroNode)
        self.connect(self.favoritesMacrosEditor.tree, Qt.SIGNAL("macroChanged"), self.macroParametersEditor.setMacroNode)
        self.connect(self.favoritesMacrosEditor.tree, Qt.SIGNAL("favoriteMacroChanged"), self.onFavoriteMacroChanged)
        
    def createActions(self):
        self.addToFavouritsAction = createAction(self,
                                                globals.MENU_FAVORITES, 
                                                self.onAddToFavorites,
                                                None,
                                                ":/status/software-update-available.svg",
                                                globals.TIP_MENU_FAVORITES)
        self.stopMacroAction = createAction(self,
                                                globals.MENU_CONTROL_STOP, 
                                                self.onStopMacro,
                                                None,
                                                ":/actions/media-playback-stop.svg",
                                                globals.TIP_MENU_CONTROL_STOP)
        self.pauseMacroAction = createAction(self,
                                                globals.MENU_CONTROL_PAUSE, 
                                                self.onPauseMacro,
                                                None,
                                                ":/actions/media-playback-pause.svg",
                                                globals.TIP_MENU_CONTROL_PAUSE)
        self.playMacroAction = createAction(self,
                                                globals.MENU_CONTROL_PLAY_MACRO, 
                                                self.onPlayMacro,
                                                None,
                                                ":/actions/media-playback-start.svg",
                                                globals.TIP_MENU_CONTROL_PLAY_MACRO)
    
    def createToolBar(self):
        self.macroToolBar = self.addToolBar("Control")
        self.macroToolBar.setObjectName("MacroToolBar")
        self.macroToolBar.addAction(self.addToFavouritsAction)
        self.addToFavouritsAction.setEnabled(False)
        self.macroComboBox = MacroComboBox(self)
        self.macroComboBox.setUseParentModel(True)
        self.macroComboBox.setModel("/MacroList")
        self.macroComboBox.setModelColumn(0)        
        self.connect(self.macroComboBox,Qt.SIGNAL("currentIndexChanged(QString)"), self.onMacroComboBoxChanged)
        self.macroToolBar.addWidget(self.macroComboBox)
        self.macroToolBar.setIconSize(Qt.QSize(36,36))
                
        self.controlToolBar = self.addToolBar("Control")
        self.controlToolBar.setObjectName("ControlToolBar")
        self.controlToolBar.setIconSize(Qt.QSize(36,36))
        addActions(self.controlToolBar, (None,
                                              self.stopMacroAction,
                                              self.pauseMacroAction,
                                              self.playMacroAction))
        self.disableControlActions()
        self.doorStateLed = TauStateLed(self)
        self.controlToolBar.addWidget(self.doorStateLed)
        
    def createMenuBar(self):
        self.quitAction = createAction(self,
                                        globals.MENU_FILE_QUIT,
                                        self.close,
                                        globals.SHORTCUT_MENU_FILE_QUIT, 
                                        ":/actions/process-stop.svg", 
                                        globals.TIP_MENU_FILE_QUIT)
        
        self.changeConfigurationAction = createAction(self,
                                                    globals.MENU_FILE_CONFIGURATION,
                                                    self.changeConfiguration,
                                                    globals.SHORTCUT_MENU_FILE_CONFIGURATION, 
                                                    ":/apps/preferences-system-session.svg", 
                                                    globals.TIP_MENU_FILE_CONFIGURATION)
        
        self.fileMenu = self.menuBar().addMenu(globals.MENU_FILE)
        addActions(self.fileMenu, (self.changeConfigurationAction, None, self.quitAction))
        
        self.controlMenu = self.menuBar().addMenu(globals.MENU_CONTROL)
        addActions(self.controlMenu, (self.playMacroAction, self.pauseMacroAction, self.stopMacroAction))
        
    def closeEvent(self, event):
        """Saves favorites before closing the application"""
        TauMainWindow.closeEvent(self, event)
        self.saveFavorites()
        
    def loadSettings(self):
        settings = self.getQSettings()
        self.onDoorChanged(str(settings.value("DoorName", Qt.QVariant(Qt.QString(""))).toString()))
        TauMainWindow.loadSettings(self)
        
    def saveSettings(self):
        settings = self.getQSettings()
        settings.setValue("DoorName", Qt.QVariant(Qt.QVariant(self.doorName())))
        TauMainWindow.saveSettings(self)
        
    def saveFavorites(self):
        settings = self.getQSettings()
        dirName = os.path.join(str(Qt.QDir.homePath()), str(settings.organizationName()), str(settings.applicationName()))
        if not os.path.isdir(dirName):
            os.makedirs(dirName)
        fileName = os.path.join(dirName, globals.FAVORITES_FILENAME)
        file = open(fileName, "w")
        try:
            xml = self.favoritesMacrosEditor.toXml().toprettyxml()
            file.write(xml)
        except Exception, e:
            print e
            traceback.print_exc(file=sys.stdout)
            Qt.QMessageBox.warning(self,"Error while saving favorites macros", "There was a problem while writing to the file: %s" % fileName)
        finally:
            file.close()
        
    def loadFavorites(self):
        settings = self.getQSettings()
        fileName = os.path.join(str(Qt.QDir.homePath()), str(settings.organizationName()), str(settings.applicationName()), globals.FAVORITES_FILENAME)
        if not os.path.isfile(fileName):
            Qt.QMessageBox.warning(self,"Error while loading favorites macros", "File with favorites macros does not exist, or has not been created yet.")
            return
        try:
            file = open(fileName,'r')
            self.favoritesMacrosEditor.fromXml(xml.dom.minidom.parse(file))
        except IOError, e:
            print e
            traceback.print_exc(file=sys.stdout)
            Qt.QMessageBox.warning(self,"Error while loading favorites macros", "File %1" % fileName)
            self.favoritesMacrosEditor.tree.resetRoot()
        except Exception, e:
            print e
            traceback.print_exc(file=sys.stdout)
            Qt.QMessageBox.warning(self,"Error while loading favorites macros", "There was a problem while loading favorites macros from file %s" % fileName)
            self.favoritesMacrosEditor.tree.resetRoot()
        finally:
            file.close()
        
    def doorStateListener(self):
        return self._doorStateListener
    
    def setDoorStateListener(self, doorStateListener):
        self._doorStateListener = doorStateListener
            
    def clearMacroComboBox(self):
        self.macroComboBox.setCurrentIndex(0)
        
    def onFavoriteMacroChanged(self, macroNode):
        if macroNode is None:
            self.playMacroAction.setEnabled(False)
            self.emit(Qt.SIGNAL("macroNameChanged"), "")
            self.emit(Qt.SIGNAL("macroChanged"), None)
        else: 
            self.playMacroAction.setEnabled(True)
            self.emit(Qt.SIGNAL("macroNameChanged"), macroNode.name())
            self.emit(Qt.SIGNAL("macroChanged"), macroNode)
        self.macroComboBox.setCurrentIndex(0)
        self.addToFavouritsAction.setEnabled(False)
        self.macroParametersEditor.setMacroNode(macroNode)
        
        
    def onMacroComboBoxChanged(self, macroName):
        macroName = str(macroName)
        if macroName == "" or macroName is None:
            self.addToFavouritsAction.setEnabled(False)
            if len(self.favoritesMacrosEditor.tree.selectedIndexes()) != 2:
                self.emit(Qt.SIGNAL("macroNameChanged"), macroName)
            self.emit(Qt.SIGNAL('macroChanged'),None)
            return
        self.emit(Qt.SIGNAL("macroNameChanged"), macroName)
        self.favoritesMacrosEditor.tree.clearSelection()
        macroInfoObj = self.getModelObj().getMacroInfoObj(str(macroName))
        self.macroNode = MacroNode(macroInfoObj=macroInfoObj)
        self.emit(Qt.SIGNAL('macroChanged'),self.macroNode)
        self.playMacroAction.setEnabled(True)
        self.addToFavouritsAction.setEnabled(True)
            
    def onAddToFavorites(self):
        self.favoritesMacrosEditor.addMacro(self.macroParametersEditor.macroNode())
        
    def onDoorChanged(self, doorName):
        self.setDoorName(doorName)
        if self.doorName() == "":
            self.doorStateLed.setModel(None)
            return
        self.doorStateLed.setModel(self.doorName() + "/State")
        
        if not self.doorStateListener() is None:
            Qt.QObject.disconnect(self.doorStateListener(),
                                      Qt.SIGNAL("doorStateChanged"),
                                      self.onDoorStateChanged)
        
        self.setDoorStateListener(DoorAttrListener("State"))
        
        Qt.QObject.connect(self.doorStateListener(),
                               Qt.SIGNAL("doorStateChanged"),
                               self.onDoorStateChanged)
       
        self.doorStateListener().setDoorName(self.doorName())
                
    def onDoorStateChanged(self, eventValue):
        self.disableControlActions()
        if eventValue == PyTango.DevState.RUNNING:
            self.macroComboBox.setEnabled(False)
            self.playMacroAction.setEnabled(False)
            self.pauseMacroAction.setEnabled(True)
            self.stopMacroAction.setEnabled(True)
        elif eventValue == PyTango.DevState.ON:
            self.macroComboBox.setEnabled(True)
            self.pauseMacroAction.setEnabled(False)
            self.stopMacroAction.setEnabled(False)
            if self.macroComboBox.currentText() != "":
                self.playMacroAction.setEnabled(True)
            
    def disableControlActions(self):
        self.pauseMacroAction.setEnabled(False)
        self.stopMacroAction.setEnabled(False)
        self.playMacroAction.setEnabled(False)
            
    def onPlayMacro(self):
        if self.doorName() == "" or self.doorName() is None:
            return
        door = Device(self.doorName())
        if door.getState() == PyTango.DevState.ON:
            macroNode = self.macroParametersEditor.macroNode()
            params,alerts = macroNode.toRun()
            if len(alerts) > 0:
                Qt.QMessageBox.warning(self,"Macro params warning", alerts)
                return
            self.emit(Qt.SIGNAL("macroStarted"), macroNode.name(), macroNode.allMotors())
            door.runMacro(str(macroNode.name()), params)
        elif door.getState() == PyTango.DevState.STANDBY:
            door.command_inout("ResumeMacro")
    
    def onStopMacro(self):
        door = Device(self.doorName())
        if door.getState() == PyTango.DevState.RUNNING or door.getState() == PyTango.DevState.STANDBY:
            door.command_inout("Abort")
    
    def onPauseMacro(self):
        door = Device(self.doorName())
        if door.getState() == PyTango.DevState.RUNNING:
            door.command_inout("PauseMacro")
        
if __name__ == "__main__": 
    import sys
    from tau.core.tango.macroserver import registerExtensions    
    registerExtensions()
    app = Qt.QApplication(sys.argv)
    app.setOrganizationName(globals.ORGANIZATION_NAME)
    app.setApplicationName(globals.MACROEXECUTOR_APPLICATION_NAME)
    app.setWindowIcon(Qt.QIcon(":/apps/preferences-system-session.svg"))
    macroExecutor = TauMacroExecutor()
    Qt.QObject.connect(macroExecutor, Qt.SIGNAL("doorChanged"), macroExecutor.onDoorChanged)
    macroExecutor.loadSettings()
    if len(sys.argv) == 3:
        macroExecutor.setModel(sys.argv[1])
        macroExecutor.emit(Qt.SIGNAL('doorChanged'),sys.argv[2])
    macroExecutor.loadFavorites()
    macroExecutor.createMenuBar()
    macroExecutor.show()
    sys.exit(app.exec_())