#from tau.lib.tau.widget.macroexecutor import macroGUI

#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
macroexecutor.py: test
"""
from PyQt4 import Qt
 
import tau.core
from tau.widget import TauMainWindow
from tau.widget.extra_xterm import XTermWidget

from macrodescriptionviewer import TauMacroDescriptionViewer
from dooroutput import DoorOutput, DoorDebug, DoorResult
from sequenceeditor import TauSequencer
from common import addActions, createAction, MacroCommon
from scanplotter import ScanPlotter
from macroexecutor import TauMacroExecutor
withMacroEditor = True
try:
    from macroeditor import MacroEditor
#    raise ImportError
except ImportError:
    print "Could not import QScintilla, MacroEditor can not be created."
    withMacroEditor = False

import tau.widget.resources.qrc_tango_icons_actions
import tau.widget.resources.qrc_tango_icons_apps
import tau.widget.resources.qrc_extra_icons
import globals 
__version__ = "0.0.1"
                
class TauMacroGui(MacroCommon, TauMainWindow):
    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent = None, designMode = False):
        TauMainWindow.__init__(self, parent, designMode)        
        MacroCommon.__init__(self)        
        self.initComponents()
        self.defineStyle()
                
    def initComponents(self):
        self.setWindowTitle(globals.MACROGUI_APPLICATION_NAME)
        
        self.mdiArea = Qt.QMdiArea(self)
        self.mdiArea.setViewMode(Qt.QMdiArea.TabbedView)
        self.mdiArea.setOption(Qt.QMdiArea.DontMaximizeSubWindowOnActivation, False)
        self.setCentralWidget(self.mdiArea)
        
        self.tauMacroExecutorPlot = ScanPlotter()
        self.tauMacroExecutorPlotSubWindow = Qt.QMdiSubWindow()
        self.tauMacroExecutorPlotSubWindow.setWidget(self.tauMacroExecutorPlot)
        self.tauMacroExecutorPlotSubWindow.setWindowTitle("Macro executor plot")
        self.mdiArea.addSubWindow(self.tauMacroExecutorPlotSubWindow)
        self.tauMacroExecutorPlotSubWindow.showMaximized()
        
        self.tauSequencerPlot = ScanPlotter()
        self.tauSequencerPlotSubWindow = Qt.QMdiSubWindow()
        self.tauSequencerPlotSubWindow.setWidget(self.tauSequencerPlot)
        self.tauSequencerPlotSubWindow.setWindowTitle("Sequence editor plot")
        
        if withMacroEditor:
            self.macroEditor = MacroEditor()
            self.macroEditorSubWindow = Qt.QMdiSubWindow()
            self.macroEditorSubWindow.setWidget(self.macroEditor)
            self.macroEditorSubWindow.setWindowTitle("Macro editor")
        
        self.macroExecutor = TauMacroExecutor(self)
        self.macroExecutor.setUseParentModel(True)
        self.dockForTauMacroExecutor = Qt.QDockWidget(globals.TITLE_MACRO_EXECUTOR_DOCK_WIDGET, self)
        self.dockForTauMacroExecutor.setObjectName("dockWidgetForTauMacroExecutor")
        self.dockForTauMacroExecutor.setWidget(self.macroExecutor)
        self.addDockWidget(Qt.Qt.LeftDockWidgetArea,self.dockForTauMacroExecutor)
        
        self.sequencer = TauSequencer(self)
        self.sequencer.setUseParentModel(True)
        self.dockForTauSequencer = Qt.QDockWidget(globals.TITLE_SEQUENCE_EDITOR_DOCK_WIDGET, self)
        self.dockForTauSequencer.setObjectName("dockWidgetForTauSequencer")
        self.dockForTauSequencer.setWidget(self.sequencer)
        self.addDockWidget(Qt.Qt.LeftDockWidgetArea,self.dockForTauSequencer)
        self.dockForTauSequencer.setHidden(True)
                
        self.tauMacroDescriptionViewer = TauMacroDescriptionViewer(self)
        self.tauMacroDescriptionViewer.setUseParentModel(True)
        self.dockForTauMacroDescriptionViewer = Qt.QDockWidget(globals.TITLE_DESCRIPTION_DOCK_WIDGET, self)
        self.dockForTauMacroDescriptionViewer.setObjectName("dockWidgetForTauMacroDescriptionViewer")
        self.dockForTauMacroDescriptionViewer.setWidget(self.tauMacroDescriptionViewer)
        self.addDockWidget(Qt.Qt.BottomDockWidgetArea,self.dockForTauMacroDescriptionViewer)
        
        self.doorOutput = DoorOutput(self)
        self.dockForDoorOutput = Qt.QDockWidget(globals.TITLE_OUTPUT_DOCK_WIDGET, self)
        self.dockForDoorOutput.setWidget(self.doorOutput)
        self.dockForDoorOutput.setObjectName("dockWidgetForDoorOutput")
        self.addDockWidget(Qt.Qt.BottomDockWidgetArea,self.dockForDoorOutput)
        
        
        self.spock = XTermWidget()
        self.spock.setCommand("spock")
        self.dockForSpock = Qt.QDockWidget(globals.TITLE_SPOCK_DOCK_WIDGET, self)
        self.dockForSpock.setWidget(self.spock)
        self.dockForSpock.setObjectName("dockWidgetForSpock")
        self.addDockWidget(Qt.Qt.BottomDockWidgetArea,self.dockForSpock)
        
        self.doorDebug = DoorDebug(self)
        self.dockForDoorDebug = Qt.QDockWidget(globals.TITLE_DEBUG_DOCK_WIDGET, self)
        self.dockForDoorDebug.setWidget(self.doorDebug)
        self.dockForDoorDebug.setObjectName("dockWidgetForDoorDebug")
        self.dockForDoorDebug.setHidden(True)
        
        self.doorResult = DoorResult(self)
        self.dockForDoorResult = Qt.QDockWidget(globals.TITLE_RESULT_DOCK_WIDGET, self)
        self.dockForDoorResult.setWidget(self.doorResult)
        self.dockForDoorResult.setObjectName("dockWidgetForDoorResult")
        self.dockForDoorResult.setHidden(True)
        
        self.tabifyDockWidget(self.dockForSpock, self.dockForTauMacroDescriptionViewer)
        self.tabifyDockWidget(self.dockForTauMacroDescriptionViewer, self.dockForDoorOutput)
        self.tabifyDockWidget(self.dockForDoorOutput, self.dockForDoorDebug)
        self.tabifyDockWidget(self.dockForDoorDebug, self.dockForDoorResult)
        self.tabifyDockWidget(self.dockForTauMacroExecutor, self.dockForTauSequencer)
        
        self.createActions()
        self.createMenuBar()
        self.createToolBar()
                
        self.connect(self,Qt.SIGNAL("doorChanged"), self.doorOutput.onDoorChanged)
        self.connect(self,Qt.SIGNAL("doorChanged"), self.doorDebug.onDoorChanged)
        self.connect(self,Qt.SIGNAL("doorChanged"), self.doorResult.onDoorChanged)
        self.connect(self,Qt.SIGNAL("doorChanged"), self.onDoorChanged)
        self.connect(self,Qt.SIGNAL("doorChanged"), self.macroExecutor.onDoorChanged)
        self.connect(self,Qt.SIGNAL("doorChanged"), self.sequencer.onDoorChanged)
        self.connect(self,Qt.SIGNAL("doorChanged"), self.tauMacroExecutorPlot.onDoorChanged)
        self.connect(self,Qt.SIGNAL("doorChanged"), self.tauSequencerPlot.onDoorChanged)
        
        #sequenceEditor plotables signals
        self.connect(self.sequencer.sequenceEditor, Qt.SIGNAL("macrosAdded"), self.tauSequencerPlot.onMacrosAdded)
        # the following connection is just for emitting a signal
        # also in case when user copy macro in a sequence 
        # by using drag and drop 
        self.connect(self.sequencer.sequenceEditor.tree.model(), Qt.SIGNAL("macrosAdded"), self.tauSequencerPlot.onMacrosAdded)
        self.connect(self.sequencer.sequenceEditor, Qt.SIGNAL("sequenceCleared"), self.tauSequencerPlot.onSequenceCleared)
        self.connect(self.sequencer.sequenceEditor.tree, Qt.SIGNAL("macrosDeleted"), self.tauSequencerPlot.onMacrosDeleted)
        self.connect(self.sequencer.macroParametersEditor.tree.model(), Qt.SIGNAL("motorChanged"), self.tauSequencerPlot.onMotorChanged)
        
        self.connect(self.macroExecutor, Qt.SIGNAL("macroStarted"), self.setFocusToDoorOutput)
        self.connect(self.macroExecutor, Qt.SIGNAL("macroChanged"), self.tauMacroExecutorPlot.onMacroChanged)
        self.connect(self.macroExecutor.favoritesMacrosEditor.tree, Qt.SIGNAL("macroChanged"), self.tauMacroExecutorPlot.onMacroChanged)
        self.connect(self.sequencer, Qt.SIGNAL("sequenceStarted"), self.setFocusToDoorOutput)
        self.connect(self.sequencer.sequenceEditor,Qt.SIGNAL("macroNameChanged"), self.tauMacroDescriptionViewer.onMacroChanged)
        self.connect(self.sequencer,Qt.SIGNAL("macroNameChanged"), self.tauMacroDescriptionViewer.onMacroChanged)
        self.connect(self.macroExecutor, Qt.SIGNAL("macroNameChanged"), self.tauMacroDescriptionViewer.onMacroChanged)
        
        #macroExecutor plotables signals
        self.connect(self.macroExecutor.macroParametersEditor.tree.model(), Qt.SIGNAL("motorChanged"), self.tauMacroExecutorPlot.onMotorChanged)
        self.connect(self.macroExecutor, Qt.SIGNAL("macroChanged"), self.tauMacroExecutorPlot.onMacroChanged)
        
        #dock widgets visibility changed actions
        self.connect(self.dockForTauMacroExecutor, Qt.SIGNAL("visibilityChanged(bool)"), self.onDockForTauMacroExecutorVisibilityChanged)
        self.connect(self.dockForTauSequencer, Qt.SIGNAL("visibilityChanged(bool)"), self.onDockForTauSequencerVisibilityChanged)
        
        self.connect(self.tauMacroExecutorPlotSubWindow, Qt.SIGNAL("aboutToActivate()"), self.onTauMacroExecutorPlotActivated)
        self.connect(self.tauSequencerPlotSubWindow, Qt.SIGNAL("aboutToActivate()"), self.onTauSequencerPlotActivated)
        
    def onMacroEditorViewAction(self):
        action = self.sender()
        if action.isChecked():
            self.mdiArea.addSubWindow(self.macroEditorSubWindow)
            self.macroEditorSubWindow.setVisible(True)
        else:
            self.mdiArea.removeSubWindow(self.macroEditorSubWindow)
            activeSubWindow = self.mdiArea.activeSubWindow() 
            if activeSubWindow:
                activeSubWindow
                                 
    def onDockForTauMacroExecutorVisibilityChanged(self, visible):
        actionChecked = self.tauMacroExecutorViewAction.isChecked() 
        if visible:
            self.mdiArea.addSubWindow(self.tauMacroExecutorPlotSubWindow)
            self.tauMacroExecutorPlotSubWindow.setVisible(True)
            self.mdiArea.setActiveSubWindow(self.tauMacroExecutorPlotSubWindow)
        else:
            if not actionChecked:
                self.mdiArea.removeSubWindow(self.tauMacroExecutorPlotSubWindow)
                activeSubWindow = self.mdiArea.activeSubWindow() 
                if activeSubWindow:
                    activeSubWindow
        
    def onDockForTauSequencerVisibilityChanged(self, visible):
        actionChecked = self.tauSequencerViewAction.isChecked()
        if visible:
            self.mdiArea.addSubWindow(self.tauSequencerPlotSubWindow)
            self.tauSequencerPlotSubWindow.setVisible(True)
            self.mdiArea.setActiveSubWindow(self.tauSequencerPlotSubWindow)
        else:
            if not actionChecked:
                self.mdiArea.removeSubWindow(self.tauSequencerPlotSubWindow)
                activeSubWindow = self.mdiArea.activeSubWindow() 
                if activeSubWindow:
                    activeSubWindow.showMaximized()

    def onTauSequencerPlotActivated(self):
        children = self.children()
        for child in children:
            if child.objectName() == "dockWidgetForTauSequencer":
                child.setHidden(False)
                child.raise_()
                break
    
    def onTauMacroExecutorPlotActivated(self):
        children = self.children()
        for child in children:
            if child.objectName() == "dockWidgetForTauMacroExecutor":
                child.setHidden(False)
                child.raise_()
                break

    def closeEvent(self, event):
        """Saves fovorites before closing the application"""
        TauMainWindow.closeEvent(self, event)
        self.macroExecutor.saveFavorites()
        
    def createActions(self):
        self.quitAction = createAction(self,
                                        globals.MENU_FILE_QUIT,
                                        self.close,
                                        globals.SHORTCUT_MENU_FILE_QUIT, 
                                        ":/actions/process-stop.svg", 
                                        globals.TIP_MENU_FILE_QUIT)
        self.changeConfigurationAction = createAction(self,
                                                    globals.MENU_FILE_CONFIGURATION,
                                                    self.changeConfiguration,
                                                    globals.SHORTCUT_MENU_FILE_CONFIGURATION, 
                                                    ":/apps/preferences-system-session.svg", 
                                                    globals.TIP_MENU_FILE_CONFIGURATION)
        self.tauMacroExecutorViewAction = self.dockForTauMacroExecutor.toggleViewAction()
        self.tauSequencerViewAction = self.dockForTauSequencer.toggleViewAction()
        self.spockViewAction = self.dockForSpock.toggleViewAction()
        self.tauMacroDescriptionViewerViewAction = self.dockForTauMacroDescriptionViewer.toggleViewAction()
        if withMacroEditor: 
            self.macroEditorViewAction = createAction(self,
                                            "Macro editor",
                                            self.onMacroEditorViewAction,  
                                            tip=globals.TIP_MENU_FILE_QUIT)
            self.macroEditorViewAction.setCheckable(True)
        
        
    def createMenuBar(self):
        self.fileMenu = self.menuBar().addMenu(globals.MENU_FILE)
        addActions(self.fileMenu, (self.changeConfigurationAction, None, self.quitAction))
        
        self.viewsMenu = self.menuBar().addMenu(globals.MENU_VIEW)
        if withMacroEditor:
            addActions(self.viewsMenu,(self.tauMacroExecutorViewAction,
                                        self.tauSequencerViewAction,
                                        self.macroEditorViewAction,
                                        self.spockViewAction,
                                        self.tauMacroDescriptionViewerViewAction,
                                        None,
                                        self.dockForDoorOutput.toggleViewAction(),
                                        self.dockForDoorDebug.toggleViewAction(),
                                        self.dockForDoorResult.toggleViewAction()))
        else:
            addActions(self.viewsMenu,(self.tauMacroExecutorViewAction,
                                        self.tauSequencerViewAction,
                                        self.spockViewAction,
                                        self.tauMacroDescriptionViewerViewAction,
                                        None,
                                        self.dockForDoorOutput.toggleViewAction(),
                                        self.dockForDoorDebug.toggleViewAction(),
                                        self.dockForDoorResult.toggleViewAction()))
#        self.onlyScansAction = createAction(self,
#                                            globals.MENU_SETTINGS_ONLYSCANS,
#                                            self.changeOnlyScans,
#                                            checkable=True,
#                                            signal="toggled(bool)")
#        
#        self.settingsMenu = self.menuBar().addMenu(globals.MENU_SETTINGS)
#        addActions(self.settingsMenu,(self.changeConfigurationAction,
#                                           self.onlyScansAction))
        
    def createToolBar(self):
        pass
#        contextToolBar = ContextToolBar("browser", "alba02", "browser")
#        contextToolBar.setIconSize(Qt.QSize(36,36))
#        self.addToolBar(contextToolBar)
#        self.addToolBar(self.basicTauToolbar())
#        self.fileToolBar = self.addToolBar("File")
#        self.fileToolBar.setObjectName("FileToolBar")
#        self.fileToolBar.setIconSize(Qt.QSize(36,36))
#        self.addActions(self.fileToolBar, (self.newSequenceAction,
#                                           self.openSequenceAction, 
#                                           self.saveSequenceAction))
#        
#        self.controlToolBar = self.addToolBar("Control")
#        self.controlToolBar.setObjectName("ControlToolBar")
#        self.controlToolBar.setIconSize(Qt.QSize(36,36))
#        self.addActions(self.controlToolBar, (None,
#                                              self.stopSequenceAction,
#                                              self.pauseSequenceAction,
#                                              self.playSequenceAction,
#                                              self.playMacroAction))
#        
#        self.doorToolBar = self.addToolBar("Door")
#        self.doorToolBar.setObjectName("DoorToolBar")
#        self.doorToolBar.setIconSize(Qt.QSize(36,36))
#                        
#        self.doorStateLed = TauStateLed(self)
##        self.doorStateLed.setLedSize(48)
#        
#        self.doorToolBar.addWidget(self.doorStateLed)
#        
#        self.doorComboBox = TauAttrListComboBox(self)
#        self.doorComboBox.setUseParentModel(True)
#        self.doorComboBox.setModel("/DoorList")
#        self.doorToolBar.addWidget(self.doorComboBox)
#        
#        Qt.QObject.connect(self.doorComboBox,Qt.SIGNAL("currentIndexChanged(QString)"),self.onDoorChanged)

    def onDoorChanged(self, doorName):
        self.setDoorName(doorName)
            
    def getModelClass(self):
        return tau.core.TauDevice
        
    def defineStyle(self):
        self.updateStyle()
        
    def updateStyle(self):
        self.update()

#    def sizeHint(self):
#        return self.minimumSizeHint()

#    def minimumSizeHint(self):
#        return Qt.QSize(1500,1500)
    
#    def updateFileMenu(self):
#        self.fileMenu.clear()
#        self.fileMenu.addAction(self.newSequenceAction)
#        self.fileMenu.addAction(self.openSequenceAction)
#        self.fileMenu.addAction(self.saveSequenceAction)
#        self.fileMenu.addAction(self.saveasSequenceAction)
#        self.fileMenu.addAction(self.changeConfigurationAction)
#        self.fileMenu.addSeparator()
#        self.recentMenu = self.fileMenu.addMenu("Recent macros")
#        index = 0
#        for macro in self.macroExecutor.getHistory():
#            displayText = macro.name()
#            params = macro.childrenValues()[0]
#            for param in params:
#                displayText += " " + param
#            action = createAction(displayText,
#                                       self.parent().loadMacro)
#            action.setData(Qt.QVariant(index))
#            self.recentMenu.addAction(action)
#            index = index + 1
#        self.fileMenu.addSeparator()
#        self.fileMenu.addAction(self.quitAction)
        
    def loadSettings(self):
        TauMainWindow.loadSettings(self)
        settings = self.getQSettings()
        doorName = str(settings.value("DoorName", Qt.QVariant(Qt.QString(""))).toString())
        if doorName != "": self.emit(Qt.SIGNAL("doorChanged"), doorName)
        self.sequencer.setSequencesPath(str(settings.value("SequencesPath", Qt.QVariant(Qt.QDir().homePath())).toString()))
        
    def saveSettings(self):
        
        settings = self.getQSettings()
        settings.setValue("SequencesPath", Qt.QVariant(Qt.QVariant(self.sequencer.sequencesPath())))
        settings.setValue("DoorName", Qt.QVariant(Qt.QString(self.doorName())))
        TauMainWindow.saveSettings(self)
#        self.sequencer.saveMacrosHistory()
            
    def setFocusToDoorOutput(self):
        children = self.children()
        for child in children:
            if child.objectName() == "dockWidgetForDoorOutput":
                child.setHidden(False)
                child.raise_()
                break
                
    def setMacrosFilter(self, filter):
        self.sequencer.setMacrosFilter(filter)  
           
    def changeOnlyScans(self, checked):
        if checked:
            self.setMacrosFilter(globals.SCAN_MACROS_FILTER)
        else:
            self.setMacrosFilter(None)
        
    def getFormatedToolTip(self,cache=True):
        """This method was overridden to get rid of the default tooltip of TauWidget"""
        return ""

if __name__ == "__main__": 
    import sys
    from tau.core.tango.macroserver import registerExtensions    
    registerExtensions()
    app = Qt.QApplication(sys.argv)
    app.setOrganizationName(globals.ORGANIZATION_NAME)
    app.setApplicationName(globals.MACROGUI_APPLICATION_NAME)
    app.setWindowIcon(Qt.QIcon(":/apps/preferences-system-session.svg"))
    macroGui = TauMacroGui()
    macroGui.loadSettings()
    if len(sys.argv) == 3:
        macroGui.setModel(sys.argv[1])
        macroGui.emit(Qt.SIGNAL("doorChanged"),sys.argv[2])
    macroGui.macroExecutor.loadFavorites()
    macroGui.show()
    sys.exit(app.exec_())

