
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
make-resources.py: 
"""




import os, sys, glob
ALLOWED_RESOURCES_EXTENSIONS = ['.svg', '.png']

def makeResources(expr):
    ''' gets a glob expression (path with optional wildcards describing resource
    items). It creates one or more resource files (one for each directory)
    and "compiles" them with pyrcc4'''
    print 
    head,tail = os.path.split(expr)
    includedDirs = glob.glob(head)
    includedDirs = [d for d in includedDirs if os.path.isdir(d)] #remove non-dirs
    modules=[]
    
    #create separate resources files for each dir
    for dir in includedDirs:
        
        items = glob.glob(os.path.join(dir,tail))
        items = [item for item in items if os.path.splitext(item)[-1].lower() in ALLOWED_RESOURCES_EXTENSIONS ]
        if len(items) == 0: continue #do not create empty resource files
        
        #create the .qrc and .html files
        qrcFileName = dir.replace(os.path.sep,'_').replace('-','_')+'.qrc'
        htmlFileName = dir.replace(os.path.sep,'_').replace('-','_')+'.html'
        outputFileName = "qrc_%s.py"%dir.replace(os.path.sep,'_').replace('-','_')
        
        h = file(htmlFileName, 'w')
        f = file(qrcFileName, 'w')
        h.write('<html><body>\n<h1>%s</h1>\n<table>\n'%outputFileName)
        h.write('<tr><th>Icon</th><th>Alias</th><th>Full name</th></tr>\n')
        f.write('<RCC>\n\t<qresource>\n')
        for n in items:
            h.write('<tr><td><iframe src="%s" width="80" height="80"></iframe></td><td>":/%s"</td><td>":/%s"</td></tr>\n'%(n,os.path.basename(n),n))
            f.write('\t\t<file alias="%s">%s</file>\n'%(os.path.basename(n),n))
            
        h.write('</table>\n</body></html>\n')
        f.write('\t</qresource>\n</RCC>')
        h.write('\n<!-- This .qrc file has been autogenerated by %s\n Do not modify it (all changes will be lost) -->\n'%sys.argv[0])
        f.write('\n<!-- This .qrc file has been autogenerated by %s\n Do not modify it (all changes will be lost) -->\n'%sys.argv[0])
        h.close()
        f.close()
        print "Created: ",qrcFileName
        print "Created: ",htmlFileName
        
        #"compile" the resources file
        cmd = 'pyrcc4 -o %s %s'%(outputFileName, qrcFileName)
        print 'Running: %s'%cmd
        result = os.system(cmd)
        if result == 0:
            modules.append(outputFileName)
    return modules

def updateTangoIcons():
    '''Offers to updates the tango-icons directory from cvs
    (or to check it out if necessary)'''
    result = None 
    if os.path.exists('tango-icons'):
        while result not in ('y','n'):
            result = raw_input('Update tango-icons from cvs? (y/n) ').lower()
        if result == 'y':
            os.chdir('tango-icons')
            cmd = 'cvs up'
            print 'Running: ', cmd, ' in "tango-icons"'
            result = os.system(cmd)
            os.chdir(os.path.pardir)
            if result != 0:
                print 'Cannot Update. Aborting'
                return result
    else:
        while result not in ('y','n'):
            result = raw_input('tango-icons does not exist. Fetch from cvs? (y/n) ').lower()
        if result.lower() == 'y':
            cmd = 'cvs -d:pserver:anoncvs@anoncvs.freedesktop.org:/cvs/tango login'
            print cmd
            print "(press enter if prompted for a password)"
            result = os.system(cmd)
            if result != 0:
                print 'Cannot connect. Aborting'
                return result
            cmd = 'cvs -d:pserver:anoncvs@anoncvs.freedesktop.org:/cvs/tango/ co -d tango-icons tango-icon-theme/scalable'
            print cmd
            result = os.system(cmd)
            if result != 0:
                print 'Error Downloading. Aborting'
                return result
    return 0

def updateInitFile(modules):
    f = file('__init__.py', 'w')
    f.write('## This file has been autogenerated by %s\n## Do not modify it (all changes will be lost)\n'%sys.argv[0])
    modules = ["'"+os.path.splitext(m)[0]+"'" for m in modules]
    f.write('__all__ = [%s]'%(', '.join(modules)))
    f.close()
    print "Created: __init__.py"
    
def createIndexHTML(modules):
    f = file('index.html', 'w')
    f.write('<html><h1>Available resource modules</h1>\n')
    for m in modules:
        htmlFileName = os.path.splitext(m)[0][4:]+".html"
        f.write('\n<p><a href="%s">%s</a></p>'%(htmlFileName,m))
    f.write('\n</html>')
    f.close()
    print "Created: index.html"
    
if __name__ == "__main__":
    
    curdir = os.path.abspath(os.curdir) #the path where we were called from
    resourcesdir = sys.path[0] #the path where the make-resources.py script resides
    modules=[]
    
    os.chdir(resourcesdir)
    
    updateTangoIcons() #update the cvs repo for tango-icons/
    modules += makeResources('tango-icons/*/*.svg') #make resources for the tango-icons dirs
    modules += makeResources('taudesigner-icons/*.*') #make resources for the icons dir
    modules += makeResources('extra-icons/*.*') #make resources for the icons dir
    updateInitFile(modules) #recreate the __init__.py file
    createIndexHTML(modules)
    
    os.chdir(curdir)