
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
jdraw_view.py: 
"""

import os
import subprocess
import jdraw
import jdraw_parser
from PyQt4 import Qt, QtGui, QtCore
import tau
from tau.widget import taubase

class TauJDrawSynopticsView(QtGui.QGraphicsView, taubase.TauBaseWidget):
    __pyqtSignals__ = ("itemsChanged","graphicItemSelected(const QString &)")

    def __init__(self, parent = None, designMode = False):
        name = self.__class__.__name__
        self.call__init__wo_kw(QtGui.QGraphicsView, parent)
        self.call__init__(taubase.TauBaseWidget, name, designMode=designMode)
        self._currF = self.modelName
        self.w_scene = None
        self.h_scene = None
        self._fileName ="Root"
        self.setInteractive(True)
        # By default the items will update the view when necessary.
        # This default value is much more efficient then the QQraphicsView default
        # value, so if you decide to change then expect a lot of processor to be
        # used by your application.
        self.setViewportUpdateMode(QtGui.QGraphicsView.NoViewportUpdate)

    def defineStyle(self):
        self.updateStyle()

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget over writing
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def isReadOnly(self):
        return True

    def update(self):
        #self.emit_signal()
        self.emitColors()

    def openJDraw(self):
        ifile = QtGui.QFileDialog.getOpenFileName( self, 'Load JDraw File', '', 'JDraw File (*.jdw)')
        if ifile.isEmpty(): return
        if not isinstance(ifile,file):
            #clear TauPLot - emit sygnal
            #self.emit(QtCore.SIGNAL("clearTauPlot"))
            fileName = ifile.split("/")
            #self.emit(QtCore.SIGNAL("setTopItemName"),fileName[-1])
            self._fileName = fileName[-1]
            self.setModel(ifile)
        return fileName[-1]

    def get_item_list(self):
        #print "emit signal which is used to refresh the tree and colors of icons depend of the current status in jdrawSynoptic"
        return [item._name for item in self.scene().items()]
    
    def get_item_colors(self,emit = False):
        item_colors = dict((item._name,item._currBgBrush.color().name()) for item in self.scene().items() if item and hasattr(item,'_name') and item._currBgBrush)
        if emit: self.emit(QtCore.SIGNAL("itemsChanged"),self.modelName.split('/')[-1].split('.')[0],item_colors)
        return item_colors
        
    @QtCore.pyqtSignature("selectGraphicItem(QString)")
    def selectGraphicItem(self,item_name):
        self.scene().selectGraphicItem(item_name)
        return False
    
    @QtCore.pyqtSignature("graphicItemSelected(const QString &)")
    def graphicItemSelected(self,item_name):
        self.emit(QtCore.SIGNAL("graphicItemSelected(const QString &)"),item_name)
        
    
    @QtCore.pyqtSignature("emitColors")
    def emitColors(self): self.get_item_colors(True)

    def resizeEvent(self, event):
        if isinstance(self.parent(),QtGui.QScrollArea):
            return
        try:
            if self.parent(): 
                wp = float(self.parent().width())#-30)
                hp =  float(self.parent().height())#-30)
                #if hasattr(self,'prevwp'): 
                    #if isinstance(self.parent(),QtGui.QScrollArea):
                        #self.info('Objects inside scroll areas are not resized after creation...')
                        #return
                    #print '%s -> (%d,%d); ' % (self.prevwp,wp,hp),
                    #if (self.prevwp[0]<wp or self.prevwp[1]<hp): print 'Size increased'
                    #else: print 'Size reduced'
                ws = float(self.w_scene)
                hs = float(self.h_scene)
            else:
                if not hasattr(self,'prevwp'): 
                    #Size is calcullated to parent ... if there's no parent or no previous size there's no resize to calcullate!
                    return
                ws,hs = self.prevwp
                wp,hp = float(self.w_scene),float(self.h_scene)
            w =wp/ws
            h = hp/hs
            self.scale(w,h)
            self.w_scene = wp
            self.h_scene = hp
            self.prevwp = (ws,hs)
            self.emitColors()
        except Exception,e:
            self.warning('Exception in JDrawView.resizeEvent: %s' % str(e))
            pass

#    def hideEvent(self,event):
#        print "--------------hideEvent()----------------------"
#        #print self
#        #print self.children()
#        #print "dialog = ", self.parent() 
#        for child in self.parent().children():
#            #print "child = ",child
#            try:
#                child.close()
#            except:
#                pass
#            try:
#                import os,signal
#                #print "table = ",child.pids
#                for pid in child.pids:
#                    #print pid
#                    os.kill(pid,signal.SIGTERM)
#            except:
#                pass
#        print "closing ..."
#
#        print ""
#        print ""
#        print ""

    def refreshModel(self):
        self.setModel(self.modelName)

    def updateStyle(self):
        self.repaint()

    def getGraphicsFactory(self):
        return jdraw.JDrawTauGraphicsFactory(self.parent())

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties 
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    @QtCore.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.modelName = str(model)
        self._currF = str(model)
        if self._currF:
            #filename = str(self._currFile.absoluteFilePath())
            filename = self._currF
            filename = os.path.realpath(filename)
            if os.path.isfile(filename):            
                self.debug("Starting to parse %s" % filename)
                factory = self.getGraphicsFactory()
                scene = jdraw_parser.parse(filename, factory)
                if not scene:
                    self.warning("TauJDrawSynopticsView.setModel(%s): Unable to parse %s!!!"%(model,filename))
                elif self.w_scene is None and scene.sceneRect():
                    self.w_scene = scene.sceneRect().width()
                    self.h_scene = scene.sceneRect().height()
                else: self.debug('JDrawView.sceneRect() is NONE!!!')
                self.setScene(scene)
                QtCore.QObject.connect(self.scene(), QtCore.SIGNAL("graphicItemSelected(const QString &)"), self, QtCore.SLOT("graphicItemSelected(const QString &)"))
                #self.emit_signal()
            else:
                self.setScene(None)
                #self.emit_signal()
            #The emitted signal contains the filename and a dictionary with the name of items and its color
            self.emitColors()#get_item_colors(emit=True)

    def getModel(self):
        return self._currF

    model = QtCore.pyqtProperty("QString", getModel, setModel)
    
    
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
if __name__ == "__main__":
    import sys
    tau.setLogLevel(tau.Info)
    app = Qt.QApplication(sys.argv)
    
    #form = QtGui.QDialog()
    #ly=QtGui.QVBoxLayout(form)
    #container=QtGui.QWidget()
    #ly.addWidget(container)   
    #for m in sys.argv[1:]:
        #tv=TauJDrawSynopticsView(container, designMode=False)
        #tv.setModel(m)
    form = tau.widget.TauJDrawSynopticsView(designMode=False)
    form.setModel(sys.argv[1])
    form.show()
    
    sys.exit(app.exec_())
