import tau.core
from tau.core.tango.macroserver import BaseMacroServer, BaseDoor

from PyQt4 import Qt

CHANGE_EVTS = (tau.core.TauEventType.Change, tau.core.TauEventType.Periodic)

class QDoor(Qt.QObject, BaseDoor):
    
    __pyqtSignals__ = ["resultUpdated","recordDataUpdated", "macroStatusUpdated"]
    __pyqtSignals__ += [ "%sUpdated" % l.lower() for l in BaseDoor.log_streams ]
    
    def __init__(self, name, qt_parent=None, **kw):
        self.call__init__wo_kw(Qt.QObject, qt_parent)
        self.call__init__(BaseDoor, name, **kw)
    
    def resultReceived(self, log_name, result):
        res = BaseDoor.resultReceived(self, log_name, result)
        self.emit(Qt.SIGNAL("resultUpdated"))
        return res
    
    def recordDataReceived(self, s, t, v):
        if t not in CHANGE_EVTS: return
        res = BaseDoor.recordDataReceived(self, s, t, v)
        self.emit(Qt.SIGNAL("recordDataUpdated"))
        return res
        
    def macroStatusReceived(self, s, t, v):
        if t not in CHANGE_EVTS: return
        res = BaseDoor.macroStatusReceived(self, s, t, v)
        macro = self.getRunningMacro()
        self.emit(Qt.SIGNAL("macroStatusUpdated"), macro)
        return res
    
    def logReceived(self, log_name, output):
        res = BaseDoor.logReceived(self, log_name, output)
        self.emit(Qt.SIGNAL("%sUpdated" % log_name.lower()), output)
        return res


class QMacroServer(Qt.QObject, BaseMacroServer):
    
    def __init__(self, name, qt_parent=None, **kw):
        self.call__init__wo_kw(Qt.QObject, qt_parent)
        self.call__init__(BaseMacroServer, name, **kw)
        
    def typesChanged(self, s, t, v):
        res = BaseMacroServer.typesChanged(self, s, t, v)
        self.emit(Qt.SIGNAL("typesUpdated"))
        return res
    
    def elementsChanged(self, s, t, v):
        res = BaseMacroServer.elementsChanged(self, s, t, v)
        self.emit(Qt.SIGNAL("elementsUpdated"))
        return res
    
    def macrosChanged(self, s, t, v):
        if t not in CHANGE_EVTS: return
        res = BaseMacroServer.macrosChanged(self, s, t, v)
        self.emit(Qt.SIGNAL("macrosUpdated"))
        return res
    
