
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
taupanel.py: 
"""

from PyQt4 import Qt
import tau, PyTango
#tau.setLogLevel(tau.Info)
from tau.widget import TauMainWindow
from tau.widget.ui.ui_TauDevPanel import Ui_TauDevPanel
    
    
class TauDevPanel( TauMainWindow, Ui_TauDevPanel):
    '''
    TauDevPanel is a Tau Application inspired in Jive and Atk Panel.
    
    It Provides a Device selector and several dockWidgets for interacting and
    displaying information from the selected device.
    '''
    def __init__(self, parent=None):
        TauMainWindow.__init__(self, parent)
        self.setupUi(self)
        
        self.loadSettings()
        self.createActions()
        
        #self.addToolBar(self.basicTauToolbar())

    def createActions(self):
        '''create actions '''
        #View Menu
        self.showAttrAction = self.viewMenu.addAction(self.attrDW.toggleViewAction())
        self.showCommandsAction = self.viewMenu.addAction(self.commandsDW.toggleViewAction())
        self.showTrendAction = self.viewMenu.addAction(self.trendDW.toggleViewAction())
        
    def loadSettings(self):
        '''extended from :class:TauMainWindow'''
        TauMainWindow.loadSettings(self)
        settings = self.getQSettings()
        self.tauCommandsForm._splitter.restoreState(settings.value("CommandsPanel/SplitterState").toByteArray())

    def saveSettings(self):
        '''extended from :class:TauMainWindow'''
        TauMainWindow.saveSettings(self)
        settings = self.getQSettings()
        settings.setValue("CommandsPanel/SplitterState", Qt.QVariant(self.tauCommandsForm._splitter.saveState()))
    
        
       

def main():
    import sys
    if len(sys.argv)>1: model=sys.argv[1]
    else: model = None
    
    app = Qt.QApplication(sys.argv)
    app.setApplicationName('TauPanel')
    app.setOrganizationName('ALBA')
    
    dialog = TauDevPanel()
    if model is not None:
        dialog.setModel(model)
    dialog.show()
    
    sys.exit(app.exec_()) 
    
if __name__ == "__main__":
    main() 