
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
valuestable.py: 
"""

#!/usr/bin/env python

from PyQt4 import Qt
    
import tau, PyTango
import numpy

from tau.widget import TauBaseWidget
from tau.core.utils import ATTRIBUTE_QUALITY_PALETTE

class TauValuesTable(Qt.QTableWidget, TauBaseWidget):
    '''This widget gets spectra or images as model and shows a table containing
    the values in the model'''
    
    __pyqtSignals__ = ("modelChanged(const QString &)",)
    
    def __init__(self, parent = None, designMode = False):
        name = self.__class__.__name__
        self.call__init__wo_kw(Qt.QTableWidget, parent)
        self.call__init__(TauBaseWidget, name, designMode=designMode)
        #@TODO add event filter once the backwards compatibility is not needed
        self.setEditTriggers(self.NoEditTriggers) #the table is non-editable by default
        self.setShowQuality(False) #by default it does not show the quality on background
        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TauBaseWidget overwriting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    
    def getModelClass(self):
        import tau.core
        return tau.core.TauAttribute

    def isReadOnly(self):
        return True
        
    def handleEvent(self, src, evt_type, val):
        if evt_type in [tau.core.TauEventType.Change, tau.core.TauEventType.Periodic] and val is not None:            
            values=numpy.array(val.value)
            #reshape the table
            if val.data_format == PyTango.AttrDataFormat.SPECTRUM:
                rows, columns = values.size, 1
            elif val.data_format == PyTango.AttrDataFormat.IMAGE:
                rows, columns = values.shape
            else:
                self.warning('unsupported data format %s'%str(val.data_format))
                return
            self.setRowCount(rows)
            self.setColumnCount(columns)
            
            #fill the table
            values = values.reshape(rows,columns) #make sure it is in matrix form (not a vector)
            for j in xrange(columns):
                for i in xrange(rows):
                    self.setItem(i,j,Qt.QTableWidgetItem(str(values[i,j])) ) #extremely innefficient!! #@TODO
            self.updateStyle()
            
    def eventHandle(self, *args):
        '''Only here for backwards compatibility. It will disapear soon. Not supported'''
        self.info('using deprecated event handling methods.')
        model = self.getModelObj()
        val = model.getValueObj()
        self.handleEvent(model, tau.core.TauEventType.Change, val)
        
    def updateStyle(self):
        if self.getShowQuality():
            val = self.getModelValueObj()
            quality = getattr(val, 'quality', None)
            stylesheet = "TauValuesTable {%s}"%ATTRIBUTE_QUALITY_PALETTE.qtStyleSheet(quality)
        else:
            stylesheet = "TauValuesTable {}"
        self.setStyleSheet(stylesheet)

        
        
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT property definition
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    model = Qt.pyqtProperty("QString", TauBaseWidget.getModel, 
                                TauBaseWidget.setModel, 
                                TauBaseWidget.resetModel)
    
    useParentModel = Qt.pyqtProperty("bool", 
                                         TauBaseWidget.getUseParentModel, 
                                         TauBaseWidget.setUseParentModel,
                                         TauBaseWidget.resetUseParentModel)
    
    showQuality = Qt.pyqtProperty("bool", 
                                      TauBaseWidget.getShowQuality,
                                      TauBaseWidget.setShowQuality,
                                      TauBaseWidget.resetShowQuality)


if __name__ == '__main__':
    import sys
    app = Qt.QApplication([])
    
    if len(sys.argv) == 2:
        model = sys.argv[1]
    else:
        model = 'bl97/pysignalsimulator/1/value1'
     
    w = TauValuesTable()
        
    w.setModel(model)
    w.setShowQuality(True)
    w.show()
    sys.exit(app.exec_())