
#!/usr/bin/env python

#############################################################################
##
## This file is part of Tau, a Tango User Interface Library
##
## http://www.tango-controls.org/static/tau/latest/doc/html/index.html
##
## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## This is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This software is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
###########################################################################

"""
widgetfactory.py: 
"""

import os.path
import tau.core.utils
import tau.widget

class TauWidgetFactory(tau.core.utils.Singleton, tau.core.utils.Logger):
    
    skip_modules = ('utils', 'designer')

    def __init__(self):
        """ Initialization. Nothing to be done here for now."""
        pass

    def init(self, *args):
        """Singleton instance initialization."""
        name = self.__class__.__name__
        self.call__init__(tau.core.utils.Logger, name)
        
        self._widgets = self._buildTauWidgets('tau.widget', __file__)

    def _buildTauWidgets(self, module_name, path, recursive = True):
        path = os.path.abspath(path)
        p = os.path.dirname(path)
        elems = os.listdir(p)
        ret = {}
        if not '__init__.py' in elems:
            return ret
        
        try:
            m = __import__(module_name, fromlist=['*'], level=0)
            dir_names = dir(m)
            for dir_name in dir_names:
                try:
                    attr = getattr(m, dir_name)
                    if issubclass(attr, tau.widget.TauBaseWidget):
                        ret[dir_name] = attr
                except Exception, e:
                    pass
        except:
            return ret
            
        if not recursive:
            return ret
        
        for elem in elems:
            abs_elem = os.path.join(p, elem)
            if (not elem.startswith('.')) and os.path.isdir(abs_elem) and \
               (not elem in self.skip_modules):
                m_name = os.path.splitext(elem)[0]
                new_module_name = '%s.%s' % (module_name, m_name)
                ret.update(self._buildTauWidgets(new_module_name, abs_elem, True))
        return ret
    
    def getTauWidgetClassNames(self):
        return self._widgets.keys()

    def getTauWidgetClasses(self):
        return self._widgets.values()

    def getTauWidgetClass(self, name):
        return self._widgets.get(name)
